import os
import Mutation_Testing.Mutant_Generation.Config as Config
import Mutation_Testing.Mutant_Generation.generateMutants as generateMutants 


class Mutate_Subject_Program: 


    # Input: Subject_Program_Name, the name of the subject program, as expressed in the Subject_Programs directory
    # Process: Gets all of the python files in the subject program, and then mutates them. 
    def mutate_subject_Program(self, Subject_Program_Name):
        Files_to_Mutate = self._get_All_File_Paths(Subject_Program_Name)
        self._mutate_Files(Subject_Program_Name, Files_to_Mutate)
        
    
    
    # Input: - Subject_Program_Name, the name of the directory in the Subject_Program's package, that pertains to the
    #          subject program of interest.
    #        - File_Names_to_Ignore, the names of files that should be ignored 
    #        - Directory_Names_to_Ignore, the names of directories that should be ignored
    # Process: Uses _recursive_File_Search to get all of the python files within the subject program
    # Output: A list of all of the python files that constitute the subject program.
    def _get_All_File_Paths(self,Subject_Program_Name, File_Names_to_Ignore = ["__init__.py"], Directory_Names_to_Ignore = ["__pycache__"]):
        
        # The path to the subject program's root directory
        Mutmut_Package = __file__[:__file__.rfind("\\")]
        Mutation_Testing_Package = Mutmut_Package[:Mutmut_Package.rfind("\\")]
        Root_Directory = Mutation_Testing_Package[:Mutation_Testing_Package.rfind("\\")]
        Subject_Programs_Package = Root_Directory + "\\Subject_Programs"
        Subject_Program_Root = Subject_Programs_Package + "\\" + Subject_Program_Name + "\\Original_Program"
                
        Subject_Program_Files = self._recursive_File_Search(Subject_Program_Root, File_Names_to_Ignore, Directory_Names_to_Ignore)
        
        return Subject_Program_Files
        
    # Input: - Root_Directory, a path to the root directory to be searched
    #        - File_Names_to_Ignore, a list of file names that should be ignored by the recursive file search
    #        - Directory_Names_to_Ignore, a list of directories taht should be ignored by the recursive file search
    # Process: Recursively searches the Root_Directory, ignoring any files and folders that are not of interest to the user, 
    #          to find all of the python files in the folder structure. These files are returned as a list.
    # Output: A list of all python files in Root_Folder, and all of its directories, and their directories and so on (apart from files and 
    #         directories that are not of interest to the user).
    def _recursive_File_Search(self,Root_Directory, File_Names_to_Ignore, Directory_Names_to_Ignore):
        
        Root_Directory_Contents = os.listdir(Root_Directory)
                
        Files = []        
        
        # For each file or folder in the root directory
        for File_or_Folder in Root_Directory_Contents:
            Path = Root_Directory+"\\"+File_or_Folder
            if os.path.isdir(Path): # if it is a directory
                if File_or_Folder not in Directory_Names_to_Ignore:
                    Files = Files + self._recursive_File_Search(Path, File_Names_to_Ignore, Directory_Names_to_Ignore)
            else: # if it is a file
                if File_or_Folder not in File_Names_to_Ignore:
                    File_Extension = File_or_Folder[-3:]
                    if File_Extension == ".py":
                        Files.append(Path)
        return Files
            
            
            
    # Input: - Subject_Program_Name, the name of the subject program, as expressed by the directory in Subject_Programs
    #        - Files_to_Mutate, a list of files that must be mutated
    # Process: For each file in Files_to_Mutate, this method creates a corresponding folder in Subject_Programs/Subject_Program_Name/Mutants,
    #          generates mutants for the file, and then stores all of the mutants in this newly generated folder.
    def _mutate_Files(self,Subject_Program_Name,Files_to_Mutate):
        
        # The path to the subject program's container and Mutants package
        Mutmut_Package = __file__[:__file__.rfind("\\")]
        Mutation_Testing_Package = Mutmut_Package[:Mutmut_Package.rfind("\\")]
        Root_Directory = Mutation_Testing_Package[:Mutation_Testing_Package.rfind("\\")]
        Subject_Programs_Package = Root_Directory + "\\Subject_Programs"
        Subject_Program_Container = Subject_Programs_Package + "\\" + Subject_Program_Name
        Mutants_Package = Subject_Program_Container + "\\Mutants"  

        # For each file to mutate
        for File_to_Mutate in Files_to_Mutate:
            
            # Create a directory for this file in the Mutants folder
            Corresponding_Directory = Mutants_Package + File_to_Mutate[len(Subject_Program_Container)+17:][:-3]
            if os.path.exists(Corresponding_Directory) == False:
                try:
                    os.makedirs(Corresponding_Directory)
                    print("Created " + Corresponding_Directory)
                except:
                    raise Exception("Could not create " + Corresponding_Directory)
            
            # Mutate the file and add the mutants to the newly created folder
            Config.classToMutate = File_to_Mutate
            Config.mutantStorageLocation = Corresponding_Directory
            generateMutants.generateAllMutants()
            print()
            
            
        


# Instructions: to run this code, all you have to do is create a Mutate_Subject_Program() object, and then run its mutate_subject_program method.

# Mutate = Mutate_Subject_Program()
# Mutate.mutate_subject_Program("Test_Toy")




