import Mutation_Testing.Mutant_Generation.Config as Config
from subprocess import Popen, PIPE
import shutil
import os
import sys
    
# The user should call this method to generate mutants. It first runs some preliminary checks, and then generates mutants according to the parameters. No other method should be directly called by the user.
def generateAllMutants():
    preliminaries()
    getMutants(generateMutants())



# Process: This method does some preliminary checks, and helps with the set-up before mutation testing occurs.
def preliminaries():
    try:
        if os.path.exists(Config.mutmut) == False:
            raise Exception("Could not locate the mutmut folder. Please ensure that mutmut is installed by using \"pip install mutmut\", and that Config.mutmut is equal to the path to the mutmut folder.")
        if "mutation" in Config.mutmut:
            raise Exception("The path to mutmut cannot contain the word 'mutation' Please change the file path/move mutmut elsewhere.")
    except Exception:
        raise Exception("Issue with the mutmut folder..")

    try: 
        if os.path.exists(Config.cmd) == False:
            raise Exception("Could not locate cmd.exe. Please ensure that Config.cmd is equal to the path to cmd.exe.")
        if (Config.cmd[-7:]) != "cmd.exe":
            raise Exception("Could not locate cmd.exe. Please ensure that Config.cmd is equal to the path to cmd.exe.")
    except Exception:
        raise Exception("Could not locate cmd.exe. Please ensure that Config.cmd is equal to the path to cmd.exe.")

    try:
        if os.path.exists(Config.classToMutate) == False:
            raise Exception("Could not locate the class to be mutated. Please specify this path in Config.classToMutate.")
        if Config.classToMutate[-3:] != ".py":
            raise Exception("The file to mutate must be a python file. Please specify the path to a python file in Config.classToMutate.")
        if " " in Config.classToMutate:
            raise Exception("The file path cannot contain white space. Please ammend this, and update Config.classToMutate.")
    except Exception:
        raise Exception("Could not locate the class to be mutated. Please specify this path in Config.classToMutate.")
    
    try:
        if os.path.exists(Config.GuranteedToPassTestSuite) == False:
            raise Exception("Could not locate the test suite. Please specify this path in Config.GuranteedToPassTestSuite.")
        
        if len(os.listdir(Config.GuranteedToPassTestSuite)) == 0:
            TestCase = open(Config.GuranteedToPassTestSuite+"\\test_test.py","w+")
            TestCase.write(
                "from unittest import TestCase\n"+
                "class Test(TestCase):\n"+
                "\tdef test_test(self):\n"+
                "\t\tself.assertEqual(2, 2)"
                )
            TestCase.close()
        if " " in Config.GuranteedToPassTestSuite:
            raise Exception("The file path cannot contain white space. Please ammend this, and update Config.GuranteedToPassTestSuite.")
    except Exception:
        raise Exception("Could not locate the test suite. Please specify this path in Config.GuranteedToPassTestSuite.")
     
     
    try:
        if os.path.exists(Config.mutantStorageLocation) == False:
            raise Exception("Could not locate the mutant storage location. Please specify this path in Config.mutantStorageLocation.")
        if len(os.listdir(Config.mutantStorageLocation)) != 0:
            raise Exception(Config.mutantStorageLocation+" must be empty.")
    except Exception:
        raise Exception("Could not locate the mutant storage location. Please specify this path in Config.mutantStorageLocation.")
    
    print("mutmut stored at: "+Config.mutmut)
    print("class to mutate stored at: "+Config.classToMutate)
    print("Test suite directory stored at: "+Config.GuranteedToPassTestSuite)
    print("cmd.exe stored at: "+Config.cmd)
    print("Mutant storage location stored at: "+Config.mutantStorageLocation)
    UserDecision = input("If the above is correct, enter y, otherwise press any key to exit.")
    if UserDecision != "y":
        sys.exit()















# Process: This method first generates a config file for mutmut to read, and stores it in the mutmut directory, where mutmut will be expecting it.
#          This method then executes mutmut, which generates the mutants and a set of IDs.
# Output: Finally, the method acquires the largest mutant ID, and returns this as output.
def generateMutants():
    try:
        mutmutConfigurationFile = open(Config.mutmut+"\\setup.cfg","w+")
        mutmutConfigurationFile.write("[mutmut]\npaths_to_mutate="+Config.classToMutate+"\nbackup=False\nrunner=python -m pytest\ntests_dir="+Config.GuranteedToPassTestSuite+"\ndict_synonyms=Struct, NamedStruct")
        mutmutConfigurationFile.close()        
    except Exception:
        raise Exception("Failed to generate mutmuts configuration file.") 
    process = Popen(Config.cmd,encoding="utf8",stdin=PIPE,stdout=PIPE,cwd=Config.mutmut)
    cmd = str(process.communicate("python __main__.py\n"))
    process.kill()    
            
    Prefix = cmd[cmd.rfind("mutation")+9:]    
    LargestMutantID = int(Prefix[:Prefix.find("-")-1])
    
    return LargestMutantID



# Input: LargestMutantID, which is the largest mutant ID that mutmut generated
# Process: Lets suppose that the largest mutant id is 4, this means that mutmut has generated mutant 1, mutant 2, and mutant 3. This method iterates over these mutants
#          in the reverse order. On each iteration it requests mutmut to modify the file stored at Config.classToMutate based on the mutant being considered by that iteration of the loop.
#          After this, the mutant is moved to Config.mutantStorageLocation and renamed to match the mutant id. The file that was mutated (pre-mutation) is stored as a .bak file;
#          Finally this is converted back into a python file.
# Output: Config.mutantStorageLocation has all of the mutants.
def getMutants(LargestMutantID):    
    MutantIDs = list(range(LargestMutantID+1))
    del MutantIDs[0] 
    MutantIDs.reverse()
    for MutantID in MutantIDs:
        process = Popen(Config.cmd,encoding="utf8",stdin=PIPE,stdout=PIPE,cwd=Config.mutmut)
        process.communicate("python __main__.py "+Config.classToMutate+" --mutation "+str(MutantID)+" --apply\n")
        process.kill()     
        shutil.move(Config.classToMutate, Config.mutantStorageLocation+"\\Mutant_"+str(MutantID)+".py")
        shutil.move(Config.classToMutate+".bak", Config.classToMutate)

























