import os
import shutil




class Mutant:
    
    # Input: - Original_File_Path, the file path of the original file 
    #        - Mutant_File_Path, the file path of the mutant file (can be a mutant file from either Mutants, Instrumented_Mutants, or Instrumented_Originals)
    #        - Subject_Program_Name, the name of the subject program, as expressed in the Subject_Programs package
    # Process: - Checks that the files exist and then sets the two file paths as global variables. Computes a path to the subject program
    #            root folder and stores this as a global variable.  
    def __init__(self, Original_File_Path, Mutant_File_Path, Subject_Program_Name):
        
        if os.path.exists(Original_File_Path) == False:
            raise Exception(Original_File_Path + " does not exist.")
            
        if os.path.exists(Mutant_File_Path) == False:
            raise Exception(Mutant_File_Path + " does not exist.")

        self.Original_File_Path = Original_File_Path
        self.Mutant_File_Path = Mutant_File_Path
    
        SMT_and_WMT_Package = __file__[:__file__.rfind("\\")]
        Mutation_Test_Package = SMT_and_WMT_Package[:SMT_and_WMT_Package.rfind("\\")]
        Project_Root_Directory = Mutation_Test_Package[:Mutation_Test_Package.rfind("\\")]
        Subject_Programs_Package = Project_Root_Directory + "\\Subject_Programs"
        Subject_Program_Container = Subject_Programs_Package + "\\" + Subject_Program_Name  
        Subject_Program_Root_Directory = Subject_Program_Container + "\\Original_Program" 
        
        self.Subject_Program_Root_Directory = Subject_Program_Root_Directory
    
    # Process: Renames the file stored at Original_File_Path to include a _backup.py suffix 
    def _backup_Original_File(self):
        
        # We want to rename the original file to have the suffix "_backup". This code computes the new file name
        Original_File_Path = self.Original_File_Path
        Original_File_Path_Backup = Original_File_Path[:Original_File_Path.rfind(".")]+"_backup.py"
        
        if os.path.exists(Original_File_Path) == False:
            raise Exception(Original_File_Path + " does not exist.")
        
        if os.path.exists(Original_File_Path_Backup):
            raise Exception(Original_File_Path_Backup + " already exists.")
        
        # This block of code infinitely tries to rename file and keeps trying until it is successful
        while (True):       
            try:            
#                 print(Original_File_Path + " renamed to " + Original_File_Path_Backup)
                os.rename(Original_File_Path, Original_File_Path_Backup)            
                break
            except PermissionError:
                pass

    # Process: copies the mutant to the location of the original file, and renames it to the original files name.
    def _loadMutant(self):
        
        # We want to copy the mutant file to the original files location, and give it the same name as the original file
        Mutant_File_Path = self.Mutant_File_Path
        Original_File_Path = self.Original_File_Path
        
        if os.path.exists(Mutant_File_Path) == False:
            raise Exception(Mutant_File_Path + " does not exist.")

        if os.path.exists(Original_File_Path):
            raise Exception(Original_File_Path + " already exists.")
        
        # Infinitely try to copy the file. Break when successful.
        while (True):       
            try:            
#                 print(Mutant_File_Path + " copied to " + Original_File_Path)
                shutil.copy(Mutant_File_Path, Original_File_Path)
                break
            except PermissionError:
                pass
    
    
    # Deletes all of the pycaches in the subject program
    def _magicBullet(self):
        
        # For each pycache in the subject program
        for Pycache in self._getAllPycaches(self.Subject_Program_Root_Directory):
            
            # Guarantee that the path ends in __pycache__
            if Pycache[Pycache.rfind("\\")+1:] != "__pycache__":
                raise Exception("File is not __pycache__")
             
            # Delete the pycache
            while (True):       
                try:  
#                     print("deleted " + Pycache)                  
                    shutil.rmtree(Pycache)
                    break
                except PermissionError:
                    pass

    # Input: Directory, a file path 
    # Process: Recursively searches Directory to find all of the __pycache__ folders.
    # Output: A list of __pycache__ folders.
    def _getAllPycaches(self, Directory):
                
        Files_or_Folders = os.listdir(Directory)
        
        List = []
        
        # For each Folder in Directory
        for Folder in Files_or_Folders:
            Path = Directory + "\\" + Folder 
            if os.path.isdir(Path):
                # If the Folder is __pycache__, store the path
                if Folder == "__pycache__":
                    List.append(Path)
                    pass
                else: 
                    # Otherwise, search the Folder for __pycache__
                    List = List + self._getAllPycaches(Path)
                    pass
        
        return List
        
    # Process: deletes the mutant that has been loaded in 
    def _deloadMutant(self):
        
        # Current paths to the mutant and original files
        Mutant = self.Original_File_Path
        Original = Mutant[:Mutant.rfind(".")] + "_backup.py"
        
        # Ensure that the mutant and original exist, before trying to delete the mutant
        if os.path.exists(Mutant) and os.path.exists(Original):
            while (True):       
                try:    
#                     print("Removed " + Mutant)
                    os.remove(Mutant)
                    break
                except PermissionError:
                    pass
        else:
            raise Exception("There is no backup file available, so I am refusing to delete the file")
        
    
    # Process: renames the original file backup to what it initially was.
    def _restore_Original_File(self):
        
        Original_File_Path = self.Original_File_Path
        Original_File_Path_Backup = Original_File_Path[:Original_File_Path.rfind(".")]+"_backup.py"
        
        if os.path.exists(Original_File_Path_Backup) == False:
            raise Exception(Original_File_Path_Backup + " does not exist")
        
        if os.path.exists(Original_File_Path):
            raise Exception(Original_File_Path + " already exists")
        
        # This block of code infinitely tries to rename file and keeps trying until it is successful
        while (True):       
            try:     
#                 print(Original_File_Path_Backup + " restored to " + Original_File_Path)       
                os.rename(Original_File_Path_Backup, Original_File_Path)            
                break
            except PermissionError:
                pass
        
    # Inject the mutant into the Subject Program
    def inject_Mutant(self):
        self._backup_Original_File() # Create a backup of the original file
        self._loadMutant() # Load the mutant
        self._magicBullet() # Delete the binary files



    # Restores the subject program
    def remove_Mutant(self):
        self._deloadMutant() # Remove the mutant
        self._restore_Original_File() # Reinstate the original
        self._magicBullet() # Delete the binary files.
        
        




    # up to here
        
        



# test code
# Original_File_Path = "C:\\Users\\Krishna\\eclipse-workspace\\NSQ_SQ_AvgFEP\\Subject_Programs\\Test_Toy\\Original_Program\\Toy_Subject_Program.py"
# Mutant_File_Path = "C:\\Users\\Krishna\\eclipse-workspace\\NSQ_SQ_AvgFEP\\Subject_Programs\\Test_Toy\\Mutants\\Toy_Subject_Program\\Mutant_1.py"
# Test = Mutant(Original_File_Path, Mutant_File_Path, "Test_Toy")
#Test.inject_Mutant()
#Test.remove_Mutant()


#Test._restore_Original_File()

#print(Test.Subject_Program_Root_Directory)

# for i in Test._getAllPycaches(Test.Subject_Program_Root_Directory):
#     print(i)

#Test._backup_Original_File()
# Test._loadMutant()
#Test._magicBullet()
#Test._deloadMutant()

# test code

