import Mutation_Testing.Mutant_Loader.LoadMutants as LoadMutants
import os
import sys
import TestSuiteExecutor.Executor as Executor

class MutantExecutor():
    
    # Input: - Subject_Program_Name, the subject program's name
    #        - Subject_Program_Main_Class, path to the main class, in dot notation
    def __init__(self, Subject_Program_Name, Subject_Program_Main_Class):
        
        # Creates a global variable that pertains to a list of all mutants
        self.AllMutants = []
        self._getAllMutants(Subject_Program_Name)
        
        self.Subject_Program_Main_Class = Subject_Program_Main_Class
        self.Subject_Program_Name = Subject_Program_Name        
        
    # Input: Subject_Program_Name, the name of the subject program
    # Process: Computes a path to the mutant root directory, and then calls _recursiveSearch to recursively search the directory for mutants
    def _getAllMutants(self,Subject_Program_Name):
    
        SMT_and_WMT_Directory = (__file__[:__file__.rfind("\\")])
        Mutation_Testing = (SMT_and_WMT_Directory[:SMT_and_WMT_Directory.rfind("\\")])
        Project_Root_Directory = (Mutation_Testing[:Mutation_Testing.rfind("\\")])
        Subject_Programs_Directory = Project_Root_Directory + "\\Subject_Programs"
        Subject_Program_Directory = Subject_Programs_Directory+"\\"+Subject_Program_Name
        Mutants_Root_Directory = Subject_Program_Directory + "\\Mutants"
        
        self._recursiveSearch(Mutants_Root_Directory)
    
    # Input: A directory
    # Process: recursively searches the directory and produces a list of files
    def _recursiveSearch(self,Directory):
        
        Directory_Contents = os.listdir(Directory)
        
        for Directory_Item in Directory_Contents:
            Directory_Item_Path = Directory + "\\" + Directory_Item
            
            if os.path.isdir(Directory_Item_Path):
                self._recursiveSearch(Directory_Item_Path)
            else: 
                self.AllMutants.append(Directory_Item_Path)


    # This will perform the actions implemented by processMutant for each mutant
    def processMutants(self,TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, SaveResults = False, ProcessOriginal=True, ProcessMutants=True, PerformAdditionalActions=False):

        ProcessedMutants = {}

        # Process the original
        if ProcessOriginal:
            ProcessedOriginal = self._processMutant(TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, PerformAdditionalActions)
            ProcessedMutants["Original_Program"] = ProcessedOriginal  
                
        if ProcessMutants:
            SMT_and_WMT_Directory = (__file__[:__file__.rfind("\\")])
            Mutation_Testing = (SMT_and_WMT_Directory[:SMT_and_WMT_Directory.rfind("\\")])
            Project_Root_Directory = (Mutation_Testing[:Mutation_Testing.rfind("\\")])
            Subject_Programs_Directory = Project_Root_Directory + "\\Subject_Programs"
            Subject_Program_Directory = Subject_Programs_Directory+"\\"+self.Subject_Program_Name
            
            for Mutant in self.AllMutants:
                # Path to original file
                Mutants_Subpath = Mutant[len(Subject_Program_Directory)+1:]        
                Mutants_Subpath = Mutants_Subpath[Mutants_Subpath.find("\\")+1:]
                Original_Path = Subject_Program_Directory + "\\Original_Program\\"+Mutants_Subpath
                Original_Path = Original_Path[:Original_Path.rfind("\\")]+".py"            
                
                # Inject mutant and perform necessary actions for mutant
                test = LoadMutants.Mutant(Original_Path, Mutant, self.Subject_Program_Name)
                test.inject_Mutant()
                ProcessedMutant = self._processMutant(TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, PerformAdditionalActions)
                test.remove_Mutant()
     
                # Mutant name
                Mutant_Name = Mutant[len(Subject_Programs_Directory)+1:-3].replace("\\",".")
    
                # Store results 
                ProcessedMutants[Mutant_Name] = ProcessedMutant        
                
        
        if SaveResults:
            self._save_Results(ProcessedMutants,TestSuiteGeneratorName, TestSuiteNumber, TimeOut)
    
    # Inputs for the Executor
    def _processMutant(self,TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, PerformAdditionalActions):
        
        # Execute the subject program with a test suite
        ExecutedTestSuite = self._ExecuteTestSuite(TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, PerformAdditionalActions)
        
        return ExecutedTestSuite 
        
    # Inputs for Executor
    def _ExecuteTestSuite(self,TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, PerformAdditionalActions):
        return Executor.Executor().execute_Subject_Program(TestSuiteGeneratorName, TestSuiteNumber, self.Subject_Program_Main_Class, TimeOut, Method_Name_Replacement, PerformAdditionalActions)
        
        
    def _save_Results(self,ProcessedMutants,TestSuiteGeneratorName, TestSuiteNumber, TimeOut):
        
        # Prepares the folder to store the the results in
        Mutant_Loader_Directory = __file__[:__file__.rfind("\\")]
        Mutation_Testing_Directory = Mutant_Loader_Directory[:Mutant_Loader_Directory.rfind("\\")]
        Project_Root = Mutation_Testing_Directory[:Mutation_Testing_Directory.rfind("\\")]
        Experimental_Results_Directory = Project_Root + "\\Experimental_Results"
        
        if os.path.exists(Experimental_Results_Directory) == False:
            try:
                os.mkdir(Experimental_Results_Directory)
            except:
                raise Exception("Couldn't create " + Experimental_Results_Directory)

        ExperimentNumber = "Experiment_"+str(len(os.listdir(Experimental_Results_Directory))+1)
        ResultsFolderName = ExperimentNumber + "(" + "Prog=" + self.Subject_Program_Name + "," + "TCG=" + TestSuiteGeneratorName + "," + "TestSuite=" + str(TestSuiteNumber) + "," + "TimeOut=" + str(TimeOut) + ")"
        ResultsFolderPath = Experimental_Results_Directory+"\\"+ResultsFolderName
        
        if os.path.exists(ResultsFolderPath):
            raise Exception(ResultsFolderPath +" already exists.")
        else:
            try:
                os.mkdir(ResultsFolderPath)
            except:
                raise Exception("Couldn't create " + ResultsFolderPath)
        
        # Stores the results and crash report
        CrashedMutants = []
        for Mutant in list(ProcessedMutants.keys()):
            if(ProcessedMutants[Mutant]["Crashes"]):
                CrashedMutants.append(Mutant)
            else:
                MutantFolderPath = ResultsFolderPath+"\\"+Mutant
                
                if os.path.exists(MutantFolderPath):
                    raise Exception(MutantFolderPath + " already exists.")
                else:
                    try:
                        os.mkdir(MutantFolderPath)
                    except:
                        raise Exception("Couldn't create " + MutantFolderPath)

                            
                TestSuite = ProcessedMutants[Mutant]["Results"]
                for TestCase in list(TestSuite.keys()):
                    TestCasePath = MutantFolderPath + "\\" + TestCase 
                
                    if os.path.exists(TestCasePath):
                        raise Exception(TestCasePath + " already exists.")
                    else:
                        try:
                            os.mkdir(TestCasePath)
                        except:
                            raise Exception("Couldn't create " + TestCasePath)
                
                    OutputPath = TestCasePath + "\\Output.txt"
                    
                    if os.path.exists(OutputPath):
                        raise Exception(OutputPath + " already exists.")
                    
                    Output = TestSuite[TestCase]
                    
                    OutputFile = None
                    try:
                        OutputFile = open(OutputPath, "w+")
                        OutputFile.write(Output)
                    except:
                        raise Exception("Couldn't store output")
                    finally:
                        if OutputFile != None:
                            OutputFile.close()
        
        CrashReportPath = ResultsFolderPath + "\\Crashed_Mutants.txt"
        if os.path.exists(CrashReportPath):
            raise Exception(CrashReportPath + " already exists.")
                    
        ReformattedCrashedMutants = ""
        for Mutant in CrashedMutants:
            ReformattedCrashedMutants = ReformattedCrashedMutants + Mutant + "\n"
            
        CrashReportFile = None
        try: 
            CrashReportFile = open(CrashReportPath, "w+")
            CrashReportFile.write(ReformattedCrashedMutants)
        except:
            raise Exception("Couldn't store crash report")
        finally:
            if CrashReportFile != None:
                CrashReportFile.close()
         
                    
                    
                    
                    




# test code
# TestSuiteGeneratorName="UniformityStatistics"
# TestSuiteNumber= "1"
# TimeOut= 5
# Method_Name_Replacement="K_n().computeStatistic"
# MutantExecutor("K_n","Subject_Programs.K_n.Original_Program.K_n").processMutants(TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, True)
# test code
