import os
import sys
import ProcessRawData.Oracles.PostProcessing.readDataToBeProcessed as reader

class EquivalentMutantDetector():
    
    def __init__(self):
        self.EquivalentMutantDetection_Directory = (__file__[:__file__.rfind("\\")])
        self.ProcessRawData_Directory = (self.EquivalentMutantDetection_Directory[:self.EquivalentMutantDetection_Directory.rfind("\\")])
        self.Project_Root = (self.ProcessRawData_Directory[:self.ProcessRawData_Directory.rfind("\\")])
        self.Experimental_Results_Directory = self.Project_Root + "\\Experimental_Results"
    
    
    def filter_Equivalent_Mutants(self, Experiment_Folder):
        EquivalentMutants = self._get_Equivalent_Mutants(Experiment_Folder)
        self._produce_Filtered_Version_of_Raw_Data(Experiment_Folder,EquivalentMutants)
    
    # Input: Experiment_Folder, an experiment folder of TYPE=Oracle_*, from Experimental_Results
    # Process: Equivalent mutants are those that were not killed by SMT.
    def _get_Equivalent_Mutants(self, Experiment_Folder):
        Experiment_Path = (self.Experimental_Results_Directory + "\\" + Experiment_Folder)
        
        if "SMT_Summary.txt" not in (os.listdir(Experiment_Path)):
            print("Run ProcessRawData.StrongMutationTesting.main, before running this.")
            sys.exit()
        
        FileContents = None
        SMTSummaryFileObject = None
        try:
            SMTSummaryFileObject = open(Experiment_Path + "\\SMT_Summary.txt", "r")
            FileContents = SMTSummaryFileObject.readlines()
        except:
            raise Exception("Can't open SMT_Summary.txt")
        finally: 
            if SMTSummaryFileObject != None:
                SMTSummaryFileObject.close()
        
        EquivalentMutants = []
        del FileContents[0]
        for Mutant in FileContents:
            Mutant_Name = Mutant.split("\t")[0].strip()
            Killed_by_SMT = (Mutant.split("\t")[1].strip())
            
            if Killed_by_SMT == "False":
                EquivalentMutants.append(Mutant_Name)
        
        return EquivalentMutants
        
    def _produce_Filtered_Version_of_Raw_Data(self, Experiment_Folder, EquivalentMutants):
        
        Raw_Data_Contents = (reader.readData().get_Data_Oracle(Experiment_Folder))
        
        for Equivalent_Mutant in EquivalentMutants:
            del Raw_Data_Contents[Equivalent_Mutant]
        
        Tables = ""
        for Mutant in list(Raw_Data_Contents.keys()):
            TableHeader = Mutant + "\n"
            TableHeader = TableHeader + "Test Case Name" + "\t" + "Actual Output" + "\t" + "Predicted Output" + "\t" + "Verdict" + "\n" 
            TestSuite = Raw_Data_Contents[Mutant]["TestSuite"]
            for TestCase in list(TestSuite.keys()):
                TableHeader = TableHeader + TestCase + "\t" + TestSuite[TestCase]["ActualOutput"] + "\t" + TestSuite[TestCase]["PredictedOutput"] + "\t" + TestSuite[TestCase]["Verdict"] + "\n"
            
            Tables = Tables +  TableHeader + "\n"
            
        Raw_Data_Path = self.Experimental_Results_Directory + "\\" + Experiment_Folder + "\\Raw_Data.txt"
        Unfiltered_Raw_Data_Path = self.Experimental_Results_Directory + "\\" + Experiment_Folder + "\\Unfiltered_Raw_Data.txt" 

        # This block of code infinitely tries to rename file and keeps trying until it is successful
        while (True):       
            try:            
                os.rename(Raw_Data_Path, Unfiltered_Raw_Data_Path)            
                break
            except PermissionError:
                pass
        
        Raw_Data_FileObject = None
        try:
            Raw_Data_FileObject = open(Raw_Data_Path, "w+")
            Raw_Data_FileObject.write(Tables)
        except:
            raise Exception("Couldn't create the raw data text file (removing equivs)")
        finally:
            if Raw_Data_FileObject != None:
                Raw_Data_FileObject.close() 
         
            

            
        
        
        
    
    
        















