import os
import sys
import TestSuites.LoadTestSuite as LoadTestSuite
import threading
from abc import ABC, abstractmethod
import copy

class Experiment_Manager(ABC):
    
    
    def __init__(self):
        self.Experiment_Manager_Directory = (__file__[:__file__.rfind("\\")])
        self.Oracles_Directory = (self.Experiment_Manager_Directory [:self.Experiment_Manager_Directory .rfind("\\")])
        self.ProcessRawData_Directory = (self.Oracles_Directory[:self.Oracles_Directory.rfind("\\")])
        self.Project_Root = (self.ProcessRawData_Directory[:self.ProcessRawData_Directory.rfind("\\")])
        self.Experimental_Results_Directory = self.Project_Root + "\\Experimental_Results"
    
    
    
    # Experiment_Folder_Name, the name of the folder from which we will take the raw data from 
    # TestSuiteGeneratorName, the name of the test suite generator that was used to produce Experiment_Folder_Name 
    # TestSuite_Number, the test suite number that was used to produce Experiment_Folder_Name
    def run_Experiment(self, Experiment_Folder_Name, TestSuiteGeneratorName, TestSuite_Number):    
        Results = self._controller(Experiment_Folder_Name, TestSuiteGeneratorName, TestSuite_Number)
        New_Experiment_Path = self._saveResults(Experiment_Folder_Name, Results)
        return New_Experiment_Path 
#         # test code
#         for Mutant in Results:
#             print(Mutant)
#             for TestCase in Results[Mutant]["TestSuite"]:
#                 print(TestCase + "\t" + str(Results[Mutant]["TestSuite"][TestCase]))
#         # test code
    
        
    
    def _controller(self, Experiment_Folder_Name, TestSuiteGeneratorName, TestSuite_Number):
        Outputs = self._get_Outputs(Experiment_Folder_Name)
        TestCases = self._get_TestCases(TestSuiteGeneratorName, TestSuite_Number)
        
        Threads = []
        self.Results = {}
        for Mutant in list(Outputs.keys()):            
            
            TestSuite = (Outputs[Mutant]["TestSuite"])
            for TestCase in list(TestSuite.keys()):            
                Input = TestCases[(TestCase[len("Test_Case_"):])]
                Output = (TestSuite[TestCase]["Output"])
                                
                Thread = threading.Thread(target=self._evaluateOracle, args=(Mutant, TestCase, Input, Output))
                Threads.append(Thread)
                Thread.start()
        
        for thread in Threads:
            thread.join()
         
         
        res = copy.deepcopy(self.Results)
        self.Results.clear()
         
        return res 
                
    
    def _get_TestCases(self, TestSuiteGeneratorName, TestSuite_Number):
        TestSuite = LoadTestSuite.LoadTestSuite().LoadTestSuite(TestSuiteGeneratorName, TestSuite_Number)
        return TestSuite
        
    # Inputs: Experiment_Folder_Name, the name of the experiment folder in Experimental_Results
    def _get_Outputs(self, Experiment_Folder_Name):
        
        Experiment_Folder_Path = self.Experimental_Results_Directory + "\\" + Experiment_Folder_Name
        
        Mutants = {}
        for Mutant in (os.listdir(Experiment_Folder_Path)):
            Mutant_Path = Experiment_Folder_Path + "\\" + Mutant
            if os.path.isdir(Mutant_Path):
                
                Mutant_Name = Mutant

                TestCases = {}                
                for TestCase in os.listdir(Mutant_Path):
                                        
                    TestCasePath = Mutant_Path + "\\" + TestCase
            
                    TestCaseName = TestCase
                    
                    OutputPath = TestCasePath + "\\Output.txt"
                    OutputObject = None
                    Output = None
                    try: 
                        OutputObject = open(OutputPath, "r")
                        Output = OutputObject.readlines()
                    except:
                        raise Exception("Couldn't open output")
                    finally:
                        if OutputObject != None:
                            OutputObject.close() 
                    
                    TestCases[TestCaseName] = {"Output":Output}
                
                Mutants[Mutant_Name] = {"TestSuite":TestCases}
        
        return Mutants        
                
    
    # Input: - Mutant, the name of mutant
    #        - TestCase, the test case number
    #        - Input, the test input 
    #        - Output, the test output
    def _evaluateOracle(self, Mutant, TestCase, Input, Output):
        
        Result = self.Oracle(Input, Output)
        
        if Mutant not in list(self.Results.keys()):
            self.Results[Mutant] = {"TestSuite":{}}
        
        if TestCase not in list(self.Results[Mutant]["TestSuite"].keys()):
            self.Results[Mutant]["TestSuite"][TestCase] = Result
        
    
    # Input: - Input, the test input
    #        - Output, the test output
    # Process: This should implement an oracle
    # Output: A dictionary with two keys, "PredictedOutput" and "Verdict", and the associated values (for verdict, this is either "Passed" or "Failed".
    @abstractmethod    
    def Oracle(self, Input, Output):
        pass
    
    # Returns the name of the oracle
    @abstractmethod    
    def get_Oracle_Name(self):
        pass

    
    
    def _saveResults(self,Experiment_Folder_Name, Results):
        
        # Compute the path to New_Experiment_Folder, the place where we want to store the results
        Experiment_Manager_Directory = __file__[:__file__.rfind("\\")]
        Oracles_Directory = Experiment_Manager_Directory[:Experiment_Manager_Directory.rfind("\\")]
        ProcessingRawData_Directory = Oracles_Directory[:Oracles_Directory.rfind("\\")]
        Root_Directory = ProcessingRawData_Directory[:ProcessingRawData_Directory.rfind("\\")]
        Experimental_Results_Directory = Root_Directory + "\\Experimental_Results" 
        
        Experiment_Number = "Experiment_" + str(len(os.listdir(Experimental_Results_Directory))+1)
        Experiment_Type = "TYPE=" + "Oracle_" + self.get_Oracle_Name()
        Raw_Data = "RAW_DATA_FROM=" + (Experiment_Folder_Name[:Experiment_Folder_Name.find("(")])
        New_Experiment_Folder = Experiment_Number + "(" + Experiment_Type + "," + Raw_Data + ")"
        New_Experiment_Path = (Experimental_Results_Directory + "\\" + New_Experiment_Folder)
        
        if os.path.exists(New_Experiment_Path):
            raise Exception(New_Experiment_Path + " already exists")
        
        os.mkdir(New_Experiment_Path)
        
        # Reformats the Results into a csv format
        Tables = ""
        for Mutant in list(Results.keys()):
            Mutant_Name = Mutant
            Table_Header = Mutant_Name + "\n"
            Table_Header = Table_Header + "Test Case Name" + "\t" +  "Actual Output" + "\t" + "Predicted Output" + "\t" + "Verdict" + "\n"
            Table = Table_Header
            TestSuite = (Results[Mutant]["TestSuite"])
            for Test_Case in TestSuite:
                Test_Case_Name = Test_Case
                ActualOutput = TestSuite[Test_Case]["ActualOutput"]
                PredictedOutput = TestSuite[Test_Case]["PredictedOutput"]
                Verdict = TestSuite[Test_Case]["Verdict"]
                Table_Row = Test_Case_Name + "\t" + str(ActualOutput) + "\t" + str(PredictedOutput) + "\t" + str(Verdict) + "\n"
                Table = Table + Table_Row
            Tables = Tables + Table + "\n"
            
        # stores the results 
        File_Path = New_Experiment_Path + "\\Raw_Data.txt"
        if os.path.exists(File_Path):
            raise Exception(File_Path + " already exists")
        
        File_Object = None
        try:
            File_Object = open(File_Path,"w+")
            File_Object.write(Tables)
        except:
            raise Exception("Couldn't write the results")
        finally: 
            if File_Object != None:
                File_Object.close()

        
        return New_Experiment_Path
        
    





# test code
# Experiment_Folder_Name = "Experiment_1(Prog=K_n,TCG=UniformityStatistics,TestSuite=1,TimeOut=5)"
# TestSuiteGeneratorName = "UniformityStatistics"
# TestSuite_Number = "1"
# Experiment_Manager()._controller(Experiment_Folder_Name, TestSuiteGeneratorName, TestSuite_Number)
# test code
