import TestSuites.UniformityStatistics.TestSuiteGenerator as TCG
import TestSuites.LoadTestSuite as LoadTestSuite
import numpy
import TestSuiteExecutor.Executor as Executor
import os

class ModelCreator:
    
    # Input: - Subject_Program_Name, name of subject program
    #        - NumberOfTestCases, number of test cases to generate (if generating test cases) 
    #        - Subject_Program_Main_Class the main class of the subject progarm in dot notation
    #        - TimeOut, in seconds
    #        - Method_Name_Replacement, if you have a test suite, where only the method name changes (e.g. the params dont change), you can use this
    #          to change the method name
    #        - GenerateNewTestSuite, true or false, if true, uses NumberOfTestCases, otherwise uses ExistingTestSuitePath
    #        - ExistingTestSuitePath, the path to an existing test suite (MUST BE A TEST SUITE FOR CRESSIE2 OR PARDO - because these can be converted for the other uniformity statistics as well)
    def createModel(self, Subject_Program_Name, Subject_Program_Main_Class, TimeOut, Method_Name_Replacement, Bucket_Boundary, GenerateNewTestSuite, ExistingTestSuitePath=None, NumberOfTestCases=None):
        TestSuitePath = None
        if GenerateNewTestSuite:
            TestSuitePath = self._generate_Test_Suites(Subject_Program_Name, NumberOfTestCases)
        else:
            TestSuitePath = ExistingTestSuitePath
            
        Model_Data = self._generate_Model_Data(TestSuitePath, Subject_Program_Main_Class, TimeOut, Method_Name_Replacement, Subject_Program_Name)
        Model_Data_Path = self._save_Model_Data(Model_Data, TestSuitePath, Subject_Program_Name, TimeOut)
        self._bucketing(Bucket_Boundary, Model_Data_Path)
    
    
    def _generate_Test_Suites(self, Subject_Program_Name, NumberOfTestCases):
        
        tcg = None
        if Subject_Program_Name == "Cressie2" or Subject_Program_Name == "Pardo":
            tcg = TCG.TestSuiteGenerator("uniformitytest","UniformityStatistics",True)
        else: 
            tcg = TCG.TestSuiteGenerator("uniformitytest","UniformityStatistics",False)
        
        TestSuitePath = tcg.generateTestSuite(NumberOfTestCases, save=True)

        return TestSuitePath


    def _generate_Model_Data(self,TestSuitePath, Subject_Program_Main_Class, TimeOut, Method_Name_Replacement, Subject_Program_Name):
        TestSuite_Number = (TestSuitePath[TestSuitePath.rfind("\\")+1:][:TestSuitePath[TestSuitePath.rfind("\\")+1:].find("(")][TestSuitePath[TestSuitePath.rfind("\\")+1:][:TestSuitePath[TestSuitePath.rfind("\\")+1:].find("(")].rfind("_")+1:])
        TestSuite = LoadTestSuite.LoadTestSuite().LoadTestSuite("UniformityStatistics", TestSuite_Number)
        
        Table = Subject_Program_Name + "\t" + "Stdev" + "\n" 
        for TestCaseID in list(TestSuite.keys()):
            TestCase = TestSuite[TestCaseID]
            
            
            if Subject_Program_Name != "Cressie2" and Subject_Program_Name != "Pardo":                        
                if TestCase[TestCase.rfind("]")+1:] != ")":
                    TestCase = (TestCase[:TestCase.rfind("]")+1]+")")
            
            
                        
            ExecutionResults = Executor.Executor().execute_TestCase(TestCase, Subject_Program_Main_Class, TimeOut, Method_Name_Replacement)
            
            if ExecutionResults["Crashes"] == "Does Crash":
                raise Exception(TestCase + " crashed.")
            
            TestStatistic = str(ExecutionResults["Results"])
            
            Sample = []
            for i in (TestCase[TestCase.find("[")+1:TestCase.find("]")].split(",")):
                Sample.append(float(i.strip()))
            StandardDeviation = str(numpy.std(Sample))
        
            Table = Table + TestStatistic + "\t" + StandardDeviation + "\n"
        
        return Table
    
    def _save_Model_Data(self, Model_Data, TestSuitePath, Subject_Program_Name, TimeOut):
        
        TestSuite_Number = (TestSuitePath[TestSuitePath.rfind("\\")+1:][:TestSuitePath[TestSuitePath.rfind("\\")+1:].find("(")][TestSuitePath[TestSuitePath.rfind("\\")+1:][:TestSuitePath[TestSuitePath.rfind("\\")+1:].find("(")].rfind("_")+1:])
        Model_Data_Directory_Name = "Model_Data(" + "Prog="+Subject_Program_Name + ","+"TCG=UniformityStatistics,"+"TestSuite="+ TestSuite_Number+","+"TimeOut=" + str(TimeOut)+")"
        
        ModelDataPath = __file__[:__file__.rfind("\\")] + "\\Model_Data\\" + Model_Data_Directory_Name
        if os.path.exists(ModelDataPath) == False:
            os.makedirs(ModelDataPath)
        
        ModelDataFilePath = ModelDataPath + "\\" + "Data_"+str(len(os.listdir(ModelDataPath))+1)+".txt"
        ModelDataFile = None
        try:             
            ModelDataFile = open(ModelDataFilePath, "w+")
            ModelDataFile.write(Model_Data)
        except:
            raise Exception("Couldn't write model data")
        finally:
            if ModelDataFile != None:
                ModelDataFile.close()
        
        return ModelDataFilePath
        



    def _bucketing(self, Bucket_Boundary,Model_Data_Path):
        
        if os.path.exists(Model_Data_Path) == False:
            raise Exception(Model_Data_Path + " does not exist.")
        
        Lines = None
        Model_Data_File = None
        try: 
            Model_Data_File = open(Model_Data_Path, "r")
            Lines = Model_Data_File.readlines()
        except:
            raise Exception("Could not read model data")
        finally:
            if Model_Data_File != None:
                Model_Data_File.close()
        
        
        Lines_Indexes = list(range(len(Lines)))
        ReformattedLines = []
        MaxStdev = None
        del Lines_Indexes[0]
        for Line_Index in Lines_Indexes:
            Statistic = float(Lines[Line_Index].split("\t")[0].strip())
            Stdev = float(Lines[Line_Index].split("\t")[1].strip())
            
            if len(ReformattedLines) == 0:
                MaxStdev = Stdev
            else:
                if Stdev > MaxStdev:
                    MaxStdev = Stdev 
            
            ReformattedLines.append([Statistic,Stdev])
        
        
        Partitions = {}
        Partition_Lowerbound = 0
        Partition_Upperbound = Bucket_Boundary
        while Partition_Lowerbound < MaxStdev:
            Partitions[(Partition_Lowerbound,Partition_Upperbound)] = []
            Partition_Lowerbound = Partition_Lowerbound + Bucket_Boundary
            Partition_Upperbound = Partition_Upperbound + Bucket_Boundary
              
        
        for Line in ReformattedLines:
            Statistic = Line[0] 
            Stdev = Line[1]
            
            for Partition in list(Partitions.keys()):
                if Stdev >= Partition[0] and Stdev < Partition[1]:
                    Partitions[Partition].append(Statistic)
                    
        Table = "Standard Deviation Midpoint"+ "\t" + "Average Statistic Value" + "\n"     
        for Partition in Partitions:
            Table = Table + (str((Partition[0]+Partition[1])/2) + "\t" + str(numpy.mean(Partitions[Partition])) ) + "\n"
        
        
        
        
        Model_Data_File = None
        try:
            Model_Data_File = open(Model_Data_Path, "a")
            Model_Data_File.write("\n")
            Model_Data_File.write(Table)
        except:
            raise Exception("Couldn't append to file")
        finally:
            if Model_Data_File != None:
                Model_Data_File.close()
            
            
            


#import Mutation_Testing.Mutant_Loader.MutantExecutor as Executor


# ExecutorInstance = Executor.MutantExecutor(Subject_Program_Name, Subject_Program_Main_Class)
# ExecutorInstance.processMutants(TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, SaveResults, ProcessOriginal, ProcessMutants)


# test code
# TestSuiteGeneratorName="UniformityStatistics"
# TestSuiteNumber= "1"
# TimeOut= 5
# Method_Name_Replacement="K_n().computeStatistic"
# MutantExecutor("K_n","Subject_Programs.K_n.Original_Program.K_n").processMutants(TestSuiteGeneratorName, TestSuiteNumber, TimeOut, Method_Name_Replacement, True)
# test code



# ModelCreator().createModel("Pardo", "Subject_Programs.Pardo.Original_Program.Pardo",  5, "Pardo().computeStatistic", True, ExistingTestSuitePath="C:\\Users\\Krishna\\eclipse-workspace\\ExperimentInfrastructure\\TestSuites\\UniformityStatistics\\Test_Suites\\Test_Suite_1(MAIN_METHOD uniformitytest;sample_size_BETWEEN 3,3;mean_BETWEEN 0.5,0.5;standarddeviation_BETWEEN 0,0.28;gap_BETWEEN 3,3).json", NumberOfTestCases=4)




