import os
import sys
import copy
import numpy
import scipy.stats as stats



class Summariser:
    
    # The name of an existing summary experiment folder - results will be stored here
    # If no folder is supplied, one is created
    def __init__(self, Existing_Experiment_Folder=None):
        
        # Path to experimental results
        self.Summaries_Directory = __file__[:__file__.rfind("\\")]
        self.RegressionOracle_Directory = self.Summaries_Directory[:self.Summaries_Directory.rfind("\\")]
        self.Oracles_Directory = self.RegressionOracle_Directory[:self.RegressionOracle_Directory.rfind("\\")]
        self.ProcessRawData_Directory = self.Oracles_Directory[:self.Oracles_Directory.rfind("\\")]
        self.Project_Root = self.ProcessRawData_Directory[:self.ProcessRawData_Directory.rfind("\\")]
        self.Experimental_Results_Directory = self.Project_Root + "\\Experimental_Results"
        
        # Get all relevant experiments
        self.RelevantExperimentFolders_Paths = []
        Experiment_Folders = (os.listdir(self.Experimental_Results_Directory))
        for Experiment_Folder in Experiment_Folders:
            if "TYPE=" in Experiment_Folder:
                Type = (Experiment_Folder[Experiment_Folder.find("TYPE")+5:][:Experiment_Folder[Experiment_Folder.find("TYPE")+5:].find(",")])
                if (Type.strip()=="Oracle_RMO"):
                    self.RelevantExperimentFolders_Paths.append(self.Experimental_Results_Directory + "\\" + Experiment_Folder)
        
        self.RelevantExperimentFolders_Paths_Programs = {}
        for Relevant_Experiment_Folder in self.RelevantExperimentFolders_Paths:
            RAW_DATA_FROM = (Relevant_Experiment_Folder[Relevant_Experiment_Folder.find("RAW_DATA_FROM=")+len("RAW_DATA_FROM="):-1]) + "("
                        
            for Experiment_Folder in Experiment_Folders:
                if (Experiment_Folder[:len(RAW_DATA_FROM)] == RAW_DATA_FROM):
                    Program = (Experiment_Folder[Experiment_Folder.find("Prog=")+len("Prog="):][:Experiment_Folder[Experiment_Folder.find("Prog=")+len("Prog="):].find(",")])
                    
                    self.RelevantExperimentFolders_Paths_Programs[Relevant_Experiment_Folder] = Program
                    break
        
        
        if Existing_Experiment_Folder == None:
            Experiment_Number = "Experiment_"+str(len(os.listdir(self.Experimental_Results_Directory))+1)
            Type = "TYPE=Summary"
            NewExperimentPath = self.Experimental_Results_Directory + "\\" + Experiment_Number + "(" + Type + ")"
            if os.path.exists(NewExperimentPath):
                raise Exception(NewExperimentPath + " already exists.")
            os.mkdir(NewExperimentPath)
            
            Paths = ""
            for Path in self.RelevantExperimentFolders_Paths:
                Paths = Paths + Path + "\n"
            FileObject = None
            try: 
                FileObject = open(NewExperimentPath + "\\Data_Base_On.txt","w+")
                FileObject.write(Paths)
            except:
                raise Exception("Couldn't create data based on")
            finally:
                if FileObject != None:
                    FileObject.close()
            
            self.ResultsFolder = NewExperimentPath
        else:
            self.ResultsFolder = Existing_Experiment_Folder
            

            
        
                
        # test code
#         for i in self.RelevantExperimentFolders_Paths_Programs:
#             print(self.RelevantExperimentFolders_Paths_Programs[i] + "\t\t\t" + i)
#         sys.exit()
        # test code
        
        
        
    # computes two tables: TPR and FPR
    def computeMutationScore(self, Write=True):
        Table = "Mutation Testing Results" + "\n"
        Table = Table + "Subject Program" + "\t" + "Killed Mutants" + "\t" + "Survived Mutants" + "\t" + "Mutation Score" + "\n"
        
        FalsePositiveTable = "False Positive Results" + "\n"
        FalsePositiveTable = FalsePositiveTable + "Subject Program" + "\t" + "False Positive Occurred" + "\n"
        for Experiment in self.RelevantExperimentFolders_Paths:
            Subject_Program_Name = self.RelevantExperimentFolders_Paths_Programs[Experiment]
            
            AvsP_File_Path = (Experiment+"\\"+"AvsP_Wilcoxon.txt")
            AvsP_File_Object = None
            AvsP_File_Contents = None
            try: 
                AvsP_File_Object = open(AvsP_File_Path, "r")
                AvsP_File_Contents = AvsP_File_Object.readlines()
            except:
                raise Exception("Could not open AvsP file")
            finally:
                if AvsP_File_Object != None:
                    AvsP_File_Object.close()

            del AvsP_File_Contents[0]
            Killed = 0
            Survived = 0
            for Line in AvsP_File_Contents:
                if (Line[:len("Original_Program")] == "Original_Program") == False:
                    Mutant = Line.split("\t")
                    if (Mutant[len(Mutant)-1].strip()) == "True":
                        Killed = Killed + 1
                    else:
                        Survived = Survived + 1
                else:
                    Original = (Line.split("\t"))
                    FalsePositiveTable = FalsePositiveTable + Subject_Program_Name + "\t" + (Original[len(Original)-1].strip()) + "\n" 
        
        
            Table = Table + Subject_Program_Name + "\t" + str(Killed) + "\t" + str(Survived) + "\t" + str(float(Killed)/(float(Killed)+float(Survived))) + "\n"
        
        
        Tables = Table + "\n" + FalsePositiveTable
        if Write:
            self.write("MutationScore_FalsePositiveRate.txt", Tables)

    def ModelAccuracy(self):
        
        Model_Accuracy_Tables = ""
        
        for Experiment in self.RelevantExperimentFolders_Paths:
            Subject_Program_Name = self.RelevantExperimentFolders_Paths_Programs[Experiment]
            
            Model_Accuracy_Table = "Model Accuracy Table for " + Subject_Program_Name + "\n"
            Model_Accuracy_Table = Model_Accuracy_Table + "Actual Output" + "\t" + "Predicted Output" + "\n"
            
            Raw_Data_File_Path = (Experiment+"\\"+"Raw_Data.txt")
            Raw_Data_File_Object = None
            Lines = None
            try:
                Raw_Data_File_Object = open(Raw_Data_File_Path, "r")
                Lines = Raw_Data_File_Object.readlines()
            except:
                raise Exception("Couldn't read raw data ")
            finally:
                if Raw_Data_File_Object != None:
                    Raw_Data_File_Object.close() 
            
            Need = False
            for Line in Lines:
                if (Line.strip() == "Original_Program"):
                    Need = True
                    continue
                
                if Need==True:
                    if Line[:len("Test Case Name")] == "Test Case Name":
                        continue
                    elif len(Line.strip()) == 0:
                        Need = False
                        break
                    else:
                        Actual = Line.split("\t")[1].strip()
                        Predicted = Line.split("\t")[2].strip()
                        Model_Accuracy_Table = Model_Accuracy_Table + Actual + "\t" + Predicted + "\n"
                        
            Model_Accuracy_Tables = Model_Accuracy_Tables + Model_Accuracy_Table + "\n"
                
        self.write("Model_Accuracy.txt",Model_Accuracy_Tables)
                
                
    def write(self, NewFileName, Content):
        FO = None
        if os.path.exists(self.ResultsFolder + "\\" + NewFileName):
            raise Exception(NewFileName + " already exists")
        try: 
            FO = open(self.ResultsFolder + "\\" + NewFileName, "w+") 
            FO.write(Content)
        except:
            raise Exception("couldn't create " + NewFileName)
        finally:
            if FO != None:
                FO.close()
            
    
    
    def _get_Killed_Or_Survived(self, RMO_Experiment_Path):
        File_Object = None
        Lines = None
        try: 
            File_Object = open(RMO_Experiment_Path + "\\AvsP_Wilcoxon.txt","r")
            Lines = File_Object.readlines()
        except:
            raise Exception("Can't read from " + RMO_Experiment_Path)
        finally:
            if File_Object != None:
                File_Object.close()
        
        KilledOrSurvived = {}
        del Lines[0]
        for Line in Lines:
            MutantName = (Line.split("\t")[0])
            Killed = (Line.split("\t")[3].strip())
            KilledOrSurvived[MutantName] = Killed
        return KilledOrSurvived
    

    def computeDifferenceBetweenMutantsAndModels(self):
        
        Tables = ""
        
        # For each RMO experiment
        for Experiment in self.RelevantExperimentFolders_Paths:
            Program_Name = self.RelevantExperimentFolders_Paths_Programs[Experiment]
        
            # For each mutant, whether killed or survived
            KilledOrSurvived = self._get_Killed_Or_Survived(Experiment)
                        
            # Read in raw data file
            Lines = None
            FileObject = None
            try: 
                FileObject = open(Experiment+"\\Raw_Data.txt","r")
                Lines = FileObject.readlines()
            except:
                raise Exception("Couldn't read raw data")
            finally:
                if FileObject != None:
                    FileObject.close()
            
            # Get all mutant tables
            Mutants = []
            Current_Mutant = []
            for Line in Lines:
                if len(Line.strip()) == 0:
#                     if Current_Mutant[0].strip() != "Original_Program":
                    Mutants.append(copy.deepcopy(Current_Mutant))
                    Current_Mutant.clear()
                    continue
                Current_Mutant.append(Line)
            if len(Current_Mutant) != 0:
                Mutants.append(Current_Mutant)

            KilledMutants = []
            SurvivedMutants = []            
            for Mutant in Mutants:
                Mutant_Name = Mutant[0].strip()
                del Mutant[0]
                del Mutant[0]
                AbsDiffs = []
                for TestCase in Mutant:
                    
                    Actual = sys.maxsize
                    try:
                        Actual = float(TestCase.split("\t")[1])
                    except:
                        pass
                    
                    Predicted = float(TestCase.split("\t")[2])
                    
                    AbsDiff = abs(Actual-Predicted)
                    AbsDiffs.append(AbsDiff)
                MeanAbsDiff = numpy.mean(AbsDiffs)
                StdevAbsDiff = numpy.std(AbsDiffs)
                SkewnessAbsDiff = stats.skew(AbsDiffs)
                KurtosisAbsDiff = stats.kurtosis(AbsDiffs)
                MinAbsDiffs = min(AbsDiffs)
                MaxAbsDiffs = max(AbsDiffs)
                
                if KilledOrSurvived[Mutant_Name] == "True":
                    KilledMutants.append({Mutant_Name:{"MeanAbsDiff":MeanAbsDiff,"StdevAbsDiff":StdevAbsDiff,"SkewnessAbsDiff":SkewnessAbsDiff,"KurtosisAbsDiff":KurtosisAbsDiff, "MinAbsDiffs":MinAbsDiffs, "MaxAbsDiffs":MaxAbsDiffs}})
                else:
                    SurvivedMutants.append({Mutant_Name:{"MeanAbsDiff":MeanAbsDiff,"StdevAbsDiff":StdevAbsDiff,"SkewnessAbsDiff":SkewnessAbsDiff,"KurtosisAbsDiff":KurtosisAbsDiff, "MinAbsDiffs":MinAbsDiffs, "MaxAbsDiffs":MaxAbsDiffs}})
            
            
                        
            
            Table = Program_Name + "\n"
            Table = Table + "Mutant Name" + "\t" + "X-axis ID" + "\t" + "MeanAbsDiff" + "\t" + "StdevAbsDiff" + "\t" + "SkewnessAbsDiff" + "\t" + "KurtosisAbsDiff" + "\t" + "MinAbsDiff" + "\t" + "MaxAbsDiff" + "\t" + "Killed?" + "\n"
            X_Axis_ID = 1
            for MutantUC in KilledMutants:
                Mut_Name = list(MutantUC.keys())[0]
                Mut_Diff = MutantUC[Mut_Name]
                X_Axis = str(X_Axis_ID)
                Table = Table + Mut_Name + "\t" + X_Axis + "\t" + str(Mut_Diff["MeanAbsDiff"]) + "\t" + str(Mut_Diff["StdevAbsDiff"]) + "\t" + str(Mut_Diff["SkewnessAbsDiff"]) + "\t" +  str(Mut_Diff["KurtosisAbsDiff"])+ "\t" + str(Mut_Diff["MinAbsDiffs"])+ "\t" +  str(Mut_Diff["MaxAbsDiffs"]) + "\t" "True" + "\n"
                X_Axis_ID = X_Axis_ID + 1
            for MutantUC in SurvivedMutants:
                Mut_Name = list(MutantUC.keys())[0]
                Mut_Diff = MutantUC[Mut_Name]
                X_Axis = str(X_Axis_ID)
                Table = Table + Mut_Name + "\t" + X_Axis + "\t" + str(Mut_Diff["MeanAbsDiff"]) + "\t" + str(Mut_Diff["StdevAbsDiff"]) + "\t" + str(Mut_Diff["SkewnessAbsDiff"]) + "\t" + str(Mut_Diff["KurtosisAbsDiff"]) + "\t" + str(Mut_Diff["MinAbsDiffs"])+ "\t" +  str(Mut_Diff["MaxAbsDiffs"]) + "\t" "False" + "\n"
                X_Axis_ID = X_Axis_ID + 1                 
            
            
            
            Tables = Tables + Table + "\n"
            
        self.write("MeanAbsDiff.txt", Tables) 

            
            


# test code
# x = Summariser("C:\\Users\\Krishna\\eclipse-workspace\\ExperimentInfrastructure\\Experimental_Results\\Experiment_37(TYPE=Summary)")
# x.computeMutationScore()
# x.ModelAccuracy()
# x.computeDifferenceBetweenMutantsAndModels()
# test code