import os
import sys


class SMT:
        
    def __init__(self):
        self.StrongMutationTesting_Directory = (__file__[:__file__.rfind("\\")])
        self.ProcessRawData_Directory = (self.StrongMutationTesting_Directory[:self.StrongMutationTesting_Directory.rfind("\\")])
        self.Root_Directory = (self.ProcessRawData_Directory[:self.ProcessRawData_Directory.rfind("\\")])        
        self.Experimental_Results_Directory = self.Root_Directory + "\\Experimental_Results" 

    # Input: - Experiment_Folder, the name of an experiment folder from Experimental_Results, that contains mutant outputs and original program outputs.
    #        - SaveToThisFolder, the name of the experiment folder from Experiment_Results, in which you would like to save the results of SMT
    def conduct_SMT(self, Experiment_Folder, SaveToThisFolder):
        Outputs = self._get_Outputs(Experiment_Folder)
        SMT_Results = self._compare_Outputs_Controller(Outputs)
        self._save_Results(SMT_Results, SaveToThisFolder)
        
    # Input: Experiment_Folder, the name of an experiment folder from Experimental_Results, that contains mutant outputs and original program outputs.
    # Process: gets all the outputs
    def _get_Outputs(self, Experiment_Folder):
        Experiment_Path = self.Experimental_Results_Directory + "\\" + Experiment_Folder
        
        Outputs = {}
        
        # For each mutant
        Mutants = (os.listdir(Experiment_Path))
        for Mutant in Mutants:
            Mutant_Name = Mutant
            Mutant_Path = Experiment_Path + "\\" + Mutant
            if os.path.isdir(Mutant_Path):
                Outputs[Mutant_Name] = {"TestSuite":{}}

                # For each test suite
                TestSuite = os.listdir(Mutant_Path)
                for TestCase in TestSuite:
                    TestCase_Path = (Mutant_Path + "\\" + TestCase + "\\Output.txt")
                    
                    # Get the test cases output
                    Output = None
                    TestCase_FileObject = None
                    try: 
                        TestCase_FileObject = open(TestCase_Path, "r")
                        Output = TestCase_FileObject.readlines()
                    except:
                        raise Exception("Couldn't read " + TestCase_Path)
                    finally:
                        if TestCase_FileObject != None:
                            TestCase_FileObject.close()
                    
                    Outputs[Mutant_Name]["TestSuite"][TestCase] = Output
    
        return Outputs
    
    def _compare_Outputs_Controller(self, Outputs):
        
        Original_Program = Outputs["Original_Program"]["TestSuite"]
        
        SMT_Results = {}
        
        # For each mutant
        Mutants = list(Outputs.keys())
        del Mutants[Mutants.index("Original_Program")]
        for Mutant in Mutants:
            SMT_Results[Mutant] = {"TestSuite":{}}
            TestSuite = Outputs[Mutant]["TestSuite"]
            
            # For each test case
            for TestCase in list(TestSuite.keys()):
                Output_m = TestSuite[TestCase]
                Output_o = Original_Program[TestCase]
                
                Killed_by_SMT = True
                if str(Output_m) == str(Output_o):
                    Killed_by_SMT = False
                
                SMT_Results[Mutant]["TestSuite"][TestCase] = {"Original Output":Output_o,"Mutant Output":Output_m,"Killed by SMT":Killed_by_SMT}
                
        
        return SMT_Results
                

    def _save_Results(self, SMT_Results, SaveToThisFolder):
        SaveHere = self.Experimental_Results_Directory + "\\" + SaveToThisFolder
        
        Tables = ""
        Summary_Results = "Mutant" + "\t" + "Killed by SMT" + "\n"
        
        # For each mutant
        for Mutant in list(SMT_Results.keys()):
            TableHeader = Mutant + "\n"
            TableHeader = TableHeader + "Test Case" + "\t" + "Original Output" + "\t" + "Mutant Output" + "\t" + "Killed by SMT" + "\n"
            
            Killed_by_SMT = False
            
            # For each test case
            TestSuite = (SMT_Results[Mutant]["TestSuite"])
            for TestCase in list(TestSuite.keys()):                
                TableHeader = TableHeader + TestCase + "\t" + str(TestSuite[TestCase]["Original Output"]) + "\t"  + str(TestSuite[TestCase]["Mutant Output"]) + "\t" + str(TestSuite[TestCase]["Killed by SMT"]) + "\n"
                
                if TestSuite[TestCase]["Killed by SMT"]:
                    Killed_by_SMT = True
            
            Summary_Results = Summary_Results + Mutant + "\t" + str(Killed_by_SMT) + "\n"
            Tables = Tables + TableHeader + "\n"
        
        BreakdownObject = None
        try:
            BreakdownObject = open(SaveHere + "\\SMT_Breakdown.txt","w+")
            BreakdownObject.write(Tables)
        except:
            raise Exception("Can't create SMT breakdown")
        finally:
            if BreakdownObject != None:
                BreakdownObject.close()
            
        SummaryObject = None
        try:
            SummaryObject = open(SaveHere+"\\SMT_Summary.txt","w+")
            SummaryObject.write(Summary_Results)
        except:
            raise Exception("Can't create SMT summary")
        finally:
            if SummaryObject != None:
                SummaryObject.close()



#SMT().conduct_SMT("Experiment_1(Prog=K_n,TCG=UniformityStatistics,TestSuite=2,TimeOut=5)","Experiment_2(TYPE=Oracle_RMO,RAW_DATA_FROM=Experiment_1)")

        



