import abc
import numpy


class UniformityStatistic():
    
    # Input: sample; a list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Sorts the list, and then has getTestStatisticValue() compute the TestStatisticValue
    # Output: TestStatisticValue, which is a float.
    def computeStatistic(self,sample,gap=None):
        sample = sample[:]
        sample.sort()        
        TestStatisticValue = None
            
        if gap == None:
            TestStatisticValue = self.getTestStatisticValue(sample)
        else: 
            TestStatisticValue = self.getTestStatisticValue(sample, gap)
    
        return TestStatisticValue
    
    # Input: sample; a sorted list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Computes the test statistic value for the sample
    # output: The test statistic value,which is a float.
    # Notes: This method should not be called from outside of the class. 
    def getTestStatisticValue(self,sample, gap=None):
        pass







class C_n_plus(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):
        Differences = set()
        elementIndex = 1.0
        SampleSize = float(len(sample))
        while elementIndex <= SampleSize:
            Prediction = elementIndex/(SampleSize+1.0)
            Difference = float(sample[int(elementIndex)-2]) - Prediction
            Differences.add(Difference)
            elementIndex = elementIndex + 1.0
        return max(Differences)



class C_n_minus(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):
        Differences = set()
        elementIndex = 1.0
        SampleSize = float(len(sample))
        while elementIndex <= SampleSize:
            Prediction = elementIndex/(SampleSize+1.0)
            Difference = float(sample[int(elementIndex)-1]) - Prediction
            Differences.add(-Difference)
            elementIndex = elementIndex + 1.0
        return max(Differences)



class C_n(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):
        C_n_plusObj = C_n_plus()
        C_n_plusValue = C_n_plusObj.computeStatistic(sample)
        
        C_n_minusObj = C_n_minus()
        C_n_minusValue = C_n_minusObj.computeStatistic(sample)
                
        if C_n_plusValue >= C_n_minusValue:
            return C_n_plusValue
        else:
            return C_n_minusValue
