import abc
import numpy


class UniformityStatistic():
    
    # Input: sample; a list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Sorts the list, and then has getTestStatisticValue() compute the TestStatisticValue
    # Output: TestStatisticValue, which is a float.
    def computeStatistic(self,sample,gap=None):
        sample = sample[:]
        sample.sort()        
        TestStatisticValue = None
            
        if gap == None:
            TestStatisticValue = self.getTestStatisticValue(sample)
        else: 
            TestStatisticValue = self.getTestStatisticValue(sample, gap)
    
        return TestStatisticValue
    
    # Input: sample; a sorted list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Computes the test statistic value for the sample
    # output: The test statistic value,which is a float.
    # Notes: This method should not be called from outside of the class. 
    def getTestStatisticValue(self,sample, gap=None):
        pass





class Cressie2(UniformityStatistic):
    
    def getTestStatisticValue(self,sample,gap):
        Result = 0.0
        for elementIndex in range(-1,(len(sample)+1-gap)):
            Gim = self.getElement(elementIndex + gap, sample) - self.getElement(elementIndex, sample)
            Result = Result + pow(float(len(sample))*Gim,2)
        return Result
    
    # Input: elementIndex and sample, which are an array index and an array of numeric values respectively
    # Process and output: If elementIndex is a valid index for the sample, then the function returns the value stored in sample at position elementIndex. 
    #                     elementIndex can validly be out of bounds i.e. sample[-1] or sample[len(sample)]. These cases are handled by returning either 0 or 1 respectively.  
    def getElement(self, elementIndex, sample):
            if elementIndex == -1:
                return 0.0
            elif elementIndex == len(sample):
                return 1.0
            else:
                return float(sample[elementIndex])
      