import abc
import numpy


class UniformityStatistic():
    
    # Input: sample; a list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Sorts the list, and then has getTestStatisticValue() compute the TestStatisticValue
    # Output: TestStatisticValue, which is a float.
    def computeStatistic(self,sample,gap=None):
        sample = sample[:]
        sample.sort()        
        TestStatisticValue = None
            
        if gap != None:
            TestStatisticValue = self.getTestStatisticValue(sample)
        else: 
            TestStatisticValue = self.getTestStatisticValue(sample, gap)
    
        return TestStatisticValue
    
    # Input: sample; a sorted list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Computes the test statistic value for the sample
    # output: The test statistic value,which is a float.
    # Notes: This method should not be called from outside of the class. 
    def getTestStatisticValue(self,sample, gap=None):
        pass


class Pardo(UniformityStatistic):
    
    def getTestStatisticValue(self,sample,gap):
        Result = 0.0
        for elementIndex in range(len(sample)):
            if (float(len(sample))*(self.getElement(elementIndex+gap,sample) - self.getElement(elementIndex-gap,sample))) != 0:
                Result = Result + (2.0*float(gap))/(float(len(sample))*(self.getElement(elementIndex+gap,sample) - self.getElement(elementIndex-gap,sample)))            
        Result = Result*(1.0/float(len(sample)))
        return Result

    # Input: elementIndex and sample, which are an array index and an array of numeric values respectively
    # Process and output: If elementIndex is a valid index for the sample, then the function returns the value stored in sample at position elementIndex. 
    #                     elementIndex can validly be out of bounds. If elementIndex is lower than the smallest valid index, then the value stored at the smallest valid index is returned.
    #                     Similarly, if elementIndex is higher than the largest valid index, then the value stored at the largest valid index is returned          
    def getElement(self,elementIndex,sample):
        if elementIndex < 0:
            return float(sample[0])
        elif elementIndex >= len(sample):
            return float(sample[len(sample)-1])
        else:
            return float(sample[elementIndex])
