import abc
import numpy


class UniformityStatistic():
    
    # Input: sample; a list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Sorts the list, and then has getTestStatisticValue() compute the TestStatisticValue
    # Output: TestStatisticValue, which is a float.
    def computeStatistic(self,sample,gap=None):
        sample = sample[:]
        sample.sort()        
        TestStatisticValue = None
            
        if gap == None:
            TestStatisticValue = self.getTestStatisticValue(sample)
        else: 
            TestStatisticValue = self.getTestStatisticValue(sample, gap)
    
        return TestStatisticValue
    
    # Input: sample; a sorted list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Computes the test statistic value for the sample
    # output: The test statistic value,which is a float.
    # Notes: This method should not be called from outside of the class. 
    def getTestStatisticValue(self,sample, gap=None):
        pass





class Greenwood(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):       
        Result = 0.0
        for elementIndex in range(len(sample)+1):
            if elementIndex == 0:
                Result = Result + (float(pow(sample[elementIndex],2)))
            elif elementIndex == len(sample):
                Result = Result - (pow(1.0-float(sample[len(sample)-1]),2))
            else:
                Result = Result + (pow(float(sample[elementIndex]) - float(sample[elementIndex-1]),2))
        return Result



class QuesenberryMiller(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):
        Spacings = []
        for elementIndex in range(len(sample)+1):
            if elementIndex == 0:
                Spacings.append(float(sample[elementIndex]))
            elif elementIndex == len(sample):
                Spacings.append(1.0-float(sample[len(sample)-1]))
            else:
                Spacings.append(float(sample[elementIndex]) - float(sample[elementIndex-1]))
        Result = 0.0
        for elementIndex in range(len(sample)):
            Result = Result + (Spacings[elementIndex]*Spacings[elementIndex+1])
        Result = Result + Greenwood().computeStatistic(sample)
        return Result
