import abc
import numpy


class UniformityStatistic():
    
    # Input: sample; a list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Sorts the list, and then has getTestStatisticValue() compute the TestStatisticValue
    # Output: TestStatisticValue, which is a float.
    def computeStatistic(self,sample,gap=None):
        sample = sample[:]
        sample.sort()        
        TestStatisticValue = None
            
        if gap == None:
            TestStatisticValue = self.getTestStatisticValue(sample)
        else: 
            TestStatisticValue = self.getTestStatisticValue(sample, gap)
    
        return TestStatisticValue
    
    # Input: sample; a sorted list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Computes the test statistic value for the sample
    # output: The test statistic value,which is a float.
    # Notes: This method should not be called from outside of the class. 
    def getTestStatisticValue(self,sample, gap=None):
        pass





class Swartz(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):
        Result = 0.0
        for elementIndex in range(len(sample)):
            Result = Result + pow(((self.getElement(elementIndex-1,sample) - self.getElement(elementIndex-1,sample))/2.0)-1.0/float(len(sample)),2) 
        Result = Result * (float(len(sample))/2.0)
        return Result

    # Input: elementIndex and sample, which are an array index and an array of numeric values respectively
    # Process and output: If elementIndex is a valid index for the sample, then the function returns the value stored in sample at position elementIndex. 
    #                     elementIndex can validly be out of bounds i.e. sample[-1] or sample[len(sample)]. These cases are handled by returning either -sample[0] or 2-sample[len(sample)-1] respectively.          
    def getElement(self,elementIndex,sample):
        if elementIndex == -1:
            return float(-sample[0])
        elif elementIndex == len(sample):
            return 2.0-float(sample[len(sample)-1])      
        else:
            return float(sample[elementIndex])

