import abc
import numpy


class UniformityStatistic():
    
    # Input: sample; a list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Sorts the list, and then has getTestStatisticValue() compute the TestStatisticValue
    # Output: TestStatisticValue, which is a float.
    def computeStatistic(self,sample,gap=None):
        sample = sample[:]
        sample.sort()        
        TestStatisticValue = None
            
        if gap == None:
            TestStatisticValue = self.getTestStatisticValue(sample)
        else: 
            TestStatisticValue = self.getTestStatisticValue(sample, gap)
    
        return TestStatisticValue
    
    # Input: sample; a sorted list of numeric values. gap is an optional input and is only necessary for certain statistics. gap is an integer.
    # Process: Computes the test statistic value for the sample
    # output: The test statistic value,which is a float.
    # Notes: This method should not be called from outside of the class. 
    def getTestStatisticValue(self,sample, gap=None):
        pass

class D_n_plus(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):        
        Results = set()
        elementIndex = 1.0
        SampleSize = float(len(sample))
        for element in sample:            
            Result = (elementIndex/SampleSize)-float(element) 
            Results.add(Result)
            elementIndex = elementIndex+1.0                    
        return max(Results)



class D_n_minus(UniformityStatistic):
    
    def getTestStatisticValue(self, sample):
        Results = set()
        elementIndex = 1.0
        SampleSize = float(len(sample))
        for element in sample:            
            Result = None
            Results.add(Result)
            elementIndex = elementIndex+1.0
        return max(Results)
    
    
    
class V_n(UniformityStatistic):
    
    def getTestStatisticValue(self,sample):
        D_n_plusObj = D_n_plus()
        D_n_plusValue = D_n_plusObj.computeStatistic(sample)
        
        D_n_minusObj = D_n_minus()
        D_n_minusValue = D_n_minusObj.computeStatistic(sample)

        return D_n_plusValue+D_n_minusValue
