from abc import ABC, abstractmethod
import os
import json
import sys

class TestSuiteGenerator(ABC):

    # Input: - Main_Method, the name of the main method e.g. amicable_numbers_sum
    #        - Subject_Program_Name, the name of the subject program
    def __init__(self, Main_Method, Subject_Program_Name):
        self.Main_Method = Main_Method
        self.Subject_Program_Name = Subject_Program_Name 
        self.set_Test_Case_Parameters()



    # Input: Number_of_Test_Cases, the numebr of test cases to be generated
    # Process: Generates a test suite, that consists of Number_of_Test_Cases, number of test cases
    # Output: A test suite, formatted in two different ways, one for SMT_WMT, and the other for Sq_NSq 
    def generateTestSuite(self, Number_of_Test_Cases, save=False):
        
        TestSuite_Sq_NSq = []
        TestSuite_SMT_WMT = {}
        for TestCaseID in range(Number_of_Test_Cases):
                
            while True:
                TestCase = self.generateTestCase()
                if TestCase not in TestSuite_Sq_NSq:
                    TestSuite_Sq_NSq.append(TestCase)
                    TestSuite_SMT_WMT[str(TestCaseID+1)] = TestCase
                    break
        
        if save==True:
            return self._save_Results(TestSuite_SMT_WMT)
        
        return TestSuite_SMT_WMT 
        
    
    # This method should generated a random test case, and return it as a string e.g. "LCM(1, 2)"
    @abstractmethod
    def generateTestCase(self): 
        raise Exception("Abstract class not instantiated")
    
    # Should set up the test case parameters as global variables. In particular, the name of the main method and the boundaries for random generation
    @abstractmethod
    def set_Test_Case_Parameters(self):
        raise Exception("Abstract class not instantiated")
    
    # Should return the test case parameters back as a string - this is used for file naming
    @abstractmethod
    def get_Test_Case_Parameters(self):
        raise Exception("Abstract class not instantiated")
    
    
    # Input: TestSuite_Sq_NSq, TestSuite_SMT_WMT, the outputs of generateTestSuite
    # Process: Saves the test suite in a JSON file 
    def _save_Results(self, TestSuite_SMT_WMT):
        TestSuites_Directory = __file__[:__file__.rfind("\\")]
        Test_Suites_Container = TestSuites_Directory + "\\" + self.Subject_Program_Name + "\\Test_Suites" 
        
        if os.path.exists(Test_Suites_Container) == False:
            try: 
                os.mkdir(Test_Suites_Container)
            except: 
                raise Exception("Could not create " + Test_Suites_Container)
        
        JSON_File_Path = Test_Suites_Container + "\\Test_Suite_"+str(len(os.listdir(Test_Suites_Container))+1) + self.get_Test_Case_Parameters()+".json"
        JSON_Content = TestSuite_SMT_WMT
        
        JSON_File_Object = None
        try: 
            JSON_File_Object = open(JSON_File_Path,"w+")
            json.dump(JSON_Content, JSON_File_Object, indent=4)
        except:
            raise Exception("Unable to save test suite")
        finally:
            if JSON_File_Object != None:
                JSON_File_Object.close()
        
        return JSON_File_Path
         
        
        
                    
    
    
    
    
    
    
    
    
    