import TestSuites.LoadTestSuite as LoadTestSuite
import sys
import threading
from subprocess import STDOUT, check_output
import copy
import time

class Executor():
    
    # Input: - TestSuiteGeneratorName, the name of the directory in TestSuites that contains the test suite
    #        - The test suite number, expressed as a string
    #        - Method_Name_Replacement, this can be used to change the method name e.g. if the test case is x(a,b) and Method_Name_Replacement 
    #          is y, then the test case will be interpreted as y(a,b)
    #        - Subject_Program_Main_Class, the main class of the subject program, in dot notation
    def execute_Subject_Program(self, TestSuiteGeneratorName, TestSuiteNumber, Subject_Program_Main_Class, TimeOut, Method_Name_Replacement=None, PerformAdditionalActions=False):
        # Get Test Suite
        TestSuite = LoadTestSuite.LoadTestSuite().LoadTestSuite(TestSuiteGeneratorName, TestSuiteNumber)
        
        Threads = []
        self.Results = {}
        self.Crash_Test_Verdicts = []
        
                
        # for each test case in the test suite
        for TestCaseID in list(TestSuite.keys()):
            TestCase = TestSuite[TestCaseID]

            

            # Reformat the test case, if necessary
            ReformattedTestCase = TestCase
            if Method_Name_Replacement != None:
                ReformattedTestCase = self._replace_Method_Name(TestCase, Method_Name_Replacement)
            
            if PerformAdditionalActions == "add_uniformity_gap_3":
                ReformattedTestCase = self.add_uniformity_gap_3(ReformattedTestCase)
                
            

            
            
            Thread = threading.Thread(target=self._execute_Subject_Program,args=(TestCaseID, ReformattedTestCase, Subject_Program_Main_Class, TimeOut))
            Threads.append(Thread)
            Thread.start()

            while True:
                LiveThreads = 0
                for thread_id in reversed(range(len(Threads))):
                    if Threads[thread_id].is_alive():
                        LiveThreads = LiveThreads + 1
                    else:
                        del Threads[thread_id]
                if LiveThreads <= 200:
                    break

            
            if "Does Crash" in self.Crash_Test_Verdicts:
                break
                
        if "Does Crash" in self.Crash_Test_Verdicts:
            for Thread in Threads:
                if Thread.is_alive():
                    #Thread.exit()
                    pass

        for Thread in Threads:
            Thread.join()
        
        Results = copy.deepcopy(self.Results)
        self.Results.clear()
        
        Verdict = False
        if "Does Crash" in self.Crash_Test_Verdicts:
            Verdict = True
        
        self.Crash_Test_Verdicts.clear()        
        
        return {"Crashes":Verdict, "Results":Results}
    
    
    
    # Input: A test case and the intended method name
    # Process: Changes the method name in the test case to Method_Name_Replacement
    # Output: New test case
    def _replace_Method_Name(self,TestCase, Method_Name_Replacement):
        return Method_Name_Replacement+TestCase[TestCase.find("("):]
    
    
        
    def _execute_Subject_Program(self, TestCaseID, TestCase, Subject_Program_Main_Class, TimeOut):
        
        cmd = "C:\\WINDOWS\\system32\\cmd.exe"        
        TestSuiteExecutor_Directory = __file__[:__file__.rfind("\\")]
        Project_Root = TestSuiteExecutor_Directory[:TestSuiteExecutor_Directory.rfind("\\")]
        Subject_Program_Root = Project_Root + "\\" + Subject_Program_Main_Class.replace(".","\\")[:Subject_Program_Main_Class.replace(".","\\").rfind("\\")]
        #process = Popen(cmd,encoding="utf8",stdin=PIPE,stdout=PIPE,cwd=Subject_Program_Root)
        
        Main_Class_File = Subject_Program_Main_Class[Subject_Program_Main_Class.rfind(".")+1:]
        Line_1_Import_Main_Class = "import " + Main_Class_File + ";"        
        Line_2_Execute = "print(\'Begin Output: \' + \'\\n\' + str("+ Main_Class_File + "." + TestCase +") + \'\\n\' + \'End Output...\')"+";"        
        #cmd = repr(process.communicate("python -c" + " \""    +Line_1_Import_Main_Class+" "+Line_2_Execute+         "\"\n")) # The input line is too long.
        
        
        
        
        Verdict = "Does not crash"
        cmd = ""
        try:
            cmd = check_output("python -c"+ " \""+Line_1_Import_Main_Class+" "+Line_2_Execute+"\"\n", stderr=STDOUT, timeout=TimeOut, cwd=Subject_Program_Root)
            cmd = cmd.decode("utf-8")
        except:
            Verdict = "Does Crash"
            cmd = "Crashed"

        Current_XML_Tag = None
        ReformattedOutput = ""
        for Line in str(cmd).split("\n"):                    
                
            # Sets the Current_XML_Tag variable to be the XML tag we are in    
            if Line.strip() == "Begin Output:":
                Current_XML_Tag = "Output"
                continue 
            elif Line.strip() == "End Output...":
                Current_XML_Tag = None
                continue
        
            # Gets the output
            if Current_XML_Tag == "Output":
                ReformattedOutput = ReformattedOutput + Line.strip()
                    
#         process.kill()            
        self.Results["Test_Case_"+TestCaseID] = ReformattedOutput
        self.Crash_Test_Verdicts.append(Verdict)
        
#         print(TestCaseID + "\t" + ReformattedOutput + "\t" + Verdict)
    













    def execute_TestCase(self, TestCase, Subject_Program_Main_Class, TimeOut, Method_Name_Replacement=None):
        
        if Method_Name_Replacement != None:
            TestCase = self._replace_Method_Name(TestCase, Method_Name_Replacement) 
        
        cmd = "C:\\WINDOWS\\system32\\cmd.exe"        
        TestSuiteExecutor_Directory = __file__[:__file__.rfind("\\")]
        Project_Root = TestSuiteExecutor_Directory[:TestSuiteExecutor_Directory.rfind("\\")]
        Subject_Program_Root = Project_Root + "\\" + Subject_Program_Main_Class.replace(".","\\")[:Subject_Program_Main_Class.replace(".","\\").rfind("\\")]
        #process = Popen(cmd,encoding="utf8",stdin=PIPE,stdout=PIPE,cwd=Subject_Program_Root)
        
        Main_Class_File = Subject_Program_Main_Class[Subject_Program_Main_Class.rfind(".")+1:]
        Line_1_Import_Main_Class = "import " + Main_Class_File + ";"        
        Line_2_Execute = "print(\'Begin Output: \' + \'\\n\' + str("+ Main_Class_File + "." + TestCase +") + \'\\n\' + \'End Output...\')"+";"        
        #cmd = repr(process.communicate("python -c" + " \""    +Line_1_Import_Main_Class+" "+Line_2_Execute+         "\"\n")) # The input line is too long.

        Verdict = "Does not crash"
        cmd = ""
        try:
            cmd = check_output("python -c"+ " \""+Line_1_Import_Main_Class+" "+Line_2_Execute+"\"\n", stderr=STDOUT, timeout=TimeOut, cwd=Subject_Program_Root)
            cmd = cmd.decode("utf-8")
        except:
            Verdict = "Does Crash"
            cmd = "Crashed"

        Current_XML_Tag = None
        ReformattedOutput = ""
        for Line in str(cmd).split("\n"):                    
                
            # Sets the Current_XML_Tag variable to be the XML tag we are in    
            if Line.strip() == "Begin Output:":
                Current_XML_Tag = "Output"
                continue 
            elif Line.strip() == "End Output...":
                Current_XML_Tag = None
                continue
        
            # Gets the output
            if Current_XML_Tag == "Output":
                ReformattedOutput = ReformattedOutput + Line.strip()
        
        return {"Results":ReformattedOutput, 
                "Crashes":Verdict}






    def add_uniformity_gap_3(self, TestCase):
        return (TestCase[:TestCase.rfind(")")]+",3)")

    
    
    

# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.D_n_plus.Original_Program.D_n_plus", 5, "D_n_plus().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.D_n_minus.Original_Program.D_n_minus", 5, "D_n_minus().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.V_n.Original_Program.V_n", 5, "V_n().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.CramerVonMises.Original_Program.CramerVonMises", 5, "CramerVonMises().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.WatsonsU_2.Original_Program.WatsonsU_2", 5, "WatsonsU_2().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.C_n_plus.Original_Program.C_n_plus", 5, "C_n_plus().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.C_n_minus.Original_Program.C_n_minus", 5, "C_n_minus().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.C_n.Original_Program.C_n", 5, "C_n().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.K_n.Original_Program.K_n", 5, "K_n().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.T_1.Original_Program.T_1", 5, "T_1().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.T_2.Original_Program.T_2", 5, "T_2().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.HegazyGreen_T_1.Original_Program.HegazyGreen_T_1", 5, "HegazyGreen_T_1().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.HegazyGreen_T_2.Original_Program.HegazyGreen_T_2", 5, "HegazyGreen_T_2().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.Greenwood.Original_Program.Greenwood", 5, "Greenwood().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.QuesenberryMiller.Original_Program.QuesenberryMiller", 5, "QuesenberryMiller().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "2", "Subject_Programs.Cressie2.Original_Program.Cressie2", 5, "Cressie2().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "1", "Subject_Programs.Swartz.Original_Program.Swartz", 5, "Swartz().computeStatistic")
# Executor().execute_Subject_Program("UniformityStatistics", "2", "Subject_Programs.Pardo.Original_Program.Pardo", 5, "Pardo().computeStatistic")


# test = Executor().execute_Subject_Program("UniformityStatistics", "3", "Subject_Programs.C_n.Original_Program.C_n", 5, "C_n().computeStatistic")      
# print(test["Crashes"])
# for i in test["Results"]:
#     print(i + "\t" + test["Results"][i])



# import numpy
# List =     [0.5003810805336137, 0.5001768328853712, 0.49966747735962885, 0.49994093639374493, 0.4996518187331114, 0.50023181646861, 0.5003498513994807, 0.4999676811753452, 0.5002710466951525, 0.5009675089530619, 0.499779372423484, 0.49977649780410044, 0.500081983425534, 0.4998067766949882, 0.4997446017246332, 0.4999854641513234, 0.5000392256922163, 0.5001246500697816, 0.5000323987305599, 0.5002682756899021, 0.5002052742294282, 0.5001309131186542, 0.49925780722794255, 0.4994551390443113, 0.49991800863837177, 0.4998360729094069, 0.49996505724795526, 0.4995053739069643, 0.5002040412656714, 0.49842025147871744, 0.49983245788024255, 0.50007641618959, 0.49963933160484936, 0.49977521825008614, 0.5004541706529062, 0.4999736136269385, 0.5001555058436427, 0.5001934154604398, 0.49997428907559954, 0.4997463555766475, 0.5000241600564775, 0.5000047908358424, 0.4996342059417463, 0.5004500898566793, 0.5001651159243563, 0.49962418928893426, 0.49970558393521985, 0.5007832391376376, 0.4995162475852805, 0.5002035193519124, 0.49999295037875485, 0.4994920973075725, 0.4998944726608044, 0.49937404060580554, 0.5003192598323947, 0.499927851974188, 0.49977727565728436, 0.5002208521145769, 0.49923039710654343, 0.4999295657959887, 0.49931321856863464, 0.49974027593197223, 0.5005894431479746, 0.5001228484973407, 0.49997262961872374, 0.4998785451443951, 0.500215622443776, 0.49989798163648125, 0.4998916421837227, 0.49933757911192644, 0.4998416848635047, 0.500448185566269, 0.49983154541653496, 0.5000110996632108, 0.5000190921643826, 0.5000618436701878, 0.49998688145458803, 0.5004113294798954, 0.4999401519477274, 0.5005572756220902, 0.4993513977531522, 0.5002727837220797, 0.5000130317418751, 0.5001441559176214, 0.5001135045468333, 0.4996053627980041, 0.5006928382213462, 0.4997715298441156, 0.49967664093844033, 0.4997845241293031, 0.5003920211625549, 0.5001907687476278, 0.49988196998563056, 0.5011088814745762, 0.5000075867233988, 0.49953874694709904, 0.5005521994995965, 0.5001288383880073, 0.49980367990942975, 0.500283627118295, 0.499943467833609, 0.5000244091512522, 0.49974720738303363, 0.5003652035004356, 0.5002491915450455, 0.4991919251813247, 0.4998100765424847, 0.49976084035234336, 0.5000955053686351, 0.49978263375668697, 0.49904466306504847, 0.500217734139565, 0.500379368163463, 0.5001323712538529, 0.4999227806751774, 0.5001409553693604, 0.5004484523873767, 0.5003133103180908, 0.500380241426312, 0.5002650672113924, 0.5004367618287225, 0.5001738664795369, 0.500279042473501, 0.5001591777658113, 0.5003306617944393, 0.5001063917319782, 0.5000568772497919, 0.4997442980626915, 0.500980872751162, 0.5005385400459911, 0.4996832676905252, 0.49977550700804674, 0.5000496913524364, 0.5005678628537381, 0.4997854070044762, 0.499303535261062, 0.5000766460564005, 0.5002426625864287, 0.4993808841478417, 0.4993639269466789, 0.5000219737908402, 0.5000016146087203, 0.49976530640083083, 0.5003504735097399, 0.5001372446361388, 0.5003846899458994, 0.49980721027796476, 0.5001268293279456, 0.4997068960799421, 0.4999598908034939, 0.49999727016229784, 0.5005675870910541, 0.5005843247030156, 0.5001490355409345, 0.5001857264785551, 0.4998793987515073, 0.49962221670673795, 0.5004308039544718, 0.500462297875419, 0.5002497613135601, 0.5001146334810336, 0.49960548026487156, 0.5003400174138922, 0.4998821163045968, 0.49961246209758, 0.5003808184773972, 0.499890665753453, 0.5000264165312315, 0.5003545246618288, 0.5008503544057785, 0.4996722352316858, 0.49922127687140944, 0.4997628411169653, 0.49989302140842784, 0.5006680787393943, 0.5001745425300508, 0.4997075896392672, 0.49971977230146103, 0.5001366921920829, 0.49986533795470117, 0.5000535673872402, 0.5001663697711818, 0.4995526596417683, 0.5005034765757651, 0.49997964424246794, 0.5013104147572013, 0.49959692657182814, 0.49972868751888183, 0.4995987461640411, 0.49992774809867074, 0.49984618714276896, 0.49923857012332673, 0.5001617403380345, 0.4996848412171179, 0.5004359976132102, 0.5000978580574316, 0.5002770694081126, 0.5005503586738754, 0.4993971187580247, 0.5000366962530298, 0.5001443989633392, 0.5000180601976092, 0.49979473723972456, 0.49987498795591706, 0.49985680299854646, 0.49975719744035807, 0.4998416236934047, 0.5004207152619199, 0.500115067236126, 0.49980708657518036, 0.4991667957792025, 0.5010981009193753, 0.499707124580645, 0.4995539107812258, 0.4998956606275515, 0.4995163177014785, 0.4998869718056214, 0.4997735566613881, 0.49956873671044644, 0.49943237820556186, 0.49983238745651126, 0.49945516005598267, 0.4992118485832396, 0.5004904522911228, 0.49996097633295167, 0.5001736134094776, 0.5001000578970539, 0.5002055271112739, 0.4997953884540676, 0.4991851123436217, 0.49982113047889454, 0.5003733710325968, 0.5001800005390178, 0.5001337733078489, 0.5002408179701906, 0.5002114290055767, 0.4997334311723581, 0.4995750961414337, 0.5002144047291494, 0.4999419190805611, 0.4993615854984995, 0.5000768659614145, 0.5001453990292761, 0.49998010107182783, 0.4998161518640898, 0.4995253561893005, 0.4996134017505186, 0.4999203062324657, 0.49992365723767324, 0.5003438921935648, 0.49976849865674783, 0.5001019129246694, 0.49997727424291005, 0.4996119202531229, 0.4995626111213664, 0.5001752534379487, 0.49976582238558975, 0.4999823154908674, 0.5002174869599596, 0.4997961777346813, 0.5000008445158007, 0.49976173288965337, 0.5003061293578467, 0.5002523881860874, 0.49953694768115736, 0.5002645056524847, 0.5002710274566898, 0.5006467445443543, 0.4998499297943329, 0.5006526421663714, 0.4999284557393803, 0.5004041954517344, 0.5006090121174619, 0.5000573359205782, 0.49976020353013395, 0.5000122111592321, 0.5001329809807441, 0.5003919646645272, 0.4992704756255969, 0.4994756719573429, 0.5000152062029596, 0.5002919303913402, 0.499802044450274, 0.500080525204457, 0.49961983842303825, 0.500165107268027, 0.4999604641822892, 0.5009710961258514, 0.5005473523305061, 0.5003352257803189, 0.4999465936116481, 0.5003483855171368, 0.5000821266217221, 0.5000432464344433, 0.4999844817810861, 0.4998790691700091, 0.5007495121117955, 0.5000077624943411, 0.500415229947272, 0.49994899746925975, 0.5001222918401496, 0.4999094189856634, 0.4999078552871373, 0.4995871160736707, 0.4998109224279852, 0.5001273165753564, 0.49985294862613383, 0.5003389993647627, 0.5001077383193543, 0.5002329677771944, 0.5003908691724823, 0.5003474266542973, 0.4996676656061023, 0.5005323962959692, 0.5001639666798705, 0.4999603253316317, 0.5005351345300261, 0.49994319389852215, 0.499871144123879, 0.49981846567649113, 0.5000960437506653, 0.49974118178000726, 0.4998828689139029, 0.5002098935538412, 0.49986628894499424, 0.5002422285109109, 0.500784313713066, 0.5002443173837645, 0.4998150298765118, 0.4997076873537212, 0.5002383647968691, 0.5008676102116583, 0.4999153698557321, 0.4998377798783499, 0.5000918920499421, 0.49974072980562656, 0.5001930188040277, 0.5005800754560423, 0.49992517980189055, 0.49983584163816663, 0.49999258494131504, 0.5004059470214088, 0.4991480393966733, 0.5005732874477838, 0.4995521030459097, 0.49998089377336274, 0.49958107954018505, 0.5004818548937118, 0.49956418546172887, 0.500046684056508, 0.5004390789556625, 0.49995131937996046, 0.5000159691016806, 0.49983613138461563, 0.5006557740600701, 0.4997371057340005, 0.5006324289251313, 0.5001121486605019, 0.5001354781122139, 0.5000536903542236, 0.5000312330969072, 0.5001290341420708, 0.49994667155634925, 0.49998407785424537, 0.5001439179876769, 0.49873040642905414, 0.5001575600058538, 0.49986403894304415, 0.5001864445247726, 0.49998717710422985, 0.5005397649642753, 0.5000474793160234, 0.5007772962711786, 0.5006324771271781, 0.49960637702865784, 0.4997395389918956, 0.5000501149520734, 0.4999884653571472, 0.5006328076948563, 0.49994959528432237, 0.4997586269973934, 0.5000248271122261, 0.5004349467147259, 0.50003828401401, 0.500240335686464, 0.49974264109585464, 0.4994973488011671, 0.500190315814551, 0.5000589050449209, 0.49983072894951613, 0.49963302613819843, 0.49955728412333206, 0.4993612662561351, 0.5002089404919604, 0.499979395008954, 0.4995845809242439, 0.5004130896391167, 0.5001044351004467, 0.49971526806037164, 0.49987697052086216, 0.49980465233165766, 0.5001328221691443, 0.5000802861556678, 0.5002319990698574, 0.4999811064844136, 0.49993321716860417, 0.5000350382745096, 0.5000231436030781, 0.500028697000707, 0.49988550043112134, 0.4998225626763056, 0.5001029102456149, 0.5000857887762801, 0.5002339198598316, 0.5000144299206114, 0.5001038940341044, 0.49996472785959967, 0.5001960219523205, 0.500183787554057, 0.49988784869163966, 0.49998655811022746, 0.4995244663102812, 0.4995918230257939, 0.49998695440746577, 0.5007116437165593, 0.5002047313116663, 0.5003807494457573, 0.5004284938655892, 0.500469117144308, 0.5006344894667905, 0.5003127723239922, 0.5004894331104497, 0.5001093647139135, 0.5001991545862985, 0.5002380547231883, 0.5000787706718349, 0.500330174439908, 0.4998059064225818, 0.5005711452520654, 0.49995582056462184, 0.4994623636150074, 0.49927097849265195, 0.5003336966989568, 0.499263165790707, 0.5000191407412766, 0.5002650103351944, 0.4997135322709587, 0.5001492354271948, 0.4998306327704233, 0.49959760177097495, 0.5000442201371428, 0.5001197987311491, 0.5006393545155882, 0.49981948776200935, 0.49986607072312095, 0.5005871173438294, 0.4995435505051052, 0.5003584973313512, 0.5003675001004741, 0.4999567283314326, 0.5002147886050845, 0.4993714720537358, 0.500273353002772, 0.5005214757016893, 0.4995052716819652, 0.4998049196827082, 0.5008338716706937, 0.49968419848192946, 0.4995764542808274, 0.49925810888812844, 0.499357895875825, 0.49956962230901847, 0.500630777492092, 0.4997304202917528, 0.49970103685964024, 0.5007499517083526, 0.5003435684607878, 0.5002937344191011, 0.5007033693228823, 0.49997179684499315, 0.49994362887208377, 0.4998258416312645, 0.5004376418325199, 0.49987801106423546, 0.5001745971395337, 0.49959448944571755, 0.5003734590831311, 0.49992080244443154, 0.5005330753306596, 0.5003544180759056, 0.5003998841502096, 0.5002754598824402, 0.4998214241016627, 0.499609728266906, 0.5002308019682565, 0.4998628242837231, 0.5000951277683403, 0.499633379642991, 0.5000992452506616, 0.4999211846807919, 0.49981013429995175, 0.5003266386255231, 0.5004011715051244, 0.4999785026841429, 0.5002515077052319, 0.4999615028553171, 0.5001153077488896, 0.5001424615119094, 0.49981053288824295, 0.5008563553181058, 0.5004248376887893, 0.5002896275285464, 0.49986289372901743, 0.49983004331370334, 0.5004620444655788, 0.5001842401119135, 0.5000006587269845, 0.5003194823358014, 0.5000293566988137, 0.5005575889777383, 0.4996269721699708, 0.49964318193313373, 0.5005835678635349, 0.5002329029770689, 0.4997184816863079, 0.5003975132791888, 0.49992572740544533, 0.5003077803643825, 0.5003156491418752, 0.5003128945909145, 0.4999280197927913, 0.4998398650910748, 0.49947381040540023, 0.4995292325157729, 0.5004068487436986, 0.5000720317512586, 0.49982530308366324, 0.4997659667323097, 0.499900895694345, 0.4992168058438218, 0.500087339634247, 0.5002945606684436, 0.4998199176919023, 0.5000764415168372, 0.5006177198433807, 0.49990074166545606, 0.5000429071975647, 0.49935151725969223, 0.4998708044048237, 0.4998206447154402, 0.49991144054187675, 0.4998437488630291, 0.5004655273837393, 0.5002164793664293, 0.49994270156185044, 0.5005769993880463, 0.4999037859782067, 0.5005212223778224, 0.49990271377787526, 0.5001691979625621, 0.5002993865144698, 0.5000154805609361, 0.4994460557012571, 0.5006091959145897, 0.5000193148465165, 0.5004435541786741, 0.5001677100423872, 0.4996303287553061, 0.49942776053637, 0.5002298622515381, 0.4999510000820409, 0.4999812797282614, 0.499864182065927, 0.5000157613079952, 0.5000776081457498, 0.4998938031846558, 0.5000862798139492, 0.49980419255713143, 0.500244709949548, 0.5003374529065658, 0.5000849935704372, 0.4999457344420733, 0.49976823676509213, 0.5002038207462864, 0.5000164567642484, 0.5002119529537536, 0.4999957463666292, 0.5000550302532014, 0.5000804811606169, 0.5008728407881297, 0.500261331343406, 0.4994333234506298, 0.5007817481573986, 0.5004008543878061, 0.5000697418512461, 0.4995801370681948, 0.5000483455209083, 0.4995348911437558, 0.4999492257201186, 0.5000387797096427, 0.500172488040687, 0.5001394932578124, 0.4997424550986646, 0.49995529717504716, 0.49950294932631556, 0.4997251725988833, 0.49961129501874435, 0.49969316338653796, 0.49992497687507886, 0.5001681915798687, 0.5000273978553884, 0.5004017223500501, 0.49990651625301846, 0.5001775554584823, 0.4998954019528927, 0.49995058533960846, 0.5003232919662317, 0.5002715374590943, 0.5002873828327024, 0.5002045529907415, 0.5002028022348696, 0.5001223922502206, 0.4995346562787823, 0.500704606660804, 0.49966569089466445, 0.500440213843298, 0.5003052899978235, 0.5001130824099091, 0.5001080718610322, 0.499387833708659, 0.49973754328018505, 0.4998876881733652, 0.5001445522234274, 0.5003860316645138, 0.50060489184139, 0.49932056924821205, 0.5000593473723043, 0.4996992352659449, 0.4993711527174311, 0.5002774885072616, 0.49958782139015695, 0.4995479844848898, 0.5001683605282575, 0.5000969620153094, 0.5008213323284132, 0.499654476767136, 0.5001597967446153, 0.5003044579238308, 0.49939583822570804, 0.5002191190209953, 0.49976369325469144, 0.5002200402695274, 0.4996209001836217, 0.4996018756244035, 0.5002635007714753, 0.4995123371758851, 0.5001016933422973, 0.49952986401742877, 0.5005396444624038, 0.4998200105338962, 0.49985555757925815, 0.5001088758312314, 0.4995681977149737, 0.5006068621139745, 0.5000322315625743, 0.5006324320410824, 0.5002254873864682, 0.500434356195666, 0.49971821590358423, 0.500087036540491, 0.5003456068993852, 0.5000224155449434, 0.49985768054493207, 0.4999918015523502, 0.49996722439714897, 0.5000386333096611, 0.4995226860045555, 0.4995476848556239, 0.5003706336373207, 0.5003303651906521, 0.49988497156759004, 0.5002259078136554, 0.5003398562236901, 0.5001780830483467, 0.5001359396686418, 0.5000024676512533, 0.49955007515042854, 0.4997890363962453, 0.5001899545440678, 0.5002424423839729, 0.4998588489752702, 0.500105372881403, 0.5000936371266129, 0.4998689877518845, 0.5002627700268429, 0.5002106705963648, 0.5002474821818732, 0.5005314647216801, 0.49947474724566265, 0.4997664635621641, 0.4995770746550656, 0.5004489088134796, 0.49990985719157344, 0.4996471509245126, 0.5002623758902082, 0.4999414323400525, 0.5007395823650733, 0.4999000078777555, 0.49973002250113857, 0.4999377073342949, 0.4996930912996392, 0.49972985114085267, 0.49976511215662833, 0.4996478403193061, 0.4997058524331764, 0.49994077996607356, 0.5001026364904775, 0.4997147767534213, 0.49994002475881677, 0.5005474390108463, 0.5005487058614615, 0.49976596070516377, 0.5002454553675019, 0.4997812838229392, 0.5001640832145372, 0.4999568960903089, 0.5001590666228436, 0.5003791105262612, 0.49999486110646796, 0.500243997135023, 0.4998289212994094, 0.4991925403018576, 0.5001674879378308, 0.4998446450319306, 0.5004093041852995, 0.4995557855173195, 0.5004475078643525, 0.4996231916314636, 0.49934231928887, 0.5002728255715129, 0.5002952276308307, 0.5012362723469926, 0.5001899385715474, 0.49957087217802276, 0.49977378781750426, 0.5003544614419119, 0.5003275682474956, 0.4998626628239427, 0.499673221723869, 0.5001526177072207, 0.49996816123357757, 0.49994329331864934, 0.49967681995864405, 0.4995781222223207, 0.5000825810770765, 0.5000291727299077, 0.49995893414891407, 0.4998412770850502, 0.5000991109552562, 0.49994462504311793, 0.49995990691202186, 0.500165830655704, 0.49973934419331356, 0.4999719985178303, 0.5003465505795462, 0.5008124248153197, 0.5001352517820568, 0.5001561198314713, 0.5000734941080013, 0.4999522884884338, 0.49985429499615414, 0.5003411849017604, 0.5006676989041774, 0.49988740923561054, 0.4998330538854049, 0.49998661624966284, 0.5000576095569269, 0.49967506350639335, 0.4999452870328737, 0.49952243132458884, 0.500260469699532, 0.49964839133560723, 0.4999030375951131, 0.5005720551558003, 0.4999922402712476, 0.5000364326390719, 0.5003023101451289, 0.4998792785101111, 0.5004172968593799, 0.5004765686988156, 0.499738367354296, 0.4998985112403221, 0.5000401693026167, 0.49946341797266347, 0.49984430266311747, 0.5000556201912884, 0.4999924042450955, 0.49998072947393707, 0.49959962830370835, 0.4998873022666221, 0.500135496852141, 0.4998611775049269, 0.5005851834201419, 0.4997901782290848, 0.5004310550643198, 0.5001182535692187, 0.49973494009508534, 0.5001088820483973, 0.5001632891533744, 0.499870415700809, 0.5001923315081118, 0.5002460973187502, 0.49944079384503365, 0.4998690386213324, 0.4998048028232418, 0.4998952207399458, 0.5002071409964484, 0.4999463167462251, 0.5003871435586369, 0.4999753553370055, 0.5002576973854606, 0.4998755596157913, 0.4999196501123649, 0.5001485324963179, 0.4998275207770931, 0.4999700805393365, 0.5004167144637648, 0.5004634239592026, 0.5002697687258855, 0.4999812714041621, 0.5000981262314899, 0.499982523578218, 0.49936199551405036, 0.5001484023634563, 0.5002233700527261, 0.49960735102746257, 0.5000460408050098, 0.49996998317279046, 0.49996783742780304, 0.5000269586911579, 0.499900980264553, 0.5001789896550517, 0.49993239386865385, 0.49973903608730613, 0.5008438701625302, 0.4998017025342841, 0.49974044463644185, 0.49974038399514975, 0.4997909623129217, 0.5002053597553339, 0.49997884357550826, 0.5001919837926576, 0.5002403648613206, 0.5007157006259583, 0.49947460438263835, 0.5003738695131317, 0.4996016224465032, 0.5004689475601544, 0.5003798831552939, 0.5005094751575453, 0.5003857805819353, 0.5000372048466779, 0.4998069613469283, 0.4995241912150839, 0.5004245786152376, 0.49969899584674393, 0.4997486921054474, 0.5001285550607897, 0.49974123311969365, 0.5002312617728386, 0.5004826060612606, 0.500127759933224, 0.49947380234978644, 0.499489467012224, 0.5000521210954951, 0.49982219593611293, 0.5005509691652699, 0.49986079510721904, 0.49944905383972227, 0.5000704187915337, 0.5004185179626153, 0.4996083215283615, 0.49980383044122945, 0.4994339503173893, 0.5008258038806355, 0.5005106845015176, 0.5005003406460338, 0.5002194986991636, 0.5003528554328398, 0.5002033491880178, 0.4997542601254893, 0.5001517571738218, 0.5010728774013462, 0.49945231338560464, 0.5000234913487496, 0.5003550096907731, 0.5004071681865659, 0.499780052410386, 0.5000237126579483, 0.500080083666368, 0.5005953516534165, 0.500368501594691, 0.5001752551491908, 0.4999599997309175, 0.49925214441869337, 0.5000082060975326, 0.4996749865593474, 0.4997754632809623, 0.49978795026488027, 0.4997281752918418, 0.49957257233685437, 0.4999699819904354, 0.4993510401945053, 0.5002022001900366, 0.4999590122165465, 0.4993025759054814, 0.500214178178945, 0.49972387916260186, 0.4999383623922063, 0.5001209902639554, 0.4999580832044358, 0.5003549239409548, 0.4996849554357261, 0.5001743840448117, 0.5005595110786261, 0.500068494777497, 0.5002488542325831, 0.4999102873019293, 0.5003271700231232, 0.5002404678159407, 0.5003282294764433, 0.4997145120023509, 0.500005298749965, 0.49929603503225733, 0.500117617078044, 0.4994810921998018, 0.4992330146567071, 0.49914766922449544, 0.49987314341689965, 0.4999212864906632, 0.4998816059290398, 0.5002218120656908, 0.499808428342615, 0.49985045387354154, 0.5002733391475442, 0.4999233550970822, 0.49924639298122025, 0.4999226260533922, 0.49984844076240276, 0.5002786292610621, 0.4999129056724952, 0.49986019256534614, 0.4999691697141343, 0.5000963872892215, 0.50015467930076, 0.5001311826396192, 0.49987128025541094, 0.4995906877632831, 0.5001316095264186, 0.5001723632757682, 0.5005668200367851, 0.49907012096602255, 0.5003210900791253, 0.49973278751278316, 0.5003951351914787, 0.49961119250850106, 0.4996463750630846, 0.5001934683587571, 0.5000842358311033, 0.5002529167505378, 0.4995812173771793, 0.499766039833329, 0.5007400213306311, 0.5002253114438234, 0.4995535350260288, 0.5000904687601284, 0.50019794306075, 0.5000954332064589, 0.5005621543144355, 0.4992663273569671, 0.4992310407182793, 0.5003684873234584, 0.499819501527126, 0.5002439744998679, 0.49993090279963315, 0.5007731616168702, 0.5000184382594187, 0.49939658180164054, 0.4997235929268649, 0.49983269060907154, 0.500289453015651, 0.5004200945890329, 0.4998674245878606, 0.49988836898847955, 0.500028507605202, 0.5001020516591665, 0.49919962561450065, 0.5002885229068124, 0.5002066602556293, 0.4999656682142824]
# List =     [0.41003763824852646, 0.7975594444648146, 0.6518300511192924, 0.8069148099913949, 0.6958763328646432, 0.6241895126245923, 0.1831832752378036, 0.8272298530360445, 0.560494917068876, 0.5719357517719559, 0.03289110795443945, 0.6586641497510818, 0.16711094838016605, 0.36526756398101473, 0.7476841975194475, 0.573696200346341, 0.6358095112993793, 0.4709551197824277, 0.07322078206600385, 0.5761701450983804, 0.36831966717409725, 0.42100583496222616, 0.5837711433951327, 0.7793377939701377, 0.47076843802048407, 0.6079996480314034, 0.7923421890653204, 0.9318325168448718, 0.5322289089259178, 0.6759437586233256, 0.6012423197982446, 0.5629467182846547, 0.622136136361923, 0.29252121960102934, 0.31798663218329404, 0.4703403904980381, 0.9029165349448645, 0.4258125976061354, 0.5572380001391759, 0.7590431300058365, 0.9318367053344245, 0.6183286678074587, 0.4859658405153258, 0.8221386679645752, 0.7467663971727245, 0.35675303627296007, 0.7765925131042173, 0.44881523875836254, 0.02039088794195293, 0.14623780268404363, 0.5836315731303486, 0.0941974899107127, 0.49660676561768846, 0.8449235500202664, 0.520352383126411, 0.6287320491209976, 0.36751143648650525, 0.7303220484800335, 0.5333598158497757, 0.3247449745600972, 0.58925980924814, 0.40135351422284327, 0.5312013470168597, 0.38698755534377843, 0.9317392551489083, 0.7264815347332074, 0.27179277726205353, 0.9857170409986775, 0.8067256228763005, 0.46976618347289106, 0.7349462220538535, 0.20233589463808888, 0.7885850027019035, 0.310098100074668, 0.3250551604391536, 0.49593184855111205, 0.694628961303495, 0.5100897078270877, 0.5271473098081968, 0.45721462705885535, 0.45532750493729973, 0.34861014053458933, 0.726656276227821, 0.34540953070927033, 0.247946422334494, 0.6312289015155659, 0.44987769148256657, 0.4959647075873057, 0.250543015306845, 0.24574637696391566, 0.6342670070287675, 0.39944638197118737, 0.837914040973982, 0.1660693878656781, 0.42177824366383654, 0.04104565686916872, 0.5553237009948474, 0.941587317907483, 0.27015523892119886, 0.19196779674897774, 0.6479832377384087, 0.778489603300563, 0.3531098483470566, 0.5111481301757681, 0.7449569675037289, 0.4095028261457119, 0.3231360054582679, 0.4339246515383297, 0.6078075024228244, 0.6883693095311176, 0.3160168501099905, 0.7906951343919837, 0.7660368443940355, 0.6446678509399832, 0.4102224397251756, 0.41980463235037446, 0.5569260814775386, 0.5131859023488629, 0.027744363658522675, 0.3712925150748685, 0.5207256372466723, 0.4795291524831834, 0.8604026622507406, 0.5175698796998495, 0.632859907812419, 0.7930854712505522, 0.8957218882333333, 0.19379266431323294, 0.7171304473508464, 0.4979409247221918, 0.6489405891066353, 0.36052706027310427, 0.957229434328514, 0.24044104942588312, 0.8979636356433991, 0.5173304659706541, 0.4454970918844575, 0.5379117989483184, 0.5163818722757663, 0.22145640156934654, 0.5523574297985409, 0.46076620939434004, 0.7501811656373918, 0.4736861565782853, 0.541244084502657, 0.4926298520997643, 0.5028579505367865, 0.47079570611187554, 0.41619713775131106, 0.296076029025504, 0.04625620376384981, 0.015537180442824816, 0.3384588937993416, 0.3762457484680687, 0.29409714727597785, 0.4765429769000038, 0.4692499711742135, 0.35379225058401353, 0.8870308200816588, 0.6046571073612872, 0.2240114555832442, 0.5749365941757908, 0.6248908305666352, 0.3423815199280514, 0.8418364687423407, 0.8327601737483995, 0.1994780073665745, 0.5434657667890019, 0.2236082279963349, 0.5049959159955754, 0.5038037067886608, 0.9639390874933331, 0.6360497781400754, 0.3889939972930753, 0.6932013450209213, 0.703748501852669, 0.17257882613731074, 0.799921731633396, 0.1654759137755506, 0.5459581308523647, 0.5734139847220918, 0.9018755665774927, 0.5005126738201409, 0.2659058775350155, 0.32262557595259445, 0.6599625832608857, 0.6153342830612962, 0.8287834857501506, 0.45822828936041315, 0.4084151644746945, 0.4823774689760306, 0.24663078171744202, 0.6775128897683317, 0.7025465702472321, 0.4091257437515323, 0.02022263758819648, 0.782061465436707, 0.4665648331926074, 0.4758667908629397, 0.08645134150272699, 0.5699251868184846, 0.16797452351651881, 0.8000529730342332, 0.8436828668222528, 0.16736684364386317, 0.36629701482062926, 0.6896009312828332, 0.42314063100508215, 0.8876577022271424, 0.36220447699293024, 0.19218155799749392, 0.23726468919507965, 0.35077046170317355, 0.3212879734569887, 0.6385693043735949, 0.48364438184062425, 0.9445730511639596, 0.5937454483352789, 0.736842513191957, 0.5016737650708661, 0.5281302045676972, 0.35850711167414995, 0.24599201853152586, 0.13664135751624823, 0.4074261616556272, 0.6363041845781419, 0.44049609852826405, 0.582658535310633, 0.6278773282326184, 0.3343773035774929, 0.7231894634262321, 0.4128339452831372, 0.5859595036176108, 0.6688464977764074, 0.18261831999884243, 0.6281113843701717, 0.5360315154722936, 0.34210365449240066, 0.4092764617210619, 0.7213088115149148, 0.5477060839373248, 0.5090013254304058, 0.3487030159590411, 0.6868847670506089, 0.4288726505389377, 0.5004298139731522, 0.8202094405015933, 0.8044077821952758, 0.6961764272219149, 0.4109171284919918, 0.6132686832923991, 0.41223075986169133, 0.05456244562226309, 0.5197806202636422, 0.4792496392135332, 0.27854625720010107, 0.6629556816304335, 0.7855895577401981, 0.19191192708672994, 0.1692745621528018, 0.4371703431793808, 0.34102630595897, 0.38510427309280015, 0.4539529509430237, 0.6947007301726749, 0.41384735391224514, 0.5160096867156708, 0.563228242621712, 0.3193621863095629, 0.4404388036756846, 0.48884837548912374, 0.6679361713045263, 0.1698268858591792, 0.016280617348944648, 0.9279596223081529, 0.10654355565247986, 0.3499950663301566, 0.261603615455582, 0.38201790594312024, 0.4027629652010982, 0.5911516897223035, 0.29190277023624234, 0.5728208060287352, 0.5522016054873023, 0.43958872007152955, 0.4125041320888497, 0.5778003775556949, 0.5238847846639838, 0.19805241873294982, 0.4190485207434319, 0.21378971445311806, 0.16082214143256446, 0.07819069293079872, 0.6178529850620018, 0.01198792790318981, 0.6786896890444696, 0.032194357137586116, 0.5443910944997116, 0.8914857409582495, 0.567687037384716, 0.9105847128654625, 0.0096677336197033, 0.7184942796529465, 0.7315684034580792, 0.47602651953750225, 0.428869893556071, 0.46805983646307336, 0.25056285808179823, 0.4214888989577786, 0.3745195005591968, 0.5628132383224093, 0.6146446868786086, 0.4881228786127498, 0.6945105226003323, 0.3906519322045869, 0.6906190150142629, 0.567494697907681, 0.5528014412087265, 0.6354447980277074, 0.501400183641877, 0.6025223479364961, 0.1300358337573963, 0.922699765610864, 0.4899014689295162, 0.3495619053843759, 0.32753781742603155, 0.29582035590123373, 0.5072694838159302, 0.6445877153459635, 0.9077795936973378, 0.5040228345525088, 0.5177078803324792, 0.5015262839611779, 0.41750985308837807, 0.372805046373639, 0.29117279575857863, 0.20976767279930525, 0.8527711901243986, 0.46356353125797295, 0.32932235401091525, 0.31395535085234, 0.4075188822889443, 0.5444260715974458, 0.23968433832768898, 0.4053140289982935, 0.5103509698185318, 0.7046059577763703, 0.21480545093653652, 0.8648299373508891, 0.6976229154210235, 0.6573026531318472, 0.4003947311042779, 0.6996530796093886, 0.7719578991939136, 0.6303162973930243, 0.9054600972560931, 0.4125391781130566, 0.8158600063950663, 0.7124922927605039, 0.5983777391673603, 0.37942460497859853, 0.5231560659780207, 0.3690514320531517, 0.9468397343561438, 0.8821382519899639, 0.4541125345620496, 0.5986110003246218, 0.43893361337192466, 0.40231473824635244, 0.5719514870710012, 0.06042765771812231, 0.5910982533246799, 0.5891571187480048, 0.5640105746101259, 0.5370629743900223, 0.39245729408431457, 0.6539598053728758, 0.2542039780712066, 0.5457566539667307, 0.47369742983537566, 0.4444210415587511, 0.49537586834474856, 0.35669151683287337, 0.2914253724190599, 0.5748844738552567, 0.7155496065137876, 0.4196815763406762, 0.20842139109165053, 0.2940996314737625, 0.334836804263548, 0.2728752066295924, 0.18449604457700708, 0.5500329497607799, 0.7835952393336332, 0.45587793077281885, 0.26984185893163626, 0.6018853190390281, 0.11819885517324857, 0.8932601105989666, 0.5606910899492921, 0.5242284637499328, 0.09986558986153465, 0.5753505580165057, 0.5867429640838412, 0.20113699365680016, 0.3172665886966355, 0.6494820592022111, 0.3726274800737289, 0.5393658709023179, 0.3181618281498858, 0.3953663721915519, 0.0980273198015984, 0.21206146256982555, 0.3026061343314454, 0.45315086133193555, 0.5251496009654513, 0.736967549067723, 0.2830962260405019, 0.6460152290686185, 0.5477029091812697, 0.7807499768178521, 0.45055771263880373, 0.7379252078381331, 0.3608199440138166, 0.5995004529310599, 0.40572738002030617, 0.393811987484783, 0.6422153556425441, 0.5326748525766538, 0.40431014441518254, 0.7100427253631089, 0.5202391273059653, 0.4186286100507188, 0.5795990892517314, 0.3082984130205343, 0.5978946631660058, 0.3610012524274291, 0.483101630801592, 0.30387298955733155, 0.6322432643322102, 0.289747341816917, 0.07826490268116404, 0.2680075474092707, 0.6024831183499557, 0.47735457352373356, 0.16381311472153498, 0.45215874772449743, 0.4519768662502959, 0.5459932387215498, 0.6622337845724312, 0.36084362246555457, 0.5292505073174711, 0.3449953371548189, 0.4073253767723759, 0.6372390750550221, 0.8513260262777287, 0.5216665980348004, 0.3155213117807163, 0.5396420044724173, 0.25929337838583616, 0.43105999623819236, 0.6025742320502446, 0.4577927269041948, 0.2708636163389623, 0.1967473228988031, 0.12415314127457938, 0.37773919972844194, 0.2894833493983587, 0.4892877585248546, 0.0714274291231668, 0.6164473708291756, 0.47514469921451435, 0.7232524331571017, 0.522579929217481, 0.15363822262922217, 0.19540709564199588, 0.14896884443010872, 0.3719877930337233, 0.3478916517963729, 0.7207314513076428, 0.4924421677068982, 0.8024456148741788, 0.4392060506397344, 0.2792179736056929, 0.46434242971078665, 0.2316432951729669, 0.5472499770731849, 0.34210263460431173, 0.26724860930700606, 0.9665186610452294, 0.3733112699317288, 0.5541803636337099, 0.31786640560091844, 0.825714459443952, 0.14360753111047692, 0.9337493556728012, 0.6302060411669601, 0.35442472944522185, 0.48378222499775275, 0.7892684024988736, 0.4285470997721169, 0.6501038697233497, 0.4289334856147038, 0.35025296331622147, 0.47147355976573135, 0.4803737925959157, 0.5007596290284055, 0.3237132809308283, 0.2155370438575857, 0.6250729237216233, 0.45833840244867674, 0.7651999407467838, 0.5956095283609557, 0.23315153756959017, 0.5493848787166051, 0.45847493636527703, 0.2495242323141026, 0.9679926819193856, 0.5206674262066457, 0.16467932765715187, 0.7100470384850404, 0.28374842158484875, 0.9199224074395347, 0.6291958698356267, 0.7309833856702994, 0.34024561859851327, 0.6616914045225746, 0.6445589907653315, 0.3735971036506599, 0.6340179375167977, 0.06719720774020682, 0.5310771801973286, 0.6944453824371567, 0.7831236572416995, 0.5613113427581278, 0.37278470621006826, 0.37851314850730616, 0.486889149418461, 0.32723409413238747, 0.65122047819695, 0.898712822265355, 0.35445455477105203, 0.45806480465882005, 0.6707472155717393, 0.30684853958146274, 0.414390829427684, 0.7671299303928112, 0.4219905150875947, 0.10821162574675358, 0.45642665572756996, 0.33931495807380035, 0.6888388646142339, 0.2147844675427874, 0.5361708979986305, 0.20088753976253237, 0.460776549680454, 0.5933731163974431, 0.534495934013441, 0.5434624694865332, 0.6233064649618304, 0.7398197653026877, 0.3810859571512707, 0.8805358383836872, 0.18305867973391493, 0.7419867051320927, 0.33945739729482693, 0.21237340145570593, 0.4358242070530347, 0.8059234847436207, 0.4046469937953343, 0.7662145267525913, 0.4362212410508768, 0.6371198930548618, 0.5819074810392704, 0.575779165809125, 0.3087344567222693, 0.3363603750323432, 0.449949944983816, 0.44665147837312524, 0.5427875913087791, 0.6198272000190558, 0.5153960100127966, 0.49711816577238166, 0.3954514068326228, 0.5711362574468473, 0.17426414497319315, 0.7597369185824476, 0.3673507089901372, 0.5167035016749235, 0.48307549504426067, 0.6630859465202382, 0.49555018120183786, 0.45527064360627134, 0.5054464088077532, 0.65767509543656, 0.7115721499332033, 0.5873728209791801, 0.5048456543709415, 0.25841564033880765, 0.5012878077826307, 0.625916290922735, 0.5799913180135066, 0.4558882935862672, 0.4470492473574792, 0.06570319169255429, 0.6459756194711432, 0.7478231105788004, 0.7675629671603409, 0.4574152195553528, 0.7972860903160681, 0.17523159128952615, 0.47291677801456977, 0.745768530855446, 0.7688711499265485, 0.3999613633723804, 0.403163280579333, 0.5222714060138501, 0.9137323413024676, 0.5827939937423322, 0.07217723737026543, 0.6084618163469842, 0.8579293837182383, 0.6515741859887743, 0.5225420935861199, 0.5688027300855607, 0.29224690703584233, 0.6293457787691924, 0.5614784657715173, 0.5726430347392859, 0.21820259287082472, 0.1821582607080186, 0.3291970314263847, 0.1762606279926887, 0.5700727243029795, 0.6325698952528486, 0.2819891222010493, 0.5718953921126246, 0.7158925272883194, 0.26558264491031036, 0.3780668284578627, 0.5239405990577055, 0.5509838378432832, 0.26740615725291383, 0.01316012019362417, 0.32408603542836645, 0.16140418058387473, 0.3212504797912132, 0.694933745971729, 0.5601007061635469, 0.175129249385931, 0.1708072761088042, 0.9159732801972043, 0.3525657312335247, 0.347293215972417, 0.6194166784900434, 0.5553168184104988, 0.5434550475171437, 0.5564349087104812, 0.6002945455095927, 0.9450877200899184, 0.33767136576727996, 0.8369124166337432, 0.7267191122944776, 0.626096205574105, 0.6699993328968211, 0.18313943980060066, 0.22500786810661189, 0.2620227689048026, 0.6474931354849629, 0.6550723242355206, 0.5281956990128097, 0.6746201086103889, 0.547109814266461, 0.08621004798884441, 0.20836775596651025, 0.035606398907535475, 0.7176332735659443, 0.6233579596289395, 0.8156837939651063, 0.5498218601128415, 0.33080905862848176, 0.5917699564586785, 0.6274387523548406, 0.08727325555069049, 0.8352421318665473, 0.3979595902701484, 0.6369814148523286, 0.706427147405704, 0.3475491729903465, 0.39200117614576213, 0.5963255254695785, 0.20569673550815926, 0.4621042345080395, 0.7239457350593755, 0.21247652871391515, 0.29557246317725316, 0.7028305678690361, 0.177328441765023, 0.3980052641650813, 0.5975639171752829, 0.440850522932578, 0.665612525229309, 0.24961412962553842, 0.14524517487402072, 0.3827758570797037, 0.3770220605548613, 0.445446204854655, 0.5043560231240348, 0.510288988147358, 0.8614282281126546, 0.5999485758125981, 0.7172728617471181, 0.5294771175223334, 0.6941243299930862, 0.154289482854381, 0.22411523584642945, 0.5326176551771871, 0.19148330079867898, 0.651310631465861, 0.3993599938735376, 0.24400124601055279, 0.6730325711743103, 0.24156497836324414, 0.8082209676173389, 0.5207354611904712, 0.4293302551827037, 0.969059768418206, 0.5671183419495329, 0.3374901137638785, 0.9339202512825414, 0.4205436844037773, 0.7306952894686194, 0.8355185131895213, 0.45972642472976266, 0.5334587564959589, 0.34296324894587027, 0.1703125252302125, 0.8446223291701305, 0.6420115267135021, 0.44709905944908923, 0.03554054645549759, 0.4927030004559588, 0.43615012554090704, 0.7764661189580631, 0.45175941111142476, 0.48964800308659, 0.24525663841885287, 0.3844944527888278, 0.49194029395692906, 0.33838277034577624, 0.7230846538302216, 0.681862446300167, 0.654244806328461, 0.2633198830151831, 0.4757810972445332, 0.5208545656369081, 0.4808196291369888, 0.5622368482249773, 0.5262313184607503, 0.5174392946934658, 0.7165026842736096, 0.6503654754835595, 0.2501091129565808, 0.7402579467754542, 0.3700634594938309, 0.5258165974030075, 0.7416327537246341, 0.5473774756241625, 0.43101162298746765, 0.6018190540938169, 0.4642984681930892, 0.4304816270265556, 0.5730028599079241, 0.651026670557675, 0.4930846863983347, 0.26226365724557854, 0.39033393083319173, 0.4513419908387168, 0.8080544189563066, 0.6375430729403053, 0.33306236345839657, 0.6045178172646024, 0.32951865986247375, 0.20004140810937848, 0.48609880368204184, 0.08503782324929549, 0.16818344903217414, 0.6731141672420697, 0.27762049871526906, 0.5017961362346253, 0.8343153727647517, 0.6303622803562701, 0.36162014289891825, 0.8072018607149603, 0.19217122994529168, 0.8245994735697066, 0.579058397551004, 0.8743596114889114, 0.9766758006349776, 0.4590875641064276, 0.5044220547143436, 0.45597188923609905, 0.23343246647249766, 0.87251611316026, 0.7475352807627087, 0.5666635963886487, 0.57648448527325, 0.7634004750823513, 0.9518775349879336, 0.6743916389547115, 0.5528327826175456, 0.8037083323047662, 0.3959723139139977, 0.7253106532863709, 0.24292724358035983, 0.1443403887391591, 0.5881246462236579, 0.47790926867702255, 0.9163014023030882, 0.6430910348903833, 0.5376685349741362, 0.5242514761098052, 0.5453893952070071, 0.6215175138599488, 0.08707839948863627, 0.2865480647845686, 0.23287103622435362, 0.46790107319691066, 0.7973090416515756, 0.1643372081068059, 0.7194055465486981, 0.5038639052335405, 0.613264072275087, 0.6057791902501796, 0.8612261095715561, 0.4736239660123119, 0.2561039702698657, 0.4726197345338949, 0.4501070501756383, 0.3410359613043572, 0.5527155231997852, 0.37066436092019356, 0.5453685754378637, 0.4906621822402281, 0.4864780929607614, 0.45775177294096947, 0.5104875181786382, 0.8025597910054538, 0.8798808540204024, 0.9277977376517099, 0.40801143538525625, 0.2486758159333765, 0.6934197081649698, 0.14511809293414235, 0.5041803354166401, 0.5417712117942188, 0.20266923550014682, 0.4964573361308741, 0.34544596887578416, 0.4203459778580703, 0.30190609679459723, 0.49198630462699006, 0.46104760449401927, 0.7183564815948063, 0.4626998097573932, 0.4599934684127989, 0.2933088812263781, 0.14493573346383304, 0.8156358871620818, 0.6297909790608269, 0.3150620639790892, 0.5368704222089401, 0.4072660838105938, 0.6185951588275378, 0.5310040009463505, 0.5915593025435829, 0.6632464333290506, 0.419575137645667, 0.14870892143313286, 0.2984469899464349, 0.4444243748885605, 0.22448176797134306, 0.35239043071453546, 0.5780676292689747, 0.3413202898415908, 0.6105242867789975, 0.5535658609856117, 0.13117737993717943, 0.6353382580057327, 0.8357303290689959, 0.628638343131533, 0.28306291970301284, 0.48501658351243426, 0.39031007204435275, 0.3361338512248727, 0.444734692684339, 0.6043280308268855, 0.683514163460307, 0.13460171706050444, 0.5353479231846423, 0.6261914995544859, 0.8499614831305465, 0.8518933087843203, 0.4057864552191831, 0.4896193895883423, 0.5625387155317679, 0.3414912794186915, 0.3021295604630806, 0.2474971496959839, 0.647378535140164, 0.5191875508123657, 0.23095685969353585, 0.9180935048891532, 0.024588503860336175, 0.7834396317101007, 0.1808030862054627, 0.5122772785362693, 0.6144558604713185, 0.7386278533230812, 0.447579604627365, 0.7961895344123902, 0.7489745043897447, 0.7738168298020296, 0.6658742642505007, 0.7104858322593688, 0.6032655570581669, 0.5242380678347269, 0.5273698043548363, 0.1889255893048154, 0.7818571622911155, 0.5340294914126005, 0.6810744216192339, 0.7754938415498098, 0.9094047684721731, 0.36998040185013636, 0.3113576965133448, 0.17691434473355694, 0.3680058838697472, 0.6028811774244563, 0.6278449039098375, 0.2124606626302683, 0.32241836745960534, 0.392921209384306, 0.6660046057765899, 0.40505189226671123, 0.3162952830488892, 0.3343754151615148, 0.505175770257976, 0.7332073103035026, 0.3771153713185448, 0.3661220728975123, 0.4599003526680718, 0.6341645571620951, 0.39814608700209375, 0.3522335907328271, 0.3921871386402877, 0.1253580915094752, 0.6277531398352272, 0.42288465870994785, 0.44456291244443497, 0.5377658254386433, 0.3187868469967386, 0.2895311673439416, 0.23460171187261425, 0.5757486185165968, 0.4327234802598017, 0.661679555936352, 0.4210427599894081, 0.5194152744746425, 0.8070708940209167, 0.2738486515953522, 0.5303035014806843, 0.21868457492157645, 0.41267049649874976, 0.2361902130388539, 0.34343823450070776, 0.26029249755610007, 0.17186458471934546, 0.4345365193113254, 0.6132457904564518, 0.27119759197897897, 0.5986762159739701, 0.41305460390666743, 0.7047140209026085, 0.4814476832573536, 0.26282005851002255, 0.33298000548327256, 0.11252021835055909, 0.3928348418071548, 0.19043323727510253, 0.5889748697338902, 0.5874441020660998, 0.6538764009324176, 0.42514242583387485, 0.8549956281624875, 0.2413749392119875, 0.3980365818192061, 0.23998009920822527, 0.1742888290371441, 0.5388589164493729]
# List =     [0.684563347765009, 0.4199992981750746, 0.6908776154114055, 0.29767498965984146, 0.5258412643676913, 0.40402933255639056, 0.36428069206622793, 0.474524295153411, 0.2900734781142987, 0.5190201304478348, 0.2471157584201868, 0.21841361225711114, 0.20343887899653323, 0.568407143865214, 0.6470500572679447, 0.36608652136084563, 0.7041735633261236, 0.7222242172239358, 0.5398996601853638, 0.48146646364877627, 0.47414745474271625, 0.36183136678039984, 0.7335487095308788, 0.43629088091759016, 0.5915266071041569, 0.22336591649673992, 0.456293565499024, 0.3019300097930518, 0.2334678449729336, 0.4862515575638887, 0.5296148300791108, 0.3212738724140398, 0.5870324146843755, 0.45477358658408673, 0.35910950420063115, 0.3543733161247574, 0.40166051465667746, 0.25865458658594376, 0.19640552513359688, 0.19510320944671294, 0.22863749936509392, 0.5150866216920309, 0.7825342984228391, 0.6754038761351137, 0.5664354332668669, 0.5140072543452565, 0.22501124524196403, 0.40204183779970454, 0.20090437394351784, 0.25379256859012234, 0.4124010089444, 0.3666129832129682, 0.5043567438881564, 0.614016637424595, 0.5080749505460588, 0.4854888583339643, 0.7771702323995004, 0.6399891425570366, 0.3010181860683755, 0.9376798443556116, 0.8140045440056788, 0.538040454771439, 0.9892554121202211, 0.2263356286503614, 0.7869960130345808, 0.34282983965911457, 0.603417142517463, 0.3653361326748543, 0.2863439754092974, 0.8321579966442479, 0.6520767985586379, 0.5880777166371062, 0.5593580128025966, 0.6221762123975895, 0.46633757587724134, 0.700179753989615, 0.4892052374445353, 0.6738414957644389, 0.35699725973433627, 0.5365718906033988, 0.42249364898761566, 0.13036914428908114, 0.4015063431415258, 0.4529392859963238, 0.5657794665039954, 0.6362729314165398, 0.5441283075368498, 0.4672583501544118, 0.4420358874264256, 0.6087635586723597, 0.7210199338630013, 0.3806810318595075, 0.3807622002394755, 0.5089563173162058, 0.49096879893147316, 0.22695982374838608, 0.6051447631879413, 0.6879858156659213, 0.4164072462388581, 0.6563316191035153, 0.7173244149011124, 0.5200995183546229, 0.6173180632747823, 0.5320063475766506, 0.4800701938492031, 0.4815576329527407, 0.4627777512790206, 0.31409808398128836, 0.4248971589240472, 0.32470468057677615, 0.5885384359773174, 0.45162618066956567, 0.2681802495810599, 0.4882443831024065, 0.6343561391562134, 0.20132227392048513, 0.639378987080333, 0.2477646048337233, 0.3064624869296003, 0.40343641444519734, 0.8150681096085957, 0.5229933189439537, 0.4098006437684962, 0.39805702118510733, 0.650277453904626, 0.6287286772613817, 0.24067007224373355, 0.48454567148047134, 0.47772252131296833, 0.3395870119258366, 0.5818468903678328, 0.3001335287561201, 0.23541989082710568, 0.8130410768327834, 0.6926470047803949, 0.6655984435535605, 0.6554787356785399, 0.5773658052035128, 0.6436190526883863, 0.38790255786414535, 0.6605519654957256, 0.9093103573058581, 0.6754271299783242, 0.4588829992000983, 0.19811106749825802, 0.4252188605937681, 0.7281097924625833, 0.36391772343486506, 0.7264380319564695, 0.24891664021225768, 0.6401001374760371, 0.16277126581580487, 0.4969794257863263, 0.37956191547646234, 0.18145105666133138, 0.4182364797098803, 0.614126521579843, 0.452163866506145, 0.5110020627882917, 0.6297944200704663, 0.47348203022342844, 0.6827384907281064, 0.19074597963883078, 0.27639288474971213, 0.28693187481394233, 0.17349508404865704, 0.4230451207884053, 0.48292275895671566, 0.43451924146655196, 0.45910434066982, 0.33332222246641435, 0.6699081588705784, 0.41938722248717814, 0.61396716981134, 0.7639190905098456, 0.2424800602302457, 0.44451067028456376, 0.19296981479659653, 0.28872011468414616, 0.3729746851217557, 0.6943525271195787, 0.36684346934568696, 0.6477034726534421, 0.3298155010602408, 0.4891460014095194, 0.6436715673811825, 0.5485215447950734, 0.5136373097941479, 0.4155389307748974, 0.5942431192361506, 0.5310313014178569, 0.8646463661829029, 0.6726607921488691, 0.7133717372211171, 0.8098210238389898, 0.23097278112607422, 0.4412462767165069, 0.397105665462814, 0.2763290031360997, 0.6423624861732218, 0.393639851210532, 0.5292492545303173, 0.5383568337729617, 0.610213478619134, 0.5276543487040699, 0.5622522640798905, 0.34090082018654644, 0.4699766362849527, 0.4130864485180419, 0.6915234282585221, 0.8359052482992941, 0.44502229834646584, 0.37453810498467843, 0.15469291704138666, 0.5587407095749902, 0.39084941430362546, 0.38640353005889194, 0.22705095936680492, 0.6068784737835725, 0.34573532613048075, 0.46147805662653213, 0.3571558612175451, 0.5655902426320685, 0.524929073844463, 0.664589177671479, 0.14199028580565237, 0.4051721950018141, 0.5440531840752117, 0.6267448706223108, 0.7893602874389989, 0.619205205829268, 0.29052371823995404, 0.5560454116095835, 0.45641035079873193, 0.5953712049824045, 0.526898053693847, 0.4728820544671956, 0.25055201474536837, 0.23752716013019348, 0.8568372721626789, 0.30076535914277946, 0.6058822725696135, 0.47827439845707537, 0.251585969678382, 0.6101746726469901, 0.3869714472196638, 0.3624762294305699, 0.48184697331791376, 0.8932297027059859, 0.40837677999784056, 0.42767822233500014, 0.6328601547417168, 0.44207302948859767, 0.28416110830354746, 0.48024226111562934, 0.4177548575130332, 0.7916526530046182, 0.5312157906797729, 0.45324498471158237, 0.5401564022831702, 0.6847960267401709, 0.3295629308569554, 0.9712299992210864, 0.7725870533945851, 0.5408503790099755, 0.5346086524770279, 0.5231954229214948, 0.5945438456879205, 0.4953402664685682, 0.5651564053676512, 0.4414163351390717, 0.5739866803970948, 0.5293622913179108, 0.5471671959095363, 0.7777542136760733, 0.6146348574838856, 0.701147906223043, 0.4575987066101442, 0.3078172949456734, 0.49322264674655475, 0.4373719795958933, 0.541399733335734, 0.41465810633252687, 0.4517871297133595, 0.2478056895137573, 0.4289118206187642, 0.8620061755597352, 0.4352525224569489, 0.6492548569027871, 0.34460026780855724, 0.5266545178582785, 0.717782952927831, 0.7394175723228008, 0.45208352845095084, 0.6441055058840853, 0.5324085730239473, 0.8801939902982008, 0.6161864204934482, 0.5474217334549294, 0.32465457442887985, 0.360513217810925, 0.5985606066848652, 0.5759936160188749, 0.5969756207780821, 0.818874659798971, 0.33087232390018295, 0.3525457338784973, 0.6317397520842873, 0.567755617291031, 0.40449190423114334, 0.7313279048853467, 0.37678055495540275, 0.6766846256384595, 0.6969740536073973, 0.3282836537480479, 0.14846128414054893, 0.25886335737726934, 0.30035301157829053, 0.527643994118377, 0.18942174265323136, 0.8776867911524919, 0.603340562365744, 0.8491409930746012, 0.4094587402847917, 0.4266927617674522, 0.3892746744549167, 0.2695447326644742, 0.4532646709520054, 0.2587904609587316, 0.5688736886760213, 0.6830247300642143, 0.30783210359817975, 0.6110004278149602, 0.39920650014760256, 0.7170212370585098, 0.4479157568084645, 0.6447099851957524, 0.5012587684579937, 0.4607807060779078, 0.6607581078321081, 0.5630003201049758, 0.4966062284852182, 0.5278829248784527, 0.37680126709591377, 0.5576686989599545, 0.2682974553474209, 0.6689765477059434, 0.5504978137667735, 0.2021828039740975, 0.6616782520162832, 0.6265009461117442, 0.7084989065221754, 0.4289416894679354, 0.44527108440102303, 0.8754075398521073, 0.09421443407908664, 0.7019711384556246, 0.6329764927899711, 0.43549695605944166, 0.3308579762842694, 0.5756550976694301, 0.16890706947675094, 0.3364289153112242, 0.49519075181601113, 0.5040060152952348, 0.4131007299192868, 0.18327192249896734, 0.7543510212173574, 0.3461123015767802, 0.34936817623042815, 0.3175782835929267, 0.2667911203731744, 0.5489398929689766, 0.5637267196796752, 0.6767538544381276, 0.07964603676535897, 0.22809565584947233, 0.7230082288797506, 0.4555281209997922, 0.47786594493723444, 0.2882552915456501, 0.49375218066228604, 0.20446853379995045, 0.6205029849038259, 0.4054647568930131, 0.799351326946829, 0.28850107313648077, 0.33235244276145925, 0.6290271338671803, 0.4227361144018925, 0.5295046030969429, 0.75001900223433, 0.2745826129545576, 0.5988646958850337, 0.7063788672392639, 0.5409757286913186, 0.589861327627997, 0.06725593226405752, 0.6296431676623835, 0.5013391844605029, 0.4376304676120683, 0.578124138109243, 0.6885274057855219, 0.3960866836458009, 0.4064575584850799, 0.6476617078316251, 0.4659814064957712, 0.7135926232119724, 0.4833666201606565, 0.6208005509346808, 0.42226519082755076, 0.5118467833441059, 0.5920432750175666, 0.45789771599092516, 0.9147388323538452, 0.5546320983788062, 0.6532356615416754, 0.43612205849332775, 0.28162429483460627, 0.4581705509604903, 0.47173768617331313, 0.46066777218991717, 0.6061779080120716, 0.5840074842627754, 0.40418997089119013, 0.2511798088251095, 0.6539459243464649, 0.34701729611860627, 0.3105030989439743, 0.7894238677541963, 0.4175628540833819, 0.7032540552228809, 0.5056701207106178, 0.7550306067087122, 0.30881830280731737, 0.6776063712081792, 0.6419146697176252, 0.45486157752437506, 0.24682311947004337, 0.6961635682062024, 0.7659818120183103, 0.6157720960882882, 0.40107756938789096, 0.25403247254838424, 0.7592693957029526, 0.6501791634650054, 0.5896824297197083, 0.46924680366423377, 0.5735849950014335, 0.6228095231384843, 0.4247755032502581, 0.23797538885084796, 0.6492977730088961, 0.46699525301081835, 0.6414055836660566, 0.49901430334503666, 0.6194906295894024, 0.21769110588945073, 0.3208180435916477, 0.30574873710112094, 0.4063916472852669, 0.325246376647662, 0.6329819795017703, 0.27409400273818674, 0.32983827281254424, 0.5575615305397899, 0.7044352992721459, 0.8418414514100725, 0.5276396486676384, 0.5667557304827242, 0.5503006469685058, 0.48878590758734136, 0.7740674350411411, 0.7424314403833989, 0.5795580141874339, 0.8035134898774339, 0.3812272612179305, 0.6255571306509256, 0.7037552574318071, 0.5951907646827118, 0.8998079959494457, 0.5017572758113986, 0.30473752736988424, 0.4929937483663454, 0.26840318544539254, 0.1895622097941428, 0.448621923004336, 0.5986986404522221, 0.72695867435707, 0.42806964008452436, 0.4803147677872233, 0.07790518167584037, 0.5724709176396436, 0.351834215170549, 0.6349095706993333, 0.7787327140661897, 0.336674606687578, 0.46193016672234327, 0.6027913901991208, 0.315513110508875, 0.32482019924849703, 0.4320276564074718, 0.6774551328676197, 0.46287263054941835, 0.3255148634605016, 0.6582903545723278, 0.1858081098375714, 0.8323191727146297, 0.680610118751972, 0.36619181504420484, 0.6488125439928484, 0.5897575429848456, 0.5759661282987225, 0.3169948067759941, 0.31049593313685986, 0.41975546406169256, 0.5284128167895451, 0.6946741843945987, 0.5743279192555073, 0.48979994378672953, 0.46520485771138487, 0.6965288073735982, 0.6595138588044138, 0.4158512809997764, 0.5886758101030218, 0.5445520225679867, 0.3054973416618367, 0.539511911933504, 0.39562842917341606, 0.39568721019135045, 0.3964592856784231, 0.40852576874010726, 0.38877534664730296, 0.5634518430285649, 0.29650303354710816, 0.3613230836371858, 0.17908082211970794, 0.07036624225828814, 0.4985844754221413, 0.6495503934713527, 0.5038812393225766, 0.5901325948156432, 0.6514294971605479, 0.720457309388916, 0.7571477164023421, 0.714096192735368, 0.5742984415429906, 0.32069322907872677, 0.4736396741387691, 0.5973880375681504, 0.6267740672551186, 0.8034550128520486, 0.7222117577311267, 0.4676154746044139, 0.5394575702159237, 0.455730711118501, 0.8442202003319748, 0.6190816177103392, 0.3354411393144821, 0.6847692537909461, 0.5301038418396711, 0.2877744973219148, 0.5543933794436996, 0.5856723930463215, 0.5429767670321127, 0.4903134587808598, 0.33669560684215116, 0.7167219735955148, 0.3964387672266689, 0.3178548297553714, 0.495700433338259, 0.6470924246813639, 0.47595065055136226, 0.5111574113464585, 0.5976020838147247, 0.7329619169838054, 0.4229855921790079, 0.7333257857997605, 0.7931244671407134, 0.44847398867894406, 0.6835207781552933, 0.622136360132951, 0.45698747268030404, 0.13376232779504293, 0.941121151512005, 0.6673115325266372, 0.6329322780905137, 0.5111867007236424, 0.42374294021319653, 0.523510826457918, 0.36891808817719207, 0.4339786444329172, 0.8200955196493771, 0.7266992079016642, 0.5868363946185571, 0.5290618955902902, 0.8537603017508857, 0.747611064138864, 0.34918357764029784, 0.6650589740241826, 0.07745777379794244, 0.45160410988134797, 0.23477315811283805, 0.5226404095872293, 0.3261650201027011, 0.2127431367486139, 0.4286761847351211, 0.4407819588290841, 0.7301763705266608, 0.47660699379098215, 0.36822637024378635, 0.47128729288932825, 0.5464683106311367, 0.40836806232025624, 0.7040182493781382, 0.6336270569075396, 0.5329117594083698, 0.5213556615642064, 0.21351318607355751, 0.7226866312948153, 0.46169914630932546, 0.7581686833949184, 0.6851176420839729, 0.5860392423605356, 0.4387521809199643, 0.5693740761625284, 0.8130957289712923, 0.6336830607144058, 0.4938059590316675, 0.712703752615285, 0.7931993470318742, 0.11974204761597307, 0.3698563886892048, 0.6470120965703308, 0.5074080714821524, 0.6588038470509504, 0.6850805300094163, 0.8616669967276025, 0.776423278276678, 0.34433927887491966, 0.38415949052556375, 0.12901570378343402, 0.40870141052039366, 0.360280923089798, 0.004959091783756253, 0.6215196851815098, 0.3307087578105776, 0.5372312425570693, 0.6847248224428533, 0.5324137917386137, 0.6608235375545577, 0.3682020950838346, 0.5992817725102392, 0.4894236690216232, 0.11617107759049833, 0.5191154252149959, 0.42497394394690857, 0.47069244545462724, 0.6612110535445834, 0.45896121533916323, 0.4374327416763035, 0.43758834487362386, 0.6293370986471807, 0.5921496983767424, 0.41978733778519134, 0.2779411095521857, 0.21023826780638905, 0.7496428457937813, 0.587855137997714, 0.44545888706851616, 0.5071507985393293, 0.4383078754616644, 0.4868275392037599, 0.3438096792478409, 0.6797042862549947, 0.5241887670951959, 0.5445289791303395, 0.3219222623925695, 0.38180039940154664, 0.5502689658279513, 0.41877979153049416, 0.28743445224283903, 0.4990693341481714, 0.538885722739976, 0.2683733606031045, 0.6305991466864511, 0.7967475352325534, 0.19696472130714165, 0.5510940552469745, 0.04608890398896193, 0.6188054995505992, 0.4422981693044237, 0.41354990263157976, 0.5230599192398645, 0.4128959902068686, 0.5740448138655895, 0.5645467335685964, 0.37604541230237154, 0.4476640358939713, 0.8449039326994918, 0.23077587806796845, 0.6118236790122897, 0.8908435452740586, 0.8837291348067582, 0.4157192324324191, 0.47454416227022517, 0.5170825599264658, 0.6305913314123995, 0.504218235699826, 0.6025258086355584, 0.27177438850871904, 0.5770012435187732, 0.47007168571442026, 0.41034704082426254, 0.6992311502694897, 0.5160747485079475, 0.40747245587985065, 0.45123666174406973, 0.4909020084436269, 0.6090571340856996, 0.5412325150927316, 0.8515679276340009, 0.26162756099578977, 0.7017780177219615, 0.6602746260650364, 0.3130407560023064, 0.5818668787786587, 0.22341787360296173, 0.3401712656753173, 0.5160329029432442, 0.4521778072257817, 0.3702822344629191, 0.7186289915402047, 0.6388412499153452, 0.4723998059797041, 0.416267367173241, 0.6793377536091358, 0.4596563132776942, 0.5422669495179911, 0.5274260910449395, 0.6761289215795017, 0.5960981954620346, 0.3334882546491075, 0.28864746383764606, 0.5690020507829464, 0.11936519960156866, 0.34828682988409015, 0.6783179856497799, 0.5788798325046716, 0.632914478067772, 0.2129504470241716, 0.5238903330975695, 0.6945914991676161, 0.6160526772916872, 0.5389108241799148, 0.4631106566489693, 0.3888212533148379, 0.5083326297825266, 0.8083541716911924, 0.8504950982189701, 0.7290152147361766, 0.8651034232547072, 0.5075948112067329, 0.5492774300658855, 0.6456804483255869, 0.45927597575868384, 0.5643423798489938, 0.5456576130767731, 0.3447099704278883, 0.5403007738424207, 0.3732414439370711, 0.661733403461231, 0.019892018578418103, 0.6448568864806178, 0.34557250203272305, 0.4705378138193881, 0.46564588988855704, 0.3384280299890027, 0.7749450073606309, 0.5820474856107278, 0.7217877242726392, 0.5657129231110023, 0.2557558146005012, 0.6030970211556199, 0.5742083180584737, 0.22072238646835107, 0.14929685287804473, 0.4449550233631313, 0.5083187015512975, 0.8891762104144625, 0.5905833414009126, 0.2606754107947058, 0.6399954405079262, 0.4421636400209618, 0.4084999882036048, 0.40728915804160437, 0.3411020374517232, 0.575289735201191, 0.3351928793992969, 0.33911847792543487, 0.46536610133761164, 0.35769988826480054, 0.43000788029430864, 0.2295129648475534, 0.7197312608044836, 0.4180966076100314, 0.6771809330623197, 0.6420744269913575, 0.5640222223741111, 0.33739966483010725, 0.7186581706091538, 0.6413676350507969, 0.48927418973197434, 0.2791497597151371, 0.799231357043862, 0.6733063345263801, 0.5467488068553132, 0.4789653077501741, 0.6325049613469266, 0.20068860363013646, 0.9016826971389753, 0.7706083542268767, 0.3830281874802738, 0.28110001975488286, 0.2714574714867364, 0.5703085973888694, 0.6398888398721245, 0.7542928397973591, 0.28895041532935084, 0.04956220863622757, 0.4797612380076977, 0.1163983482408304, 0.7966086950839322, 0.4006342845030421, 0.6042214695822329, 0.5049678613915852, 0.3329679513581294, 0.8058114943033419, 0.8095439038151024, 0.34484184134159723, 0.609355255748418, 0.60872594595109, 0.37736656955426484, 0.493193175142139, 0.5418116644823019, 0.40588987443736163, 0.9846960806436559, 0.5318646358978846, 0.3024890049682404, 0.4218658435990741, 0.3829845932619413, 0.722780261967532, 0.5896673493279612, 0.3854887507373775, 0.4889641047996977, 0.3928945508994129, 0.7172400274546883, 0.578496150747326, 0.5721233609874454, 0.5707507389720765, 0.5841780987889611, 0.3277131492007561, 0.39088766559511623, 0.3899858146312427, 0.6250930306982544, 0.3324673601881114, 0.6413849038769445, 0.5638301504444546, 0.5703208526224453, 0.1580700046329031, 0.513192433555855, 0.22330321528359254, 0.5592797786654453, 0.250098310930038, 0.20294423597678307, 0.7435061909094374, 0.5232217253900499, 0.39593445153086937, 0.6828868986571859, 0.5085139882840586, 0.3712722066755037, 0.6292862820722095, 0.5495201708474413, 0.5035303960819046, 0.22193012991481587, 0.6218027080559524, 0.8057881097301869, 0.3858955571256123, 0.4371342420789491, 0.5993710628731698, 0.6569484883167164, 0.6660711666545533, 0.37683476408241484, 0.09490535694414881, 0.08815836064105137, 0.22649937731351072, 0.4448795194356726, 0.7578610613384372, 0.6674109893110276, 0.39128901922253245, 0.5908563504457778, 0.6018277311984591, 0.6498290305950667, 0.5414363246660355, 0.5581781765673223, 0.3478707270043455, 0.6794592071291509, 0.5039362786654639, 0.5109601096801643, 0.47904150037476795, 0.580236224785827, 0.5667986875196915, 0.5961903481769357, 0.4181946857468233, 0.3394014768827695, 0.4757819826705428, 0.6836967894284057, 0.4965864388700519, 0.3163011345308837, 0.7639189168038515, 0.4615783822596655, 0.6217212948920378, 0.6150282694144317, 0.5614839020886432, 0.2580535587878115, 0.1335920802900824, 0.5262326531326535, 0.47624118487150724, 0.5851326494965094, 0.5795653991606605, 0.4969149187031046, 0.7412345738549739, 0.7807922621381552, 0.8207591880150562, 0.5319947886066707, 0.27844901300920055, 0.2471728900298829, 0.4045197677294566, 0.6639056488239774, 0.6916768670961091, 0.7117654323216095, 0.3682917343380182, 0.7028391373620485, 0.22220580141397628, 0.5843259650043425, 0.5606916224148645, 0.6635881384384554, 0.6013154711449298, 0.2817512606918864, 0.21184261251720987, 0.49166253483170036, 0.47976607000554455, 0.8208903305221867, 0.662421397287871, 0.6405869729108533, 0.738281429742352, 0.5485990262159486, 0.2381955027868229, 0.4501765560548573, 0.6894320034737549, 0.5183566544589261, 0.5392407035259917, 0.36989243848238645, 0.33533778533239805, 0.18291200182430395, 0.5639200984173545, 0.2610869327631743, 0.7363165946687256, 0.5513995047991896, 0.6142454612383316, 0.058850034695633546, 0.27967406960423524, 0.4829359250724405, 0.6832878506365665, 0.6512749622348035, 0.5207351751416204, 0.15231326849566307, 0.49307886882485696, 0.7125083912059343, 0.4739606208783201, 0.5214481891842292, 0.5510222662538714, 0.19254506366428586, 0.37923363319366143, 0.6902144973304905, 0.6443574989489261, 0.3155521659577823, 0.7813828183957915, 0.6045576516864796, 0.47573473276231604, 0.32565657591998737, 0.4349054729519861, 0.5260953560013476, 0.795832418468899, 0.7459523100841289, 0.6668903758376106, 0.3585970359298527, 0.39753881716519124, 0.8816361613320736, 0.6152552942828061]
# List =     [0.25629196640094015, 0.780975335650484, 0.4345280306324438, 0.3796439558908895, 0.38273980190248147, 0.13489017038751455, 0.20377534097518918, 0.6445165468062979, 0.35964082155460214, 0.4974053809376548, 0.21703504474638802, 0.8232310627942667, 0.36264871946657823, 0.5549345028059763, 0.823465171659573, 0.6024586795109779, 0.4168848298243604, 0.1574062887278611, 0.6879485581265314, 0.4342469556455655, 0.1905870841857093, 0.7172449810910443, 0.22615911932126576, 0.313141552758346, 0.4869412502126636, 0.6874678880604179, 0.2904181157949174, 0.4057073547016299, 0.6987666837362628, 0.48013599342178065, 0.4809759002618674, 0.587856207817033, 0.590653903094097, 0.2052611023677614, 0.22661557999659826, 0.36433497481625854, 0.691095770309486, 0.384861132230096, 0.8379812778370512, 0.5678545916757931, 0.21188216214267164, 0.39863827476989827, 0.3440491345308143, 0.6473274582036853, 0.36553155959983674, 0.3934805102515214, 0.5530502413380658, 0.5668787339110941, 0.34206091275589595, 0.5360011307700627, 0.7724736044651717, 0.0720227878879035, 0.849968134537832, 0.06693321955859677, 0.662945525666127, 0.35017499511284844, 0.5818904120588648, 0.29061903895820834, 0.8967682812055333, 0.7466778899049749, 0.7514105008993663, 0.06099739331702908, 0.45628566466351517, 0.7099912060210319, 0.3350977653212087, 0.5117452949227038, 0.16949948472861226, 0.774923693611479, 0.9636594307004, 0.28915633439543187, 0.4788243286797705, 0.4663253198331152, 0.2246920220064133, 0.503519996814102, 0.42467811493767244, 0.525696498690157, 0.12561703359838272, 0.4690362299891183, 0.876203496371244, 0.7103333093247333, 0.11320390129980418, 0.2437934911196229, 0.6248828091926574, 0.11469695245452655, 0.7075574505775729, 0.5762346430877924, 0.26098063640265334, 0.5690320564224177, 0.4601186565784392, 0.5229336835220391, 0.6085165302141771, 0.8310027756077956, 0.6525751768382597, 0.4751725082804546, 0.38975386848401017, 0.8534168399059208, 0.3496873998111309, 0.10693711715719373, 0.8746216896517884, 0.4233051711946939, 0.3991244769641469, 0.2006717454039405, 0.5184169964449865, 0.42392912299644386, 0.33254779622964303, 0.25372907122548705, 0.5784411296156838, 0.30252719049961674, 0.46682715723710033, 0.741306414338845, 0.7594482434231999, 0.4851798730933391, 0.508057118836936, 0.6515916859753258, 0.9151001034560851, 0.5219087653719988, 0.7647844810351621, 0.6514105133495927, 0.5310661376410583, 0.6932112248159722, 0.5571908799738432, 0.3363777947694838, 0.14610160782016496, 0.35558452094017523, 0.3524924515250589, 0.3828333595518638, 0.5745365796542409, 0.42817638672355035, 0.24064088960495572, 0.5835439563459529, 0.8764139307627836, 0.5186578218435532, 0.5569160405502797, 0.7575546057159701, 0.18010840046392962, 0.8508214466710511, 0.34774722745011966, 0.3251700295630488, 0.4413299337027246, 0.7591141208135369, 0.2253989896769037, 0.5918495658493235, 0.3205689883168208, 0.4887048379914031, 0.6492819090300282, 0.28116599907024764, 0.7477304194023, 0.5053059924477551, 0.3134232688008639, 0.7201946720596021, 0.4726697752339325, 0.8996807009980804, 0.4805165553030597, 0.5605297243448095, 0.8315362001413467, 0.5093688274901572, 0.9729578904584923, 0.6648282442591322, 0.5736421935096125, 0.646815204706777, 0.7124121275692513, 0.9152064990361407, 0.8686625452136185, 0.35867039233458486, 0.6270608453087438, 0.7312900496121153, 0.3711224534241113, 0.737212351084207, 0.6405416689260736, 0.46939613555345105, 0.6878027176542314, 0.7625637040695987, 0.597350104659931, 0.42963202198159345, 0.5505816267509136, 0.22252603116887248, 0.9055159624371554, 0.5252992348865899, 0.8360814502300318, 0.12049593113128954, 0.49018225017384565, 0.3160442704603714, 0.4510870719541572, 0.7631176805179135, 0.30490869005551, 0.7067137860826691, 0.33416896436954596, 0.6237801986233482, 0.5295726494599994, 0.49160140837991084, 0.4248928595923195, 0.38714907407405935, 0.5590462888568821, 0.6445079564864933, 0.53683233221796, 0.45432844305678066, 0.6762161461641678, 0.5070314974366389, 0.6863042613341003, 0.5241322047685685, 0.7127445816451938, 0.47241969983577176, 0.4068102146067964, 0.6935963004322965, 0.5886383075581451, 0.7108795465515401, 0.39377196341753307, 0.31744385391730306, 0.38310906319717364, 0.47841577774375466, 0.4494370281083531, 0.7610930178541029, 0.6017429416429516, 0.9101740643563236, 0.48475750333743123, 0.6648197177035088, 0.5357159033333319, 0.45801027186263843, 0.2833164985794405, 0.8242466568936195, 0.6775812883532801, 0.31941702082257206, 0.46116784895507223, 0.37450107579962943, 0.570733343497578, 0.6977680947068141, 0.6023955067387653, 0.533248361065082, 0.5397705796744376, 0.7117710139503717, 0.3293155199579322, 0.48148139003600526, 0.6900709623129656, 0.30400549182184733, 0.4274709012298473, 0.4958167649768515, 0.43780807853045145, 0.8465834515309749, 0.5265867655463016, 0.48453494686512855, 0.3829426042728531, 0.6110709930697997, 0.6867226498085207, 0.6538625871194765, 0.8651188360600683, 0.7434921398581239, 0.3047564664585273, 0.545269540502086, 0.36741640425961053, 0.47945703625987024, 0.22873390516712144, 0.11695362107361651, 0.45345994361555986, 0.7181417127577934, 0.5897843508531313, 0.5275339356891707, 0.08152516897050699, 0.6154160834168745, 0.8102880358042633, 0.4042982393351212, 0.1385918328051382, 0.7700144124773203, 0.8199630124832614, 0.5030317470419857, 0.6444352681817106, 0.029110666821835163, 0.6795448553846251, 0.4700647623302135, 0.6246214449763401, 0.4387114253556401, 0.7646081560767046, 0.8322425678062103, 0.45492875560828633, 0.11251248565796734, 0.5001936196547839, 0.77654326895366, 0.5981528825169105, 0.4360785806489766, 0.5626239553990945, 0.6569315385808869, 0.4971147496946702, 0.29133461161106733, 0.4407364046074941, 0.6720269351972207, 0.09024130415586945, 0.4730443097965149, 0.1295887887641186, 0.10408027453000201, 0.8486126292015892, 0.6746196250780596, 0.5012182585073619, 0.5052292336598874, 0.4210423262119542, 0.7400279273612861, 0.544287121088304, 0.4064481446439465, 0.34881166160579236, 0.6322437317648528, 0.5381118991645569, 0.3883374502666068, 0.7829345196071602, 0.18026063708714113, 0.43286480853561055, 0.6165468231289676, 0.385081810180269, 0.5972197601492164, 0.47602187728725254, 0.6825788251283793, 0.5158350748418971, 0.4728109785594964, 0.24190231747085056, 0.6719224794679899, 0.8373475834612807, 0.37244620619280033, 0.5367547348001914, 0.510516004888611, 0.458737765924875, 0.39718354554981106, 0.4047146757338098, 0.0760477555656463, 0.4140306544143032, 0.2518875534965338, 0.07970555466498336, 0.20918352627967052, 0.6406111054643946, 0.48541708046346405, 0.4793154615785018, 0.3377722187096703, 0.481115659720304, 0.31062818290255645, 0.15718835489996796, 0.44078463006251917, 0.31919516323241015, 0.9486817307010718, 0.869179330771373, 0.4917485942081632, 0.7641794679580274, 0.6709468154259575, 0.4464534187642676, 0.6775045205775913, 0.7837920901789635, 0.5958554738129148, 0.5576539663489538, 0.8140726866784502, 0.3582169277639207, 0.4678341587284399, 0.5721223323374311, 0.32723131554363827, 0.3690017027871514, 0.48585951505120917, 0.32375281486639124, 0.7089326532124689, 0.3198193477010466, 0.2888770788509988, 0.9141678774084958, 0.3944412782673884, 0.48554205866497185, 0.27473596302761577, 0.618377578641295, 0.5887881394637633, 0.3770589686854452, 0.7813569112506322, 0.20581179036155, 0.7136530382450188, 0.3307479828250206, 0.5440984223341043, 0.17929407074376946, 0.9129448663026329, 0.5380544132419196, 0.6544575851611909, 0.6848598299488249, 0.25954952983904245, 0.4022464983556733, 0.4545830360781759, 0.6523877125787322, 0.38271660563757265, 0.43516968987882565, 0.7842291256761599, 0.2550534762686083, 0.35928790841111236, 0.5596616748707944, 0.05581629193553023, 0.29239559294438633, 0.17530489202359434, 0.5746889537857837, 0.4595325937379012, 0.4510087877956601, 0.5629299609978039, 0.2167198989181618, 0.5673917018478905, 0.9651134197096061, 0.41580207494483445, 0.4381249474777597, 0.41949418678395956, 0.41944282944245603, 0.7865727375412646, 0.37267120924636893, 0.3261203427259518, 0.22360579391521362, 0.49504579693402095, 0.39440121080416585, 0.3791839035803518, 0.46648314363508736, 0.5724633878684743, 0.10795751667214842, 0.6985366336457789, 0.2648071020716022, 0.69910634544837, 0.4423527294930659, 0.9020845494305512, 0.03444276298352922, 0.907893312135149, 0.6932567104684213, 0.41530482810970293, 0.48644681218631725, 0.565517451438883, 0.45421692762793037, 0.4565643661609697, 0.5533187206490974, 0.47404379639940036, 0.5784890488043114, 0.7428309290399506, 0.12851729520019528, 0.7195863815582563, 0.8766966958348756, 0.33641961407687127, 0.5242447022831113, 0.6524820335429702, 0.810342239932261, 0.462694277694243, 0.44697620922133996, 0.30172501584300393, 0.22038938870066277, 0.735620773397814, 0.7415038535959433, 0.5242537330124261, 0.9199170380790674, 0.38320555118372895, 0.2697595993086508, 0.9149847779924006, 0.20753383430284583, 0.3077501837284739, 0.2677546823026681, 0.1585138414700636, 0.3552821797047996, 0.6049934118676122, 0.37765674666509275, 0.43542979912494273, 0.3188866220535299, 0.4974574674409639, 0.4200630940861377, 0.4405102704690654, 0.41861568415352823, 0.5108018472205254, 0.5080488878224187, 0.45350011215307817, 0.3637013962709621, 0.5360251893208434, 0.124084032001604, 0.39869963778903766, 0.9538484986469089, 0.32824799906726615, 0.5640353163972398, 0.4052091997269005, 0.7491003473891507, 0.8546394397016892, 0.8359222962184749, 0.5150574485452202, 0.34696082573808135, 0.3019517747616063, 0.7313364208371238, 0.6375590915123593, 0.9829710875722133, 0.8978173243170158, 0.2916240242818613, 0.861126613156064, 0.4955611971384635, 0.30200102423613806, 0.7524693809941578, 0.10950423832895684, 0.24726360041226403, 0.504646707071022, 0.9181066231175958, 0.49297699375375936, 0.5075984487554951, 0.24044913368627796, 0.5397342165355316, 0.5804643785078981, 0.23884933159927896, 0.5675508079009416, 0.25323991392138323, 0.6106308510134661, 0.5315508695997836, 0.8716675146622918, 0.5170553872181332, 0.6236368887360729, 0.29667363325667306, 0.5176341646933673, 0.7097075696787054, 0.1288232745789052, 0.5149526674716253, 0.689702190924912, 0.3331045570213506, 0.40332138579710797, 0.6104019914283593, 0.3219633467724207, 0.49095023704315716, 0.40131782503722013, 0.09529358264811955, 0.8790712569035242, 0.492505526501707, 0.5883381194352721, 0.7694121239297798, 0.21238686972954507, 0.659656762239258, 0.27857215568180904, 0.4376193628921092, 0.6592116520150247, 0.5959258783803922, 0.510733428771532, 0.15481488481418, 0.2844197169206625, 0.855004432757462, 0.4449819212119328, 0.26562022813732883, 0.3590899655678491, 0.5718628428431926, 0.6097623127469234, 0.3519246589181633, 0.3661811205664117, 0.5823131188622462, 0.36865612284224936, 0.8201066271117249, 0.2843124415174851, 0.7357102546310114, 0.466789518765272, 0.09103325370006526, 0.26371210612860835, 0.8432214455463807, 0.54048207894232, 0.6968249798774832, 0.6901853278149335, 0.9929537487000712, 0.4217781563576689, 0.42326278453035143, 0.34650094831719225, 0.5871964909247215, 0.2934560853885726, 0.19202681271167554, 0.49972179863751504, 0.415347954610602, 0.6927575684176162, 0.531199976439409, 0.007822250965807909, 0.07013228743319755, 0.4076107906088456, 0.5588041003024293, 0.8358480434025031, 0.9242876571565799, 0.5837523357644124, 0.26622642552256853, 0.4263117792552433, 0.6486273444614328, 0.5527971662414519, 0.4658717469464507, 0.6593512693468292, 0.6878673930921155, 0.8274738693503719, 0.6330312865966783, 0.8700286654885576, 0.06514846249911732, 0.6223610347603115, 0.34523412401467923, 0.305440641135443, 0.583038785214625, 0.18764833708972933, 0.8303017177661931, 0.6217163396133816, 0.13043351440830997, 0.042740087427340256, 0.615615572899852, 0.2319778954683267, 0.5422237503396525, 0.7476514781651028, 0.1554410930468012, 0.7064907559624538, 0.1175312550973483, 0.37893621680381506, 0.4234686068272035, 0.9397327174740778, 0.23652215518880532, 0.24196616790008318, 0.6230512191695956, 0.517784757417272, 0.6630748634957276, 0.2888553819295262, 0.21407121359393494, 0.6342065668392608, 0.5267128836440363, 0.4820625321288648, 0.25118130186546805, 0.3754595125324958, 0.3796731973329402, 0.6531173639539652, 0.7652115963662118, 0.5889452597696389, 0.6043047357598322, 0.10628088179406259, 0.80058925728446, 0.5886343240400166, 0.7704325343713372, 0.15601130215985298, 0.7253865914969174, 0.3421209990706546, 0.43094722722961243, 0.8871699934706778, 0.30868102398718567, 0.13669042709499685, 0.07381919906089673, 0.6708181576330088, 0.45802446988843337, 0.3349529628071246, 0.6584697820508246, 0.3234583485861002, 0.5827105673209544, 0.5955291436914145, 0.683769378668454, 0.1031416325479882, 0.7116361891786099, 0.8019354177247751, 0.4830249553342152, 0.5106690491378999, 0.39028466419925245, 0.2521911899086807, 0.43157000911159515, 0.4832743711998978, 0.5288041602189761, 0.5966753999374217, 0.6136391201269261, 0.4109675715614841, 0.28397167687969593, 0.20837730691248935, 0.11971762332620117, 0.4142084531821651, 0.6652902312378997, 0.8244041066314316, 0.13352304238859258, 0.7118706269155008, 0.546103315513029, 0.6517982534984338, 0.7247633885719389, 0.40351313852210774, 0.14122298175603232, 0.6629667218754647, 0.7278085435890599, 0.3525796885784579, 0.6039705297828046, 0.31576532122539314, 0.8362208328925206, 0.8773606491124477, 0.41036834123655075, 0.4796549314599603, 0.36518884743722535, 0.5159741576554472, 0.6699834001132716, 0.7554800766658885, 0.41566436642667454, 0.7059519424834346, 0.06703970078100491, 0.6180964387212933, 0.4584271397114152, 0.5449816469177752, 0.8040858940834319, 0.2603168284702547, 0.6917539793093789, 0.6290671666219225, 0.8021319798690592, 0.3234569164358585, 0.24968384581742242, 0.7477482417185191, 0.9422671253630761, 0.48219825313664433, 0.6686071133015327, 0.28201275508930856, 0.3211050251323877, 0.3371862379805175, 0.31479084588167394, 0.34402762256784314, 0.993606746879022, 0.25716507194711113, 0.3173362764905652, 0.48485114848717303, 0.10711385729728234, 0.9531591324900781, 0.09725354846592787, 0.1775093280893254, 0.49476935933602584, 0.1337942892923057, 0.5275792695014487, 0.17383197234744785, 0.8007621669843132, 0.4673127208226501, 0.36460616815662905, 0.4783336209500513, 0.7761078426483055, 0.1362349164319212, 0.39668235684357817, 0.39251324425378087, 0.7781214702128606, 0.307703706527293, 0.5678130660062432, 0.2555331221826879, 0.41004360867152057, 0.5219497656669518, 0.4948011169274658, 0.7229589769415783, 0.28109624508267056, 0.8930779003455295, 0.19719266096264482, 0.43168820198242824, 0.340365949296095, 0.4566859007872169, 0.5491661080725578, 0.41533599647694996, 0.49949516664581606, 0.48130387951555603, 0.39480963484998577, 0.771247735020615, 0.4284265309987426, 0.16187473987065681, 0.6337816333864232, 0.39731355765678505, 0.38040361893233615, 0.8241969549838306, 0.17220663632507083, 0.19385674214583964, 0.4785250695448416, 0.06479796831881474, 0.3920767009238723, 0.3319491845761974, 0.9021654404160484, 0.41991821870741064, 0.19915901677083608, 0.26892425861658814, 0.5245329610760636, 0.49688062750143447, 0.6202927412515267, 0.3846407069946607, 0.6354027882328428, 0.5142881813207667, 0.5641021120465515, 0.10314840780846268, 0.8263010255852927, 0.197536317114328, 0.4253609048899423, 0.7060470807115853, 0.10400423073093062, 0.18785341784672133, 0.6068291123525101, 0.4413945213248762, 0.4030366442211708, 0.4354726971177084, 0.10453419224867938, 0.08628706756622884, 0.636527509962984, 0.6852396042488692, 0.6848846330397993, 0.3100631500402943, 0.6011534755498938, 0.2675328538336767, 0.8626768801158191, 0.23182152000122708, 0.7708377915545762, 0.4068375918550425, 0.5527902018603854, 0.48637321376938514, 0.6655633272836298, 0.6359329487911191, 0.4460273417257776, 0.9156843336825089, 0.5589218408463064, 0.9812492386781964, 0.4733729582054612, 0.6473273351798525, 0.4563736101193869, 0.43778549383069165, 0.6216443328649458, 0.19539868925496945, 0.4544454597096183, 0.4671515586119055, 0.20441802135708192, 0.5541370935056574, 0.785902469690382, 0.42907390953344526, 0.47868891915102296, 0.4789904355376012, 0.15434927018737954, 0.9230057056512895, 0.6145664261497895, 0.7037902697051961, 0.5035140226445428, 0.444881413148067, 0.3028690684487503, 0.22280292462247359, 0.3041595937225898, 0.10499589246135838, 0.4610877329842324, 0.4602598644032332, 0.6728180514541936, 0.313224598640853, 0.49637048399258377, 0.4686225274235659, 0.670866373747539, 0.11846319037901554, 0.5434275768894045, 0.7341793530110085, 0.7627486000226665, 0.7542795203428521, 0.5002046279387834, 0.2986876585871483, 0.4868075381962206, 0.7796616402442843, 0.2934515607341518, 0.34116873626941124, 0.3609792237606607, 0.5773335539306179, 0.526761660901833, 0.36895319613535094, 0.7486195487620911, 0.688147166138491, 0.4720462197945216, 0.5455395276726855, 0.3089144817820205, 0.343067424249194, 0.2822816168166857, 0.2060533782771511, 0.704378722936515, 0.4853573867019288, 0.43834935086513854, 0.45683896743106633, 0.3781527303025798, 0.5818445345429663, 0.39122690753483025, 0.594234909773469, 0.562255716069271, 0.39281788796160266, 0.1531879899137467, 0.4941847555619331, 0.9020435174527478, 0.9457527226478049, 0.6225820620222744, 0.8045894579469774, 0.5689386478832369, 0.4753121700418397, 0.7573383793691448, 0.9773481166525386, 0.7490156863959826, 0.2969680037559326, 0.6810368445210868, 0.4215918502702382, 0.6818663462476173, 0.3113573367144706, 0.7495895875270764, 0.6883734615158036, 0.2833666396652714, 0.638453965384446, 0.4736490351811706, 0.38625961977065426, 0.05717919386585396, 0.8090174558438793, 0.4822359640634883, 0.9591510601144406, 0.5859908828193777, 0.5801031765463648, 0.7148779301981681, 0.39577667492903273, 0.5982723021684495, 0.7061708674049907, 0.4943890251814599, 0.25557067158906055, 0.65208213531016, 0.7516399580404899, 0.564979222602737, 0.5530844731920305, 0.8446216305342242, 0.25182247721554585, 0.45638159364452097, 0.643611506941542, 0.22695576705512222, 0.6684604828459326, 0.4810468155022207, 0.21196623387954283, 0.8238389012475629, 0.6881216928915802, 0.5898551984248946, 0.8674875817829997, 0.5926573348404537, 0.7444770731421163, 0.03127288778268905, 0.422183663790222, 0.6294916745865564, 0.4873180614137264, 0.8800238581542267, 0.8226352530600753, 0.36590817091545125, 0.0948311162415092, 0.6060563371784762, 0.5932045282495912, 0.36262850399168806, 0.24321612979473428, 0.19941773659366485, 0.3163740932400296, 0.6462867125271758, 0.83274368027707, 0.4595571543863276, 0.533608180622224, 0.70728349782294, 0.0813641185350753, 0.4379957049376579, 0.6162287051366483, 0.15124403514031048, 0.2878501152883857, 0.6627963335483043, 0.8359822125428771, 0.17503741942464957, 0.5821229121401807, 0.5085267382154918, 0.07007258917707188, 0.569526099486598, 0.05675996003065725, 0.16267100157719222, 0.30642834642350847, 0.358500775203782, 0.35218584175866036, 0.34645508774726697, 0.6377435852603826, 0.8315255544769857, 0.35319875573128523, 0.4115550329319855, 0.899118852611827, 0.6135098492080289, 0.7884489153616432, 0.19705804814714922, 0.763578882120564, 0.23269574384470876, 0.3779202972681931, 0.5122923933418744, 0.6045682059563663, 0.5832385059965915, 0.2935292417199211, 0.16348019901299077, 0.6522311154320911, 0.5507048557400946, 0.46059063236250364, 0.46754519180220316, 0.7085667639767439, 0.7872391191939523, 0.4093561259708238, 0.6836421496463834, 0.9170303538255565, 0.21882660165714574, 0.6217044410986757, 0.5084876400279601, 0.7155239799051796, 0.9411450925926008, 0.4585559570243225, 0.4785327534048464, 0.30014200709856576, 0.415457339362298, 0.6106165557308877, 0.7688938324900827, 0.2556686367412503, 0.8690786839868043, 0.9725088567778246, 0.2913670215894323, 0.10394396739531958, 0.391113862809102, 0.19816317925569743, 0.23545196469065283, 0.6439955733301008, 0.6676833406987929, 0.033726059062192004, 0.2943441907886163, 0.6310353103044637, 0.6619108469013619, 0.6601011158077985, 0.8232235464459501, 0.45698472715794175, 0.38096403458313566, 0.4613186907621656, 0.49598426536006623, 0.7932255921297048, 0.34776169054503436, 0.9118971771663845, 0.10655032866608, 0.5345191276587862, 0.43435580746803826, 0.05273100482510673, 0.3204034210578241, 0.6321852093337601]
# List =     [0.7518268251975111, 0.4006519924851435, 0.4621305753430892, 0.4245377123593248, 0.8461486908432568, 0.5239129435735469, 0.33827753867125376, 0.854216033847546, 0.40653449011103276, 0.3657755645277949, 0.8900102671863347, 0.563671638772298, 0.6252948877306461, 0.37769249263216603, 0.44194820812579794, 0.6852350477749214, 0.4250737173929675, 0.6851012449148037, 0.6922360365205487, 0.6123635619000225, 0.6124973466445874, 0.3165233964439793, 0.2213411394158526, 0.3487459194715131, 0.7367714058875888, 0.7073001502794438, 0.8184965546801362, 0.8152361416161171, 0.3157388331692108, 0.6948205452652925, 0.33670730893766854, 0.8991223752411078, 0.43123124200520635, 0.5861590952864177, 0.36126146626484507, 0.703811289901412, 0.7386980560252632, 0.33186565562810555, 0.6399709764089347, 0.4585801724487225, 0.6931520743627847, 0.2382854648218104, 0.5322639269033326, 0.5546799170336925, 0.5506130536007978, 0.3387319030295152, 0.37914969794073367, 0.7581365551583951, 0.8539382599756801, 0.5077272309606596, 0.6162802594960481, 0.4921515767644038, 0.3174328646711937, 0.31830407854242637, 0.6283096388967642, 0.29433243644272555, 0.4912966208169524, 0.4635132565512085, 0.38028694101792704, 0.519937699284649, 0.4626221461751634, 0.8517454849261858, 0.35695729143921406, 0.25482732694565985, 0.24863831891672344, 0.7215384410810506, 0.5062736500810989, 0.20586622177701697, 0.2795866740506445, 0.5486397083923156, 0.3492385959889147, 0.3258101565616505, 0.4842845581082017, 0.1734474907285869, 0.5978768908013772, 0.3694993654812451, 0.56322866050841, 0.5346546003408402, 0.3174022439413497, 0.6567535259195962, 0.524985110940771, 0.5519761081137317, 0.43110911327341234, 0.5147073661578954, 0.6058885967639488, 0.4188835361421398, 0.5236930761471016, 0.09956138588138336, 0.6285424029594989, 0.3371958885431645, 0.5892104531298146, 0.5707990570474617, 0.6716092228449504, 0.3053019763328242, 0.5868503486753327, 0.666719148034451, 0.6987876342852246, 0.6233536646172667, 0.6253722853913116, 0.8856629359965713, 0.45766858895355966, 0.5152279522315498, 0.6323840783986193, 0.7637974582155923, 0.24930713738457294, 0.19118358920388956, 0.3553996416440569, 0.3511426920170072, 0.5661955763681575, 0.7529839169163467, 0.27090646161909837, 0.5684588980274681, 0.5171036822285893, 0.4645714333374977, 0.40928919132797054, 0.5230615249899077, 0.47481736970910693, 0.6257307943887176, 0.566564982066003, 0.5033761541999557, 0.42700388830981917, 0.4596280168026554, 0.5560001661354458, 0.6489198506736542, 0.48470764046267384, 0.39413959045594193, 0.32076067409852094, 0.6137797426621071, 0.3947319842110812, 0.21990190048968233, 0.5468242929436709, 0.5125553485379265, 0.50257956647904, 0.5180999844220744, 0.37707846345347473, 0.6110083500686713, 0.5323404201908616, 0.6140251631665218, 0.5602504432706694, 0.509536981281305, 0.3043217485722195, 0.5078395975302431, 0.7891388872916212, 0.5398783748562997, 0.6565526797297255, 0.7910042816568261, 0.6456732617521052, 0.20366733450506852, 0.432320744712283, 0.4099042022425173, 0.7222504891569101, 0.4834103832155853, 0.5458384719455184, 0.6453407577024761, 0.5917093299251804, 0.4558916206941337, 0.2831629927385758, 0.3222574909319552, 0.4231383835776206, 0.9454355400092467, 0.5805795280973036, 0.4557779479359578, 0.32574735181460046, 0.5227695252122059, 0.4947800171086265, 0.6446851258910451, 0.6354254530221225, 0.5918363211388484, 0.8416941469966949, 0.17527752737064534, 0.6237837851347553, 0.45970980407420053, 0.3290724790909282, 0.29363692609140435, 0.47500592238626826, 0.40902790087440244, 0.33795542648624577, 0.5730108191367845, 0.47567255866882563, 0.8776277255985262, 0.25991781488214677, 0.3839861761109596, 0.6685269055693326, 0.6150553922470077, 0.7637447809094885, 0.4989224939279843, 0.6309183888324164, 0.36438630843878267, 0.39149271005633896, 0.6671854402941014, 0.6314584247133351, 0.5617748452415259, 0.5588470558935431, 0.4446857584389991, 0.29292009587219253, 0.586089487523136, 0.44886420648930947, 0.5105311784589317, 0.64063122694193, 0.43136763885264917, 0.9079303006839269, 0.49496318504811776, 0.4260527739878095, 0.6844036942349725, 0.6235408070162811, 0.4179950074955687, 0.926974552727388, 0.3794595744624375, 0.5913974564555963, 0.24415491998007915, 0.5303694439990728, 0.3444758041416765, 0.2750234166304937, 0.2629987946090758, 0.7457655912246387, 0.3500722689311391, 0.6749890723243307, 0.5415436936446538, 0.5526630562629344, 0.26415705713343995, 0.22454404465811462, 0.4934233727485557, 0.5787281195557809, 0.18642212768271182, 0.5401100914767898, 0.5165657363245479, 0.6763981978565501, 0.5973778996361081, 0.4366898372426118, 0.6351092725701493, 0.4778891460056287, 0.46659636679822175, 0.6734855571277505, 0.45462202495380244, 0.5993719928786292, 0.6126770355816472, 0.466329148514599, 0.6118247605477608, 0.4392756438635609, 0.6041683138660482, 0.3420813329915793, 0.30524972935896333, 0.5602748151105773, 0.39386864890256185, 0.35447562472091654, 0.7599237240150083, 0.5822147153480268, 0.5473711231607052, 0.5291969935476217, 0.45568972148434195, 0.8249610382887949, 0.7071751880810361, 0.5626942210289282, 0.3585452688478664, 0.7736710097565966, 0.25524795877907486, 0.3370439905436771, 0.6225147423330311, 0.5290202890117471, 0.9646843541359256, 0.45462004024697983, 0.5116938304044137, 0.4622498959220145, 0.5564917428742686, 0.5171756215169065, 0.49465088539808627, 0.5175315863878436, 0.44454191751912736, 0.3536291342133545, 0.3645559998694826, 0.46318500867586565, 0.8037234483998572, 0.6643885511865413, 0.27956282264080745, 0.41842195748046207, 0.6480932535701236, 0.2727358196806921, 0.5784212103626212, 0.37590375270139326, 0.5795431663588326, 0.3520657643019919, 0.37830765331665933, 0.3833619998970706, 0.3985751814038257, 0.49978091835440874, 0.5217703198301384, 0.588112584859984, 0.7053061428412246, 0.6290316250797033, 0.28538686296526733, 0.7091195678918646, 0.8207739213483465, 0.6501792901677418, 0.6009042729994475, 0.5330750628140027, 0.49716938320501974, 0.5339155682846669, 0.44339867309086106, 0.5262045803470573, 0.7532611866430445, 0.7300104845347734, 0.4560097366855317, 0.6832292182130335, 0.4856258804828578, 0.4782053973727881, 0.6714038720662789, 0.35959760442502187, 0.7362342906083521, 0.635134983013636, 0.5941353737257274, 0.24627556089259944, 0.781221684827101, 0.3773291644305188, 0.4108520224569478, 0.5179090704466525, 0.15753393269662636, 0.5877668706630398, 0.4883448926998328, 0.370596303866354, 0.7317782061595133, 0.5150499084645681, 0.722092514626661, 0.9885368105402557, 0.29625504459655394, 0.30584055050516645, 0.23603812200552027, 0.39074788544272404, 0.5049657346590808, 0.2127470656825659, 0.6841888218261055, 0.6086943249477919, 0.34010835434810804, 0.6427223242714737, 0.67440893106206, 0.4367587804765416, 0.5291572125567349, 0.47958877179441833, 0.7008471223959816, 0.2702911807560841, 0.41238139994537726, 0.48719115755280046, 0.6747069434231483, 0.6681459147355906, 0.13248561581870177, 0.7124591678652537, 0.535197077100315, 0.4734346209233812, 0.7300689550441339, 0.7951158726968177, 0.5329529718555233, 0.5994573930889325, 0.6528733045485563, 0.513004975571937, 0.3399122240010811, 0.35320162465964317, 0.7105327147423961, 0.6269524759645346, 0.5251244061400486, 0.13636258073699753, 0.34074872568711634, 0.5146623824604978, 0.4669253913880599, 0.35993679950514645, 0.64287061007851, 0.6164527091287859, 0.45352859855800454, 0.19359443200060972, 0.5758270291024463, 0.38458927075248384, 0.5351777832996285, 0.35979437388185076, 0.21874193364961597, 0.9787531970561072, 0.44067872413935466, 0.49750269542017644, 0.44378748787788636, 0.7843045167669674, 0.3985561184319897, 0.353155846252469, 0.2730683529372425, 0.4805470884400596, 0.6241787914511763, 0.4856567818814991, 0.6474825092737985, 0.5368774175238226, 0.20987001739227235, 0.5580995237493243, 0.48811634108844176, 0.7704212060459992, 0.24010690676496105, 0.7007135452112228, 0.41471501073666633, 0.4266824241444492, 0.49069728873472274, 0.5800028482159788, 0.3371362336165714, 0.1983205288286326, 0.4648255233706281, 0.1293757550916269, 0.596825702233986, 0.5790394035779922, 0.5365459016730256, 0.6412521477763796, 0.4394788335264399, 0.7866633823006213, 0.46602390724038145, 0.22379800925606153, 0.3355668090566512, 0.28205667932947903, 0.40030455330448833, 0.5500936403422519, 0.4879128809320507, 0.41990179460408594, 0.7789993591804979, 0.278598448233167, 0.21591191569589901, 0.40125087131985177, 0.6218293520096372, 0.696957076058468, 0.34558054066150873, 0.23749226962796416, 0.5487412318317173, 0.32645362963151386, 0.27121329524508087, 0.10012191219945898, 0.9317889478969493, 0.3721760970385042, 0.7592449468506572, 0.5665029942691738, 0.09327063070982122, 0.6798044135378218, 0.6871569754082782, 0.5951381605342331, 0.5723207138649274, 0.6512265250204426, 0.761505693811839, 0.31082638763569226, 0.30874260441896195, 0.5439348786161998, 0.04950773407044495, 0.28357227895308845, 0.8034471465565929, 0.5376200875696158, 0.6020541767011668, 0.7772083887491248, 0.48846820847236244, 0.4565143857572788, 0.5284081517470678, 0.4570002541446994, 0.801186546644802, 0.5130254075519731, 0.16463652492329012, 0.29875178840503935, 0.6082581914250493, 0.45778517344229114, 0.38458366013986806, 0.8285989456021825, 0.4157096545209012, 0.8517035911348318, 0.28322454429287736, 0.5089743866623596, 0.5316157394848287, 0.5995358718820394, 0.44012142077734673, 0.7184869172927999, 0.6962244834361717, 0.46322408772123425, 0.4164766755096305, 0.3419501953159861, 0.48328756929801525, 0.7236497352836962, 0.9380980169483679, 0.6047149624214667, 0.39575133562647774, 0.25624476296173704, 0.6526829719616208, 0.5456649267355815, 0.5526962325773009, 0.48570876574277355, 0.8219773453066995, 0.5347692879805194, 0.8595256552228401, 0.5749714832482694, 0.40569582537721305, 0.574101358639873, 0.42381842209344756, 0.5969967655178748, 0.6249800626581062, 0.23393104898822603, 0.37702323868617876, 0.5075955195449139, 0.25730732243640436, 0.5453292608531525, 0.33365829586406914, 0.31209334129540645, 0.3821513917324475, 0.730527056036438, 0.5902846166637397, 0.2975066762381756, 0.7235697140696025, 0.412769715968762, 0.41179256106465956, 0.537212806343247, 0.7737057468320876, 0.18897844293170324, 0.676574167119325, 0.700278506310791, 0.49727619146180074, 0.5396600599330399, 0.4960067350542189, 0.6612715083583198, 0.6480530655530155, 0.41985078115790087, 0.5559936379559113, 0.939322116693869, 0.7067975234534931, 0.7334484324253749, 0.09552674631859548, 0.7525071895797516, 0.45589818678913047, 0.2853044264693625, 0.3580812682949972, 0.37731233547643106, 0.5358399256919005, 0.12940269329717946, 0.43361293300627857, 0.7473649015924816, 0.5205612283010943, 0.5374055576378199, 0.31154491944326884, 0.5055139271333183, 0.8282786423796893, 0.5178910701345221, 0.7379796471170731, 0.4524608316157652, 0.595023891138443, 0.6741034377737466, 0.3871589774006009, 0.7442932316294014, 0.5964053390528171, 0.5353028753143395, 0.8829268624342327, 0.42961132107810673, 0.3261952861154506, 0.34598970604682333, 0.5741725233043522, 0.7366176849396237, 0.6737105330580163, 0.7596935088941774, 0.08310505371043725, 0.48648952280562635, 0.5264071849613572, 0.35622223399542075, 0.43641860720605075, 0.7743997185483535, 0.22162239035503234, 0.7045086272374085, 0.8112149288652732, 0.547409984999148, 0.49180057745403083, 0.8912561833344497, 0.5506594977275946, 0.2758618478189585, 0.29338225744541735, 0.6599656088044623, 0.9721228879424182, 0.7691276160062857, 0.4562447460581192, 0.2433202847155761, 0.7271910240075593, 0.8053532236560705, 0.3109676228529313, 0.6683188236374688, 0.6376462321995414, 0.4799871646436078, 0.7717198513979204, 0.3962353412813169, 0.3612054938564056, 0.33574696029631146, 0.5253178089894605, 0.7498098138344249, 0.34809048484859517, 0.44228129178603603, 0.5773847392515599, 0.6152360353065284, 0.4450539862128649, 0.5589275494199418, 0.43748547742142385, 0.14547911119068568, 0.22953220709913025, 0.6117500620982349, 0.6863962100252433, 0.5707703125719203, 0.3984896456321164, 0.4432294746623181, 0.44632912793761664, 0.2856786473370164, 0.49673727052379535, 0.45460046158685086, 0.8839066177168511, 0.5866076352826776, 0.6748630212817979, 0.6730127362990199, 0.29121033398593976, 0.18086208538248488, 0.616969117641336, 0.33739820423646794, 0.7001524120340885, 0.7239288695065098, 0.5397852051796245, 0.41562846738418563, 0.43192500236551995, 0.7360016473940468, 0.38277852145401414, 0.5671193059099411, 0.4529268765260961, 0.18585256453858634, 0.3960387584803041, 0.21495161942970048, 0.7617242844832401, 0.5925861173435301, 0.5884111648113064, 0.12429353121027814, 0.5550066944170944, 0.578346696753514, 0.3883390429586614, 0.3200731904205417, 0.330396787817141, 0.44145805927752785, 0.496554416239722, 0.7155666587085865, 0.4284985954430174, 0.437575674363076, 0.23068747023823827, 0.7095037578693637, 0.31335212637141785, 0.41818907174153536, 0.6904869589504048, 0.4227079329394476, 0.6602211081349183, 0.6303971642082198, 0.5367673353470616, 0.6923349687940848, 0.5372206727532618, 0.5437664444311912, 0.5599274449877113, 0.4273916425612774, 0.6004058574925797, 0.5309129772510066, 0.531072988377237, 0.725142793849298, 0.11889967024284653, 0.7436647449664436, 0.41623542054188983, 0.3276110907483457, 0.8410513757588638, 0.45046724447301223, 0.46309594886287947, 0.26484428990184816, 0.4476331178835784, 0.737987074378548, 0.7157705345045009, 0.5779310312701016, 0.5397699613579003, 0.37297107842977684, 0.5793854969839884, 0.5811690140986768, 0.5637072363213576, 0.23456880361408866, 0.4897026947356076, 0.7149296214168228, 0.16661747712009534, 0.30103092280764254, 0.11602312490589622, 0.5241903169659767, 0.5878045898633284, 0.7181869126663049, 0.3797771826284773, 0.4419446192878605, 0.686178016399938, 0.4021249790942624, 0.6686227764828736, 0.5726743522177743, 0.2875858478171595, 0.7244024723740318, 0.6266367987288868, 0.7054407922047112, 0.4837591298251068, 0.6069668297333098, 0.5887967997138247, 0.5019929411552969, 0.5474859050266758, 0.406670231941818, 0.4475909186052282, 0.35208521031543005, 0.508102626580923, 0.8460521792353067, 0.5353077822807907, 0.5435872670558589, 0.7543233579008204, 0.46239824249571776, 0.6762626328304027, 0.5464915954554788, 0.1729155320203566, 0.4357532704717354, 0.4273691597105753, 0.7250809649359395, 0.36799738563357276, 0.4772318799358212, 0.5625275000147385, 0.707844881755001, 0.7480951788420689, 0.49244366492960534, 0.7393380572702668, 0.5777358718381908, 0.4153201169564637, 0.3502486432755571, 0.5027019364716188, 0.7523192776794414, 0.4406533292692428, 0.7038014716639877, 0.5869947654834012, 0.6410771237056794, 0.6922729104136558, 0.3745083890470055, 0.3994256139741285, 0.37772024029999396, 0.49163435928839444, 0.5258069870683768, 0.0965845456108011, 0.7726432095652146, 0.5335393006961642, 0.4565055135767427, 0.5711875787094207, 0.616198626051812, 0.5816191147723883, 0.5447405405781306, 0.12806418254433777, 0.5519160405369558, 0.6713593784746342, 0.3516053429246043, 0.5606207096630659, 0.3502270361234997, 0.6515148351450993, 0.524002945995373, 0.48736414914141846, 0.21073507212902037, 0.3745782449820304, 0.572452369586344, 0.7247655076217919, 0.16454551765235625, 0.3427069350704373, 0.13520838263893986, 0.23211816923480522, 0.24097448906158847, 0.3025896439173225, 0.4759755423452171, 0.61233546571697, 0.5515657090104672, 0.49932740652131125, 0.47751046927405266, 0.732591201399831, 0.8490800222687798, 0.4810373922524631, 0.6068276952831899, 0.7595616555607323, 0.49783877805584364, 0.28047203699902945, 0.6980387036800647, 0.6709449177033309, 0.710089644435218, 0.38670739749368205, 0.4149578848393598, 0.7795832828476198, 0.4059471590368217, 0.30193526210436794, 0.29795024088373295, 0.7076883360736115, 0.5745747884969101, 0.5702064520938266, 0.6879425908309165, 0.6390979803077997, 0.5649035661524642, 0.1664669937325296, 0.2599572328787325, 0.22322223945864017, 0.2550391287707467, 0.878647958759623, 0.6366532230669556, 0.7070949826181705, 0.7314292299238583, 0.654294167806164, 0.8072179868412047, 0.7067379323709533, 0.23838312707424225, 0.3075754397798123, 0.4837698953029739, 0.5096277069857196, 0.4954547048883361, 0.2770519081588333, 0.5164591605202145, 0.2272622383090706, 0.2759076587336712, 0.6909374738354049, 0.5366554195127466, 0.5062838222880864, 0.5573732529480291, 0.33195665027541416, 0.4752098114031314, 0.27159371000940913, 0.5191743787289139, 0.47847323589994434, 0.2934405652213265, 0.6647929893920345, 0.45104688841134233, 0.35451701000488833, 0.5212153304623979, 0.5510016464801002, 0.42670428338300315, 0.6042534325763085, 0.4400125183731524, 0.6115171900815185, 0.6844344327818008, 0.8215135601181831, 0.3775910228748861, 0.5878546965631692, 0.6413359163584005, 0.6971292794741155, 0.4606274419831146, 0.803974930046189, 0.3283913595508042, 0.4244880826946621, 0.5767880093703446, 0.7500290254893575, 0.4086567399392489, 0.6360589079896564, 0.3709577961923526, 0.36151075345917355, 0.6179453616057855, 0.5583300420203309, 0.6259143259083462, 0.5448361881545495, 0.7309506710918926, 0.5096460949972856, 0.5520266884617087, 0.37573242316346567, 0.4746626588537643, 0.650157600365247, 0.5534724863367143, 0.3953185629911047, 0.19101002918878623, 0.4839769250452771, 0.6257507040321355, 0.6169718288316968, 0.26807813105175693, 0.3389438086101443, 0.3155629726060469, 0.8067558628772231, 0.7027597367681698, 0.5437417185220524, 0.34903577560532406, 0.5472097191669847, 0.5450605087606789, 0.5969738155267584, 0.2140919047176688, 0.40478775714358595, 0.5229294884557779, 0.6022296296960948, 0.6075539290958134, 0.6693355494855241, 0.8217601154175302, 0.592376903232055, 0.8053725275083174, 0.6247093477733727, 0.44019578866212694, 0.523167855389557, 0.5201792053501522, 0.8298578473309994, 0.363884249932825, 0.3705333253564371, 0.5887702367969954, 0.1725743496336743, 0.6566912764352406, 0.42895177711934845, 0.47301081324565925, 0.3471718964729614, 0.40710331775025743, 0.3783249904140483, 0.6353182493107705, 0.423281935056654, 0.5750835899770222, 0.4248477663358899, 0.4639009095148876, 0.5801468815017481, 0.562144798399253, 0.4988249299387746, 0.612608374390124, 0.5976575945516422, 0.47530795935943754, 0.5470136702226014, 0.40328619732180576, 0.7375331362308739, 0.5573813331581481, 0.3169252037794551, 0.49434982482396633, 0.7565690433716302, 0.34797928207773776, 0.4720286758906787, 0.3459581008925066, 0.70384340566371, 0.5530523996415767, 0.7684557899985386, 0.7144563644366011, 0.5398377423810468, 0.4050759524721913, 0.38168937413643755, 0.6456897214351496, 0.44726243379590336, 0.41145579981829405, 0.51010348880925, 0.5770411393651291, 0.42173843834200864, 0.3968023416311327, 0.5939586691656004, 0.5617589686124014, 0.6421914567573841, 0.7233595959965953, 0.4559578923667179, 0.6170339965175132, 0.44135251661427416, 0.6155337800265358, 0.4109957946711743, 0.4022831988474425, 0.43436607388627463, 0.3188184446380766, 0.5072301463273975, 0.5524883519629002, 0.5062242886628572, 0.4300528914501458, 0.5155539962609587, 0.5330501702341727, 0.6920728268712917, 0.5417364291606177, 0.3361147630023096, 0.4272436260098639, 0.39752305833420554, 0.43940415774240227, 0.661690231805979, 0.23707838427149558, 0.8314110575257923, 0.6506934953763046, 0.49417791710928616, 0.5464518178070431, 0.5197769321971856, 0.5765561266857259, 0.41690599739069356, 0.34182061102971806, 0.5949323577073105, 0.47092193899059526, 0.007875956804902995, 0.47991716328124623, 0.4473242060686456, 0.38196105953109916, 0.35396593712576796, 0.7740801265781618, 0.24698185405644124, 0.5223684832681841, 0.2730462739455106, 0.5909411649362055, 0.474856913259972, 0.7772768062887376, 0.5529488346415309, 0.20217311759948364, 0.5423514246831219, 0.4846619748518348, 0.6743520410333166, 0.36221703486266743, 0.5605919531091504, 0.1284817633235465, 0.5574721504622644, 0.27121056522265574, 0.6297120529009369, 0.4918948818915881, 0.5208275562701365, 0.8358111984074492, 0.733266116605213, 0.529744784258619, 0.46925258872153625, 0.18628603807213917, 0.39157606896879116, 0.37006824788461, 0.5426170340162411, 0.40501524573182096, 0.33492919017153805, 0.7724935748822431, 0.27764936997474626, 0.5573480132677943, 0.8361975721595771, 0.3319315117972952, 0.707616208583626]
# print(numpy.std(List))
# print(Executor().execute_TestCase("uniformitytest("+str(List)+",3)", "Subject_Programs.Pardo.Original_Program.Pardo", 5, "Pardo().computeStatistic"))








